@@ -23,6 +23,7 @@ import org.gradle.api.logging.LoggingManager;
 import org.gradle.api.plugins.Convention;
 import org.gradle.api.plugins.ExtensionAware;
 import org.gradle.api.specs.Spec;
+import org.gradle.api.tasks.Internal;
 import org.gradle.api.tasks.TaskDependency;
 import org.gradle.api.tasks.TaskInputs;
 import org.gradle.api.tasks.TaskOutputs;
@@ -43,7 +44,6 @@ import java.util.Set;
  * <pre>
  * task myTask
  * task myTask { configure closure }
- * task myType &lt;&lt; { task action }
  * task myTask(type: SomeType)
  * task myTask(type: SomeType) { configure closure }
  * </pre>
@@ -56,12 +56,12 @@ import java.util.Set;
  * <h3>Task Actions</h3>
  *
  * <p>A <code>Task</code> is made up of a sequence of {@link Action} objects. When the task is executed, each of the
- * actions is executed in turn, by calling {@link Action#execute}.  You can add actions to a task by calling {@link
+ * actions is executed in turn, by calling {@link Action#execute}. You can add actions to a task by calling {@link
  * #doFirst(Action)} or {@link #doLast(Action)}.</p>
  *
  * <p>Groovy closures can also be used to provide a task action. When the action is executed, the closure is called with
  * the task as parameter.  You can add action closures to a task by calling {@link #doFirst(groovy.lang.Closure)} or
- * {@link #doLast(groovy.lang.Closure)}  or using the left-shift {@code <<} operator.</p>
+ * {@link #doLast(groovy.lang.Closure)}.</p>
  *
  * <p>There are 2 special exceptions which a task action can throw to abort execution and continue without failing the
  * build. A task action can abort execution of the action and continue to the next action of the task by throwing a
@@ -140,31 +140,32 @@ import java.util.Set;
  * See {@link org.gradle.api.tasks.ParallelizableTask} for more details on writing parallelizable tasks.
  */
 public interface Task extends Comparable<Task>, ExtensionAware {
-    public static final String TASK_NAME = "name";
+    String TASK_NAME = "name";
 
-    public static final String TASK_DESCRIPTION = "description";
+    String TASK_DESCRIPTION = "description";
 
-    public static final String TASK_GROUP = "group";
+    String TASK_GROUP = "group";
 
-    public static final String TASK_TYPE = "type";
+    String TASK_TYPE = "type";
 
-    public static final String TASK_DEPENDS_ON = "dependsOn";
+    String TASK_DEPENDS_ON = "dependsOn";
 
-    public static final String TASK_OVERWRITE = "overwrite";
+    String TASK_OVERWRITE = "overwrite";
 
-    public static final String TASK_ACTION = "action";
+    String TASK_ACTION = "action";
 
     /**
      * <p>Returns the name of this task. The name uniquely identifies the task within its {@link Project}.</p>
      *
      * @return The name of the task. Never returns null.
      */
+    @Internal
     String getName();
 
     /**
      * A {@link org.gradle.api.Namer} namer for tasks that returns {@link #getName()}.
      */
-    static class Namer implements org.gradle.api.Namer<Task> {
+    class Namer implements org.gradle.api.Namer<Task> {
         public String determineName(Task c) {
             return c.getName();
         }
@@ -175,6 +176,7 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      *
      * @return The project this task belongs to. Never returns null.
      */
+    @Internal
     Project getProject();
 
     /**
@@ -183,6 +185,7 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      *
      * @return The task actions in the order they are executed. Returns an empty list if this task has no actions.
      */
+    @Internal
     List<Action<? super Task>> getActions();
 
     /**
@@ -197,6 +200,7 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      *
      * @return The dependencies of this task. Never returns null.
      */
+    @Internal
     TaskDependency getTaskDependencies();
 
     /**
@@ -204,6 +208,7 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      *
      * @return The dependencies of this task. Returns an empty set if this task has no dependencies.
      */
+    @Internal
     Set<Object> getDependsOn();
 
     /**
@@ -303,6 +308,7 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      *
      * @return The execution state of this task. Never returns null.
      */
+    @Internal
     TaskState getState();
 
     /**
@@ -320,6 +326,7 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      *
      * @return true if this task did any work
      */
+    @Internal
     boolean getDidWork();
 
     /**
@@ -328,6 +335,7 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      *
      * @return the path of the task, which is equal to the path of the project plus the name of the task.
      */
+    @Internal
     String getPath();
 
     /**
@@ -370,7 +378,10 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      *
      * @param action The action closure to execute.
      * @return This task.
+     *
+     * @deprecated Use {@link #doLast(Closure action)}
      */
+    @Deprecated
     Task leftShift(Closure action);
 
     /**
@@ -385,6 +396,7 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      *
      * @see #setEnabled(boolean)
      */
+    @Internal
     boolean getEnabled();
 
     /**
@@ -410,6 +422,7 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      *
      * @return The <code>AntBuilder</code>
      */
+    @Internal
     AntBuilder getAnt();
 
     /**
@@ -417,6 +430,7 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      *
      * @return The logger. Never returns null.
      */
+    @Internal
     Logger getLogger();
 
     /**
@@ -426,6 +440,7 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      *
      * @return the LoggingManager. Never returns null.
      */
+    @Internal
     LoggingManager getLogging();
 
     /**
@@ -487,6 +502,7 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      *
      * @return The convention object. Never returns null.
      */
+    @Internal
     Convention getConvention();
 
     /**
@@ -494,6 +510,7 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      *
      * @return the description. May return null.
      */
+    @Internal
     String getDescription();
 
     /**
@@ -510,6 +527,7 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      *
      * @return The task group for this task. Might be null.
      */
+    @Internal
     String getGroup();
 
     /**
@@ -532,6 +550,7 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      *
      * @return The inputs. Never returns null.
      */
+    @Internal
     TaskInputs getInputs();
 
     /**
@@ -539,6 +558,7 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      *
      * @return The outputs. Never returns null.
      */
+    @Internal
     TaskOutputs getOutputs();
 
     /**
@@ -548,6 +568,7 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      *
      * @return The directory. Never returns null. The directory will already exist.
      */
+    @Internal
     File getTemporaryDir();
 
     /**
@@ -598,6 +619,7 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      * @return The tasks that this task must run after. Returns an empty set if this task has no tasks it must run after.
      */
     @Incubating
+    @Internal
     TaskDependency getMustRunAfter();
 
     /**
@@ -642,6 +664,7 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      * @return The tasks that finalize this task. Returns an empty set if there are no finalising tasks for this task.
      */
     @Incubating
+    @Internal
     TaskDependency getFinalizedBy();
 
     /**
@@ -692,6 +715,6 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      * @return The tasks that this task should run after. Returns an empty set if this task has no tasks it must run after.
      */
     @Incubating
+    @Internal
     TaskDependency getShouldRunAfter();
 }
-
