@@ -20,6 +20,7 @@ import org.gradle.api.Action;
 import org.gradle.api.Incubating;
 import org.gradle.api.artifacts.Dependency;
 import org.gradle.api.artifacts.query.ArtifactResolutionQuery;
+import org.gradle.api.artifacts.transform.VariantTransform;
 import org.gradle.api.attributes.AttributesSchema;
 
 import java.util.Map;
@@ -320,7 +321,7 @@ public interface DependencyHandler {
     Dependency gradleApi();
 
     /**
-     * Creates a dependency on the <a href="https://docs.gradle.org/current/userguide/test_kit.html">Gradle test-kit</a> API.
+     * Creates a dependency on the <a href="https://docs.gradle.org/current/userguide/test_kit.html" target="_top">Gradle test-kit</a> API.
      *
      * @return The dependency.
      * @since 2.6
@@ -404,4 +405,12 @@ public interface DependencyHandler {
     @Incubating
     AttributesSchema getAttributesSchema();
 
+    /**
+     * Register an artifact transformation.
+     *
+     * @see org.gradle.api.artifacts.transform.ArtifactTransform
+     * @since 3.5
+     */
+    @Incubating
+    void registerTransform(Action<? super VariantTransform> registrationAction);
 }
