@@ -23,6 +23,7 @@ import org.gradle.api.internal.tasks.TaskExecuter;
 import org.gradle.api.internal.tasks.TaskStateInternal;
 import org.gradle.api.internal.tasks.execution.TaskValidator;
 import org.gradle.api.specs.Spec;
+import org.gradle.api.tasks.Internal;
 import org.gradle.internal.Factory;
 import org.gradle.logging.StandardOutputCapture;
 import org.gradle.util.Configurable;
@@ -36,16 +37,21 @@ public interface TaskInternal extends Task, Configurable<Task> {
 
     // Can we just override Task.getActions()?
     // Would need to change return type on Task API to: List<? super Action<? super Task>> : not certain this is back-compatible
+    @Internal
     List<ContextAwareTaskAction> getTaskActions();
 
+    @Internal
     Set<ClassLoader> getActionClassLoaders();
 
+    @Internal
     Spec<? super TaskInternal> getOnlyIf();
 
     void execute();
 
+    @Internal
     StandardOutputCapture getStandardOutputCapture();
 
+    @Internal
     TaskExecuter getExecuter();
 
     void setExecuter(TaskExecuter executer);
@@ -56,12 +62,15 @@ public interface TaskInternal extends Task, Configurable<Task> {
     @Override
     TaskOutputsInternal getOutputs();
 
+    @Internal
     List<TaskValidator> getValidators();
 
     void addValidator(TaskValidator validator);
 
+    @Override
     TaskStateInternal getState();
 
+    @Internal
     boolean getImpliesSubProjects();
 
     void setImpliesSubProjects(boolean impliesSubProjects);
@@ -71,13 +80,16 @@ public interface TaskInternal extends Task, Configurable<Task> {
      * <p>
      * The getTemporaryDir() method creates the directory which can be problematic. Use this to delay that creation.
      */
+    @Internal
     Factory<File> getTemporaryDirFactory();
 
     void prependParallelSafeAction(Action<? super Task> action);
 
     void appendParallelSafeAction(Action<? super Task> action);
 
+    @Internal
     boolean isHasCustomActions();
 
+    @Internal
     Path getIdentityPath();
 }
