@@ -17,6 +17,7 @@ package org.gradle.api.internal.changedetection;
 
 import org.gradle.api.internal.TaskExecutionHistory;
 import org.gradle.api.tasks.incremental.IncrementalTaskInputs;
+import org.gradle.caching.internal.tasks.TaskOutputCachingBuildCacheKey;
 
 import java.util.Collection;
 
@@ -33,6 +34,16 @@ public interface TaskArtifactState {
 
     IncrementalTaskInputs getInputChanges();
 
+    /**
+     * Returns whether it is okay to use results loaded from cache instead of executing the task.
+     */
+    boolean isAllowedToUseCachedResults();
+
+    /**
+     * Returns the calculated cache key for the task's current state.
+     */
+    TaskOutputCachingBuildCacheKey calculateCacheKey();
+
     /**
      * Called before the task is to be executed. Note that {@link #isUpToDate(java.util.Collection)} may not necessarily have been called.
      */
@@ -46,7 +57,7 @@ public interface TaskArtifactState {
     /**
      * Called when this state is finished with.
      */
-    void finished(boolean wasUpToDate);
+    void finished();
 
     /**
      * Returns the history for this task.
