@@ -17,17 +17,22 @@
 package org.gradle.api.internal.changedetection.state;
 
 import org.gradle.api.internal.changedetection.rules.TaskStateChange;
-import org.gradle.api.internal.tasks.cache.TaskCacheKeyBuilder;
+import org.gradle.caching.internal.BuildCacheHasher;
 
 import java.io.File;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.Iterator;
 import java.util.Map;
 
+import static org.gradle.api.internal.changedetection.state.TaskFilePropertyCompareStrategy.UNORDERED;
+
 /**
  * An immutable snapshot of the contents and meta-data of a collection of files or directories.
  */
 public interface FileCollectionSnapshot {
+    FileCollectionSnapshot EMPTY = new DefaultFileCollectionSnapshot(Collections.<String, NormalizedFileSnapshot>emptyMap(), UNORDERED, true);
+
     boolean isEmpty();
 
     /**
@@ -35,9 +40,17 @@ public interface FileCollectionSnapshot {
      */
     Iterator<TaskStateChange> iterateContentChangesSince(FileCollectionSnapshot oldSnapshot, String title);
 
+    /**
+     * Returns the elements of this snapshot, including regular files, directories and missing files
+     */
+    Collection<File> getElements();
+
+    /**
+     * Returns the regular files that make up this snapshot.
+     */
     Collection<File> getFiles();
 
     Map<String, NormalizedFileSnapshot> getSnapshots();
 
-    void appendToCacheKey(TaskCacheKeyBuilder builder);
+    void appendToHasher(BuildCacheHasher hasher);
 }
