@@ -35,9 +35,6 @@ import org.gradle.api.artifacts.ConfigurationContainer;
 import org.gradle.api.artifacts.dsl.ArtifactHandler;
 import org.gradle.api.artifacts.dsl.DependencyHandler;
 import org.gradle.api.artifacts.dsl.RepositoryHandler;
-import org.gradle.api.attributes.AttributeMatchingStrategy;
-import org.gradle.api.attributes.AttributesSchema;
-import org.gradle.api.attributes.CompatibilityRuleChain;
 import org.gradle.api.component.SoftwareComponentContainer;
 import org.gradle.api.file.ConfigurableFileCollection;
 import org.gradle.api.file.ConfigurableFileTree;
@@ -121,29 +118,13 @@ import java.util.Set;
 import java.util.TreeMap;
 import java.util.TreeSet;
 
+import static com.google.common.base.Strings.isNullOrEmpty;
 import static java.util.Collections.singletonMap;
-import static org.gradle.api.internal.artifacts.ArtifactAttributes.*;
+import static org.gradle.util.ConfigureUtil.configureUsing;
 import static org.gradle.util.GUtil.addMaps;
-import static org.gradle.util.GUtil.isTrue;
 
 @NoConventionMapping
 public class DefaultProject extends AbstractPluginAware implements ProjectInternal, DynamicObjectAware {
-    private static final Action<AttributeMatchingStrategy<String>> ARTIFACT_ATTRIBUTE_CONFIG = new Action<AttributeMatchingStrategy<String>>() {
-        @Override
-        public void execute(AttributeMatchingStrategy<String> stringAttributeMatchingStrategy) {
-            CompatibilityRuleChain<String> compatibilityRules = stringAttributeMatchingStrategy.getCompatibilityRules();
-            compatibilityRules.assumeCompatibleWhenMissing();
-        }
-    };
-    private static final Action<AttributesSchema> CONFIGURE_DEFAULT_SCHEMA_ACTION = new Action<AttributesSchema>() {
-        @Override
-        public void execute(AttributesSchema attributesSchema) {
-            attributesSchema.attribute(ARTIFACT_FORMAT, ARTIFACT_ATTRIBUTE_CONFIG);
-            attributesSchema.attribute(ARTIFACT_CLASSIFIER, ARTIFACT_ATTRIBUTE_CONFIG);
-            attributesSchema.attribute(ARTIFACT_EXTENSION, ARTIFACT_ATTRIBUTE_CONFIG);
-        }
-    };
-
     private static final ModelType<ServiceRegistry> SERVICE_REGISTRY_MODEL_TYPE = ModelType.of(ServiceRegistry.class);
     private static final ModelType<File> FILE_MODEL_TYPE = ModelType.of(File.class);
     private static final ModelType<ProjectIdentifier> PROJECT_IDENTIFIER_MODEL_TYPE = ModelType.of(ProjectIdentifier.class);
@@ -209,8 +190,6 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
     private final Path path;
     private Path identityPath;
 
-    private final AttributesSchema configurationAttributesSchema;
-
     public DefaultProject(String name,
                           ProjectInternal parent,
                           File projectDir,
@@ -249,16 +228,10 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
 
         evaluationListener.add(gradle.getProjectEvaluationBroadcaster());
 
-        configurationAttributesSchema = services.get(AttributesSchema.class);
-
         populateModelRegistry(services.get(ModelRegistry.class));
-        configureSchema();
-    }
-
-    private void configureSchema() {
-        configurationAttributesSchema(CONFIGURE_DEFAULT_SCHEMA_ACTION);
     }
 
+    @SuppressWarnings("unused")
     static class BasicServicesRules extends RuleSource {
         @Hidden @Model
         SourceDirectorySetFactory sourceDirectorySetFactory(ServiceRegistry serviceRegistry) {
@@ -343,10 +316,12 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return "Project.<init>." + path + "()";
     }
 
+    @Override
     public ProjectInternal getRootProject() {
         return rootProject;
     }
 
+    @Override
     public GradleInternal getGradle() {
         return gradle;
     }
@@ -363,56 +338,69 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
     }
 
     @Inject
+    @Override
     public ScriptHandler getBuildscript() {
         // Decoration takes care of the implementation
         throw new UnsupportedOperationException();
     }
 
+    @Override
     public File getBuildFile() {
         return getBuildscript().getSourceFile();
     }
 
+    @Override
     public void setScript(groovy.lang.Script buildScript) {
         extensibleDynamicObject.addObject(new BeanDynamicObject(buildScript).withNoProperties().withNotImplementsMissing(),
             ExtensibleDynamicObject.Location.BeforeConvention);
     }
 
+    @Override
     public ScriptSource getBuildScriptSource() {
         return buildScriptSource;
     }
 
+    @Override
     public File getRootDir() {
         return rootProject.getProjectDir();
     }
 
+    @Override
     public ProjectInternal getParent() {
         return parent;
     }
 
+    @Override
     public ProjectIdentifier getParentIdentifier() {
         return parent;
     }
 
+    @Override
     public DynamicObject getAsDynamicObject() {
         return extensibleDynamicObject;
     }
 
+    @Override
     public DynamicObject getInheritedScope() {
         return extensibleDynamicObject.getInheritable();
     }
 
+    @Override
     public String getName() {
         return name;
     }
 
+    @Override
     public String getDescription() {
         return description;
     }
 
+    @Override
     public void setDescription(String description) {
         this.description = description;
     }
 
+    @Override
     public Object getGroup() {
         if (group != null) {
             return group;
@@ -423,42 +411,52 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return group;
     }
 
+    @Override
     public void setGroup(Object group) {
         this.group = group;
     }
 
+    @Override
     public Object getVersion() {
         return version == null ? DEFAULT_VERSION : version;
     }
 
+    @Override
     public void setVersion(Object version) {
         this.version = version;
     }
 
+    @Override
     public Object getStatus() {
         return status == null ? DEFAULT_STATUS : status;
     }
 
+    @Override
     public void setStatus(Object status) {
         this.status = status;
     }
 
+    @Override
     public Map<String, Project> getChildProjects() {
         return childProjects;
     }
 
+    @Override
     public List<String> getDefaultTasks() {
         return defaultTasks;
     }
 
+    @Override
     public void setDefaultTasks(List<String> defaultTasks) {
         this.defaultTasks = defaultTasks;
     }
 
+    @Override
     public ProjectStateInternal getState() {
         return state;
     }
 
+    @Override
     public FileResolver getFileResolver() {
         if (fileResolver == null) {
             fileResolver = services.get(FileResolver.class);
@@ -474,6 +472,7 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         this.ant = ant;
     }
 
+    @Override
     public ArtifactHandler getArtifacts() {
         if (artifactHandler == null) {
             artifactHandler = services.get(ArtifactHandler.class);
@@ -486,11 +485,13 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
     }
 
     @Inject
+    @Override
     public RepositoryHandler getRepositories() {
         // Decoration takes care of the implementation
         throw new UnsupportedOperationException();
     }
 
+    @Override
     public ConfigurationContainer getConfigurations() {
         if (configurationContainer == null) {
             configurationContainer = services.get(ConfigurationContainer.class);
@@ -502,10 +503,12 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         this.configurationContainer = configurationContainer;
     }
 
+    @Override
     public Convention getConvention() {
         return extensibleDynamicObject.getConvention();
     }
 
+    @Override
     public String getPath() {
         return path.toString();
     }
@@ -522,20 +525,24 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return identityPath;
     }
 
+    @Override
     public int getDepth() {
         return depth;
     }
 
     @Inject
+    @Override
     public ProjectRegistry<ProjectInternal> getProjectRegistry() {
         // Decoration takes care of the implementation
         throw new UnsupportedOperationException();
     }
 
+    @Override
     public int depthCompare(Project otherProject) {
         return new Integer(getDepth()).compareTo(otherProject.getDepth());
     }
 
+    @Override
     public int compareTo(Project otherProject) {
         int depthCompare = depthCompare(otherProject);
         if (depthCompare == 0) {
@@ -545,6 +552,7 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         }
     }
 
+    @Override
     public String absoluteProjectPath(String path) {
         return this.path.absolutePath(path);
     }
@@ -564,10 +572,12 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return path.child(name);
     }
 
+    @Override
     public String relativeProjectPath(String path) {
         return this.path.relativePath(path);
     }
 
+    @Override
     public ProjectInternal project(String path) {
         ProjectInternal project = findProject(path);
         if (project == null) {
@@ -576,29 +586,35 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return project;
     }
 
+    @Override
     public ProjectInternal findProject(String path) {
-        if (!isTrue(path)) {
+        if (isNullOrEmpty(path)) {
             throw new InvalidUserDataException("A path must be specified!");
         }
         return getProjectRegistry().getProject(absoluteProjectPath(path));
     }
 
+    @Override
     public Set<Project> getAllprojects() {
         return new TreeSet<Project>(getProjectRegistry().getAllProjects(getPath()));
     }
 
+    @Override
     public Set<Project> getSubprojects() {
         return new TreeSet<Project>(getProjectRegistry().getSubProjects(getPath()));
     }
 
+    @Override
     public void subprojects(Action<? super Project> action) {
         configure(getSubprojects(), action);
     }
 
+    @Override
     public void allprojects(Action<? super Project> action) {
         configure(getAllprojects(), action);
     }
 
+    @Override
     public <T> Iterable<T> configure(Iterable<T> objects, Action<? super T> configureAction) {
         for (T object : objects) {
             configureAction.execute(object);
@@ -606,6 +622,7 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return objects;
     }
 
+    @Override
     public AntBuilder getAnt() {
         if (ant == null) {
             ant = createAntBuilder();
@@ -613,6 +630,7 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return ant;
     }
 
+    @Override
     public AntBuilder createAntBuilder() {
         return getAntBuilderFactory().create();
     }
@@ -620,10 +638,12 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
     /**
      * This method is used when scripts access the project via project.x
      */
+    @Override
     public Project getProject() {
         return this;
     }
 
+    @Override
     public DefaultProject evaluate() {
         getProjectEvaluator().evaluate(this, state);
         return this;
@@ -639,10 +659,12 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return this;
     }
 
+    @Override
     public TaskContainerInternal getTasks() {
         return taskContainer;
     }
 
+    @Override
     public void defaultTasks(String... defaultTasks) {
         if (defaultTasks == null) {
             throw new InvalidUserDataException("Default tasks must not be null!");
@@ -656,14 +678,17 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         }
     }
 
+    @Override
     public void addChildProject(ProjectInternal childProject) {
         childProjects.put(childProject.getName(), childProject);
     }
 
+    @Override
     public File getProjectDir() {
         return projectDir;
     }
 
+    @Override
     public File getBuildDir() {
         if (buildDirCached == null) {
             buildDirCached = file(buildDir);
@@ -671,11 +696,13 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return buildDirCached;
     }
 
+    @Override
     public void setBuildDir(Object path) {
         buildDir = path;
         buildDirCached = null;
     }
 
+    @Override
     public void evaluationDependsOnChildren() {
         for (Project project : childProjects.values()) {
             DefaultProject defaultProjectToEvaluate = (DefaultProject) project;
@@ -683,8 +710,9 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         }
     }
 
+    @Override
     public Project evaluationDependsOn(String path) {
-        if (!isTrue(path)) {
+        if (isNullOrEmpty(path)) {
             throw new InvalidUserDataException("You must specify a project!");
         }
         DefaultProject projectToEvaluate = (DefaultProject) project(path);
@@ -714,10 +742,12 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return builder.toString();
     }
 
+    @Override
     public String toString() {
         return getDisplayName();
     }
 
+    @Override
     public Map<Project, Set<Task>> getAllTasks(boolean recursive) {
         final Map<Project, Set<Task>> foundTargets = new TreeMap<Project, Set<Task>>();
         Action<Project> action = new Action<Project>() {
@@ -733,8 +763,9 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return foundTargets;
     }
 
+    @Override
     public Set<Task> getTasksByName(final String name, boolean recursive) {
-        if (!isTrue(name)) {
+        if (isNullOrEmpty(name)) {
             throw new InvalidUserDataException("Name is not specified!");
         }
         final Set<Task> foundTasks = new HashSet<Task>();
@@ -763,62 +794,91 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         throw new UnsupportedOperationException();
     }
 
+    @Override
     public File file(Object path) {
         return getFileOperations().file(path);
     }
 
+    @Override
     public File file(Object path, PathValidation validation) {
         return getFileOperations().file(path, validation);
     }
 
+    @Override
     public URI uri(Object path) {
         return getFileOperations().uri(path);
     }
 
+    @Override
     public ConfigurableFileCollection files(Object... paths) {
         return getFileOperations().files(paths);
     }
 
+    @Override
     public ConfigurableFileCollection files(Object paths, Closure closure) {
-        return ConfigureUtil.configure(closure, getFileOperations().files(paths));
+        return ConfigureUtil.configure(closure, files(paths));
     }
 
+    @Override
+    public ConfigurableFileCollection files(Object paths, Action<? super ConfigurableFileCollection> configureAction) {
+        ConfigurableFileCollection files = files(paths);
+        configureAction.execute(files);
+        return files;
+    }
+
+    @Override
     public ConfigurableFileTree fileTree(Object baseDir) {
         return getFileOperations().fileTree(baseDir);
     }
 
+    @Override
     public ConfigurableFileTree fileTree(Object baseDir, Closure closure) {
-        return ConfigureUtil.configure(closure, getFileOperations().fileTree(baseDir));
+        return ConfigureUtil.configure(closure, fileTree(baseDir));
     }
 
+    @Override
+    public ConfigurableFileTree fileTree(Object baseDir, Action<? super ConfigurableFileTree> configureAction) {
+        ConfigurableFileTree fileTree = fileTree(baseDir);
+        configureAction.execute(fileTree);
+        return fileTree;
+    }
+
+    @Override
     public ConfigurableFileTree fileTree(Map<String, ?> args) {
         return getFileOperations().fileTree(args);
     }
 
+    @Override
     public FileTree zipTree(Object zipPath) {
         return getFileOperations().zipTree(zipPath);
     }
 
+    @Override
     public FileTree tarTree(Object tarPath) {
         return getFileOperations().tarTree(tarPath);
     }
 
+    @Override
     public ResourceHandler getResources() {
         return getFileOperations().getResources();
     }
 
+    @Override
     public String relativePath(Object path) {
         return getFileOperations().relativePath(path);
     }
 
+    @Override
     public File mkdir(Object path) {
         return getFileOperations().mkdir(path);
     }
 
+    @Override
     public boolean delete(Object... paths) {
         return getFileOperations().delete(paths);
     }
 
+    @Override
     public WorkResult delete(Action<? super DeleteSpec> action) {
         return getFileOperations().delete(action);
     }
@@ -830,6 +890,7 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return antBuilderFactory;
     }
 
+    @Override
     public DependencyHandler getDependencies() {
         if (dependencyHandler == null) {
             dependencyHandler = services.get(DependencyHandler.class);
@@ -841,62 +902,76 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         this.dependencyHandler = dependencyHandler;
     }
 
+    @Override
     public ProjectEvaluationListener getProjectEvaluationBroadcaster() {
         return evaluationListener.getSource();
     }
 
+    @Override
     public void beforeEvaluate(Action<? super Project> action) {
         evaluationListener.add("beforeEvaluate", action);
     }
 
+    @Override
     public void afterEvaluate(Action<? super Project> action) {
         evaluationListener.add("afterEvaluate", action);
     }
 
+    @Override
     public void beforeEvaluate(Closure closure) {
         evaluationListener.add(new ClosureBackedMethodInvocationDispatch("beforeEvaluate", closure));
     }
 
+    @Override
     public void afterEvaluate(Closure closure) {
         evaluationListener.add(new ClosureBackedMethodInvocationDispatch("afterEvaluate", closure));
     }
 
+    @Override
     public Logger getLogger() {
         return BUILD_LOGGER;
     }
 
+    @Override
     public StandardOutputCapture getStandardOutputCapture() {
         return getLogging();
     }
 
     @Inject
+    @Override
     public LoggingManagerInternal getLogging() {
         // Decoration takes care of the implementation
         throw new UnsupportedOperationException();
     }
 
     @Inject
+    @Override
     public SoftwareComponentContainer getComponents() {
         // Decoration takes care of the implementation
         throw new UnsupportedOperationException();
     }
 
+    @Override
     public Object property(String propertyName) throws MissingPropertyException {
         return extensibleDynamicObject.getProperty(propertyName);
     }
 
+    @Override
     public Object findProperty(String propertyName) {
         return hasProperty(propertyName) ? property(propertyName) : null;
     }
 
+    @Override
     public void setProperty(String name, Object value) {
         extensibleDynamicObject.setProperty(name, value);
     }
 
+    @Override
     public boolean hasProperty(String propertyName) {
         return extensibleDynamicObject.hasProperty(propertyName);
     }
 
+    @Override
     public Map<String, ?> getProperties() {
         return DeprecationLogger.whileDisabled(new Factory<Map<String, ?>>() {
             public Map<String, ?> create() {
@@ -905,26 +980,32 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         });
     }
 
+    @Override
     public WorkResult copy(Closure closure) {
-        return copy(ConfigureUtil.configureUsing(closure));
+        return copy(configureUsing(closure));
     }
 
+    @Override
     public WorkResult copy(Action<? super CopySpec> action) {
         return getFileOperations().copy(action);
     }
 
+    @Override
     public WorkResult sync(Action<? super CopySpec> action) {
         return getFileOperations().sync(action);
     }
 
+    @Override
     public CopySpec copySpec(Closure closure) {
-        return copySpec(ConfigureUtil.configureUsing(closure));
+        return ConfigureUtil.configure(closure, copySpec());
     }
 
+    @Override
     public CopySpec copySpec(Action<? super CopySpec> action) {
         return Actions.with(copySpec(), action);
     }
 
+    @Override
     public CopySpec copySpec() {
         return getFileOperations().copySpec();
     }
@@ -935,54 +1016,79 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         throw new UnsupportedOperationException();
     }
 
+    @Override
     public ExecResult javaexec(Closure closure) {
-        return javaexec(ConfigureUtil.configureUsing(closure));
+        return javaexec(configureUsing(closure));
     }
 
+    @Override
     public ExecResult javaexec(Action<? super JavaExecSpec> action) {
         return getProcessOperations().javaexec(action);
     }
 
+    @Override
     public ExecResult exec(Closure closure) {
-        return exec(ConfigureUtil.configureUsing(closure));
+        return exec(configureUsing(closure));
     }
 
+    @Override
     public ExecResult exec(Action<? super ExecSpec> action) {
         return getProcessOperations().exec(action);
     }
 
+    @Override
     public ServiceRegistry getServices() {
         return services;
     }
 
+    @Override
     public ServiceRegistryFactory getServiceRegistryFactory() {
         return services.get(ServiceRegistryFactory.class);
     }
 
+    @Override
     public Module getModule() {
         return services.get(DependencyMetaDataProvider.class).getModule();
     }
 
+    @Override
     public AntBuilder ant(Closure configureClosure) {
         return ConfigureUtil.configure(configureClosure, getAnt());
     }
 
+    @Override
+    public AntBuilder ant(Action<? super AntBuilder> configureAction) {
+        AntBuilder ant = getAnt();
+        configureAction.execute(ant);
+        return ant;
+    }
+
+    @Override
     public void subprojects(Closure configureClosure) {
         configure(getSubprojects(), configureClosure);
     }
 
+    @Override
     public void allprojects(Closure configureClosure) {
         configure(getAllprojects(), configureClosure);
     }
 
+    @Override
     public Project project(String path, Closure configureClosure) {
         return ConfigureUtil.configure(configureClosure, project(path));
     }
 
+    @Override
+    public Project project(String path, Action<? super Project> configureAction) {
+        return Actions.with(project(path), configureAction);
+    }
+
+    @Override
     public Object configure(Object object, Closure configureClosure) {
         return ConfigureUtil.configure(configureClosure, object);
     }
 
+    @Override
     public Iterable<?> configure(Iterable<?> objects, Closure configureClosure) {
         for (Object object : objects) {
             configure(object, configureClosure);
@@ -990,26 +1096,37 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return objects;
     }
 
+    @Override
     public void configurations(Closure configureClosure) {
         ((Configurable<?>) getConfigurations()).configure(configureClosure);
     }
 
+    @Override
     public void repositories(Closure configureClosure) {
         ConfigureUtil.configure(configureClosure, getRepositories());
     }
 
+    @Override
     public void dependencies(Closure configureClosure) {
         ConfigureUtil.configure(configureClosure, getDependencies());
     }
 
+    @Override
     public void artifacts(Closure configureClosure) {
         ConfigureUtil.configure(configureClosure, getArtifacts());
     }
 
+    @Override
+    public void artifacts(Action<? super ArtifactHandler> configureAction) {
+        configureAction.execute(getArtifacts());
+    }
+
+    @Override
     public void buildscript(Closure configureClosure) {
         ConfigureUtil.configure(configureClosure, getBuildscript());
     }
 
+    @Override
     public Task task(String task) {
         return taskContainer.create(task);
     }
@@ -1018,6 +1135,7 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return taskContainer.create(task.toString());
     }
 
+    @Override
     public Task task(String task, Closure configureClosure) {
         return taskContainer.create(task).configure(configureClosure);
     }
@@ -1026,6 +1144,7 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return task(task.toString(), configureClosure);
     }
 
+    @Override
     public Task task(Map options, String task) {
         return taskContainer.create(addMaps(options, singletonMap(Task.TASK_NAME, task)));
     }
@@ -1034,6 +1153,7 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return task(options, task.toString());
     }
 
+    @Override
     public Task task(Map options, String task, Closure configureClosure) {
         return taskContainer.create(addMaps(options, singletonMap(Task.TASK_NAME, task))).configure(configureClosure);
     }
@@ -1043,12 +1163,14 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
     }
 
     @Inject
+    @Override
     public ProjectConfigurationActionContainer getConfigurationActions() {
         // Decoration takes care of the implementation
         throw new UnsupportedOperationException();
     }
 
     @Inject
+    @Override
     public ModelRegistry getModelRegistry() {
         // Decoration takes care of the implementation
         throw new UnsupportedOperationException();
@@ -1066,6 +1188,7 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
     }
 
     @Inject
+    @Override
     public PluginManagerInternal getPluginManager() {
         // Decoration takes care of the implementation
         throw new UnsupportedOperationException();
@@ -1083,10 +1206,12 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         throw new UnsupportedOperationException();
     }
 
+    @Override
     public ClassLoaderScope getClassLoaderScope() {
         return classLoaderScope;
     }
 
+    @Override
     public ClassLoaderScope getBaseClassLoaderScope() {
         return baseClassLoaderScope;
     }
@@ -1098,21 +1223,25 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return object;
     }
 
+    @Override
     public <T> NamedDomainObjectContainer<T> container(Class<T> type) {
         Instantiator instantiator = getServices().get(Instantiator.class);
         return instantiator.newInstance(FactoryNamedDomainObjectContainer.class, type, instantiator, new DynamicPropertyNamer());
     }
 
+    @Override
     public <T> NamedDomainObjectContainer<T> container(Class<T> type, NamedDomainObjectFactory<T> factory) {
         Instantiator instantiator = getServices().get(Instantiator.class);
         return instantiator.newInstance(FactoryNamedDomainObjectContainer.class, type, instantiator, new DynamicPropertyNamer(), factory);
     }
 
+    @Override
     public <T> NamedDomainObjectContainer<T> container(Class<T> type, Closure factoryClosure) {
         Instantiator instantiator = getServices().get(Instantiator.class);
         return instantiator.newInstance(FactoryNamedDomainObjectContainer.class, type, instantiator, new DynamicPropertyNamer(), factoryClosure);
     }
 
+    @Override
     public ExtensionContainerInternal getExtensions() {
         return (ExtensionContainerInternal) getConvention();
     }
@@ -1133,6 +1262,7 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         throw new UnsupportedOperationException();
     }
 
+    @Override
     public void addDeferredConfiguration(Runnable configuration) {
         getDeferredProjectConfiguration().add(configuration);
     }
@@ -1142,14 +1272,4 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         getDeferredProjectConfiguration().fire();
     }
 
-    @Override
-    public AttributesSchema configurationAttributesSchema(Action<? super AttributesSchema> configureAction) {
-        configureAction.execute(configurationAttributesSchema);
-        return configurationAttributesSchema;
-    }
-
-    @Override
-    public AttributesSchema getAttributesSchema() {
-        return configurationAttributesSchema;
-    }
 }
