@@ -20,6 +20,7 @@ import org.gradle.api.internal.TaskInternal;
 import org.gradle.api.internal.changedetection.state.ClasspathSnapshotNormalizationStrategy;
 import org.gradle.api.internal.changedetection.state.ClasspathSnapshotter;
 import org.gradle.api.internal.tasks.TaskInputFilePropertyBuilderInternal;
+import org.gradle.api.internal.changedetection.state.FileCollectionSnapshotter;
 import org.gradle.api.tasks.Classpath;
 import org.gradle.api.tasks.InputFiles;
 import org.gradle.api.tasks.TaskInputFilePropertyBuilder;
@@ -28,7 +29,7 @@ import org.gradle.util.DeprecationLogger;
 import java.lang.annotation.Annotation;
 import java.util.concurrent.Callable;
 
-public class ClasspathPropertyAnnotationHandler implements OverridingPropertyAnnotationHandler {
+public class ClasspathPropertyAnnotationHandler implements OverridingPropertyAnnotationHandler, FileSnapshottingPropertyAnnotationHandler {
     @Override
     public Class<? extends Annotation> getAnnotationType() {
         return Classpath.class;
@@ -39,6 +40,10 @@ public class ClasspathPropertyAnnotationHandler implements OverridingPropertyAnn
         return InputFiles.class;
     }
 
+    public Class<? extends FileCollectionSnapshotter> getSnapshotterType() {
+        return ClasspathSnapshotter.class;
+    }
+
     @Override
     public void attachActions(final TaskPropertyActionContext context) {
         context.setConfigureAction(new UpdateAction() {
@@ -47,7 +52,7 @@ public class ClasspathPropertyAnnotationHandler implements OverridingPropertyAnn
                     ((TaskInputFilePropertyBuilderInternal) task.getInputs().files(futureValue))
                     .withPropertyName(context.getName())
                     .withSnapshotNormalizationStrategy(ClasspathSnapshotNormalizationStrategy.INSTANCE)
-                    .withSnapshotter(ClasspathSnapshotter.class)
+                    .withSnapshotter(getSnapshotterType())
                     .optional(context.isOptional());
                 DeprecationLogger.whileDisabled(new Runnable() {
                     @Override
