@@ -65,7 +65,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
             @Override
             public void childSpecAdded(CopySpecInternal.CopySpecAddress path, final CopySpecInternal spec) {
                 if (getState().getExecuting()) {
-                    if (getOutputs().isCacheEnabled() && getProject().getGradle().getStartParameter().isTaskOutputCacheEnabled()) {
+                    if (getProject().getGradle().getStartParameter().isBuildCacheEnabled() && getState().getTaskOutputCaching().isEnabled()) {
                         throw new GradleException("It is not allowed to modify child specs of the task at execution time when task output caching is enabled. "
                             + CONFIGURE_SPEC_DURING_CONFIGURATION + ".");
                     }
@@ -135,7 +135,7 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
                 });
             }
         });
-        this.getOutputs().doNotCacheIf(new Spec<Task>() {
+        this.getOutputs().doNotCacheIf("Has custom actions", new Spec<Task>() {
             @Override
             public boolean isSatisfiedBy(Task task) {
                 return rootSpec.hasCustomActions();
@@ -174,15 +174,19 @@ public abstract class AbstractCopyTask extends ConventionTask implements CopySpe
 
     @TaskAction
     protected void copy() {
-        Instantiator instantiator = getInstantiator();
-        FileSystem fileSystem = getFileSystem();
-
-        CopyActionExecuter copyActionExecuter = new CopyActionExecuter(instantiator, fileSystem);
+        CopyActionExecuter copyActionExecuter = createCopyActionExecuter();
         CopyAction copyAction = createCopyAction();
         WorkResult didWork = copyActionExecuter.execute(rootSpec, copyAction);
         setDidWork(didWork.getDidWork());
     }
 
+    protected CopyActionExecuter createCopyActionExecuter() {
+        Instantiator instantiator = getInstantiator();
+        FileSystem fileSystem = getFileSystem();
+
+        return new CopyActionExecuter(instantiator, fileSystem, false);
+    }
+
     /**
      * Returns the source files for this task.
      * @return The source files. Never returns null.
