@@ -16,8 +16,12 @@
 
 package org.gradle.cache.internal;
 
+import com.google.common.hash.Hashing;
 import org.gradle.api.Nullable;
+import org.gradle.api.internal.hash.FileHasher;
 import org.gradle.cache.CacheRepository;
+import org.gradle.internal.classloader.ClassLoaderHierarchyHasher;
+import org.gradle.internal.classloader.ClasspathHasher;
 import org.gradle.util.GradleVersion;
 
 import java.io.File;
@@ -40,4 +44,8 @@ public class CacheRepositoryServices {
             scopeMapping,
             factory);
     }
+
+    protected CacheKeyBuilder createCacheKeyBuilder(FileHasher fileHasher, ClasspathHasher classpathHasher, ClassLoaderHierarchyHasher classLoaderHierarchyHasher) {
+        return new DefaultCacheKeyBuilder(Hashing.md5(), fileHasher, classpathHasher, classLoaderHierarchyHasher);
+    }
 }
