@@ -15,6 +15,9 @@
  */
 package org.gradle.cache.internal;
 
+import org.gradle.api.Nullable;
+import org.gradle.api.Transformer;
+
 /**
  * An indexed cache that may perform updates asynchronously.
  */
@@ -22,8 +25,14 @@ public interface MultiProcessSafeAsyncPersistentIndexedCache<K, V> extends UnitO
     /**
      * Fetches the given entry, blocking until the result is available.
      */
+    @Nullable
     V get(K key);
 
+    /**
+     * Fetches the given entry, producing if necessary, blocking until the result is available. This method may or may not block until any updates have completed and will invoke the given completion action when the operation is complete.
+     */
+    V get(K key, Transformer<? extends V, ? super K> producer, Runnable completion);
+
     /**
      * Submits an update to be applied later. This method may or may not block, and will invoke the given completion action when the operation is complete.
      */
