@@ -20,12 +20,14 @@ import com.google.common.collect.Lists;
 import groovy.lang.Closure;
 import org.gradle.BuildAdapter;
 import org.gradle.BuildListener;
+import org.gradle.BuildResult;
 import org.gradle.StartParameter;
 import org.gradle.api.Action;
 import org.gradle.api.Nullable;
 import org.gradle.api.Project;
 import org.gradle.api.ProjectEvaluationListener;
 import org.gradle.api.initialization.IncludedBuild;
+import org.gradle.api.initialization.Settings;
 import org.gradle.api.internal.GradleInternal;
 import org.gradle.api.internal.file.FileResolver;
 import org.gradle.api.internal.initialization.ClassLoaderScope;
@@ -35,8 +37,6 @@ import org.gradle.api.internal.plugins.PluginManagerInternal;
 import org.gradle.api.internal.project.AbstractPluginAware;
 import org.gradle.api.internal.project.ProjectInternal;
 import org.gradle.api.invocation.Gradle;
-import org.gradle.caching.internal.BuildCacheConfiguration;
-import org.gradle.caching.internal.BuildCacheConfigurationInternal;
 import org.gradle.configuration.ScriptPluginFactory;
 import org.gradle.execution.TaskGraphExecuter;
 import org.gradle.initialization.ClassLoaderScopeRegistry;
@@ -46,7 +46,7 @@ import org.gradle.internal.installation.CurrentGradleInstallation;
 import org.gradle.internal.installation.GradleInstallation;
 import org.gradle.internal.service.ServiceRegistry;
 import org.gradle.internal.service.scopes.ServiceRegistryFactory;
-import org.gradle.listener.ActionBroadcast;
+import org.gradle.internal.MutableActionSet;
 import org.gradle.listener.ClosureBackedMethodInvocationDispatch;
 import org.gradle.util.GradleVersion;
 import org.gradle.util.Path;
@@ -66,7 +66,7 @@ public class DefaultGradle extends AbstractPluginAware implements GradleInternal
     private final ListenerBroadcast<BuildListener> buildListenerBroadcast;
     private final ListenerBroadcast<ProjectEvaluationListener> projectEvaluationListenerBroadcast;
     private final Collection<IncludedBuild> includedBuilds = Lists.newArrayList();
-    private ActionBroadcast<Project> rootProjectActions = new ActionBroadcast<Project>();
+    private MutableActionSet<Project> rootProjectActions = new MutableActionSet<Project>();
     private Path identityPath;
     private final ClassLoaderScope classLoaderScope;
 
@@ -126,27 +126,33 @@ public class DefaultGradle extends AbstractPluginAware implements GradleInternal
         identityPath = path;
     }
 
+    @Override
     public GradleInternal getParent() {
         return parent;
     }
 
+    @Override
     public String getGradleVersion() {
         return GradleVersion.current().getVersion();
     }
 
+    @Override
     public File getGradleHomeDir() {
         GradleInstallation gradleInstallation = getCurrentGradleInstallation().getInstallation();
         return gradleInstallation == null ? null : gradleInstallation.getGradleHome();
     }
 
+    @Override
     public File getGradleUserHomeDir() {
         return startParameter.getGradleUserHomeDir();
     }
 
+    @Override
     public StartParameter getStartParameter() {
         return startParameter;
     }
 
+    @Override
     public ProjectInternal getRootProject() {
         if (rootProject == null) {
             throw new IllegalStateException("The root project is not yet available for " + this + ".");
@@ -154,10 +160,12 @@ public class DefaultGradle extends AbstractPluginAware implements GradleInternal
         return rootProject;
     }
 
+    @Override
     public void setRootProject(ProjectInternal rootProject) {
         this.rootProject = rootProject;
     }
 
+    @Override
     public void rootProject(Action<? super Project> action) {
         if (rootProjectActions != null) {
             rootProjectActions.add(action);
@@ -167,6 +175,7 @@ public class DefaultGradle extends AbstractPluginAware implements GradleInternal
         }
     }
 
+    @Override
     public void allprojects(final Action<? super Project> action) {
         rootProject(new Action<Project>() {
             public void execute(Project project) {
@@ -175,85 +184,134 @@ public class DefaultGradle extends AbstractPluginAware implements GradleInternal
         });
     }
 
+    @Override
     public ProjectInternal getDefaultProject() {
         return defaultProject;
     }
 
+    @Override
     public void setDefaultProject(ProjectInternal defaultProject) {
         this.defaultProject = defaultProject;
     }
 
     @Inject
+    @Override
     public TaskGraphExecuter getTaskGraph() {
         throw new UnsupportedOperationException();
     }
 
+    @Override
     public ProjectEvaluationListener addProjectEvaluationListener(ProjectEvaluationListener listener) {
         addListener(listener);
         return listener;
     }
 
+    @Override
     public void removeProjectEvaluationListener(ProjectEvaluationListener listener) {
         removeListener(listener);
     }
 
+    @Override
     public void beforeProject(Closure closure) {
         projectEvaluationListenerBroadcast.add(new ClosureBackedMethodInvocationDispatch("beforeEvaluate", closure));
     }
 
+    @Override
+    public void beforeProject(Action<? super Project> action) {
+        projectEvaluationListenerBroadcast.add("beforeEvaluate", action);
+    }
+
+    @Override
     public void afterProject(Closure closure) {
         projectEvaluationListenerBroadcast.add(new ClosureBackedMethodInvocationDispatch("afterEvaluate", closure));
     }
 
+    @Override
+    public void afterProject(Action<? super Project> action) {
+        projectEvaluationListenerBroadcast.add("afterEvaluate", action);
+    }
+
+    @Override
     public void buildStarted(Closure closure) {
         buildListenerBroadcast.add(new ClosureBackedMethodInvocationDispatch("buildStarted", closure));
     }
 
+    @Override
+    public void buildStarted(Action<? super Gradle> action) {
+        buildListenerBroadcast.add("buildStarted", action);
+    }
+
+    @Override
     public void settingsEvaluated(Closure closure) {
         buildListenerBroadcast.add(new ClosureBackedMethodInvocationDispatch("settingsEvaluated", closure));
     }
 
+    @Override
+    public void settingsEvaluated(Action<? super Settings> action) {
+        buildListenerBroadcast.add("settingsEvaluated", action);
+    }
+
+    @Override
     public void projectsLoaded(Closure closure) {
         buildListenerBroadcast.add(new ClosureBackedMethodInvocationDispatch("projectsLoaded", closure));
     }
 
+    @Override
+    public void projectsLoaded(Action<? super Gradle> action) {
+        buildListenerBroadcast.add("projectsLoaded", action);
+    }
+
+    @Override
     public void projectsEvaluated(Closure closure) {
         buildListenerBroadcast.add(new ClosureBackedMethodInvocationDispatch("projectsEvaluated", closure));
     }
 
+    @Override
+    public void projectsEvaluated(Action<? super Gradle> action) {
+        buildListenerBroadcast.add("projectsEvaluated", action);
+    }
+
+    @Override
     public void buildFinished(Closure closure) {
         buildListenerBroadcast.add(new ClosureBackedMethodInvocationDispatch("buildFinished", closure));
     }
 
+    @Override
+    public void buildFinished(Action<? super BuildResult> action) {
+        buildListenerBroadcast.add("buildFinished", action);
+    }
+
+    @Override
     public void addListener(Object listener) {
         getListenerManager().addListener(listener);
     }
 
+    @Override
     public void removeListener(Object listener) {
         getListenerManager().removeListener(listener);
     }
 
+    @Override
     public void useLogger(Object logger) {
         getListenerManager().useLogger(logger);
     }
 
+    @Override
     public ProjectEvaluationListener getProjectEvaluationBroadcaster() {
         return projectEvaluationListenerBroadcast.getSource();
     }
 
+    @Override
     public void addBuildListener(BuildListener buildListener) {
         addListener(buildListener);
     }
 
+    @Override
     public BuildListener getBuildListenerBroadcaster() {
         return buildListenerBroadcast.getSource();
     }
 
     @Override
-    public void buildCache(Action<? super BuildCacheConfiguration> action) {
-        action.execute(getBuildCache());
-    }
-
     public Gradle getGradle() {
         return this;
     }
@@ -278,6 +336,7 @@ public class DefaultGradle extends AbstractPluginAware implements GradleInternal
         throw new NoSuchElementException("Included build '" + name + "' not found.");
     }
 
+    @Override
     public ServiceRegistry getServices() {
         return services;
     }
@@ -292,15 +351,11 @@ public class DefaultGradle extends AbstractPluginAware implements GradleInternal
         return new DefaultObjectConfigurationAction(getFileResolver(), getScriptPluginFactory(), getScriptHandlerFactory(), getClassLoaderScope(), this);
     }
 
+    @Override
     public ClassLoaderScope getClassLoaderScope() {
         return classLoaderScope;
     }
 
-    @Inject
-    public BuildCacheConfigurationInternal getBuildCache() {
-        throw new UnsupportedOperationException();
-    }
-
     @Inject
     protected ScriptHandlerFactory getScriptHandlerFactory() {
         throw new UnsupportedOperationException();
