@@ -26,6 +26,7 @@ import org.gradle.internal.jvm.inspection.JvmVersionDetector;
 import org.gradle.internal.logging.events.OutputEventListener;
 import org.gradle.internal.remote.MessagingServer;
 import org.gradle.process.internal.JavaExecHandleFactory;
+import org.gradle.process.internal.health.memory.MemoryManager;
 import org.gradle.process.internal.worker.child.ApplicationClassesInSystemClassLoaderWorkerFactory;
 
 import java.io.File;
@@ -39,10 +40,11 @@ public class DefaultWorkerProcessFactory implements WorkerProcessFactory {
     private final JavaExecHandleFactory execHandleFactory;
     private final OutputEventListener outputEventListener;
     private final ApplicationClassesInSystemClassLoaderWorkerFactory workerFactory;
+    private final MemoryManager memoryManager;
     private int connectTimeoutSeconds = 120;
 
     public DefaultWorkerProcessFactory(LogLevel workerLogLevel, MessagingServer server, ClassPathRegistry classPathRegistry, IdGenerator<?> idGenerator,
-                                       File gradleUserHomeDir, TemporaryFileProvider temporaryFileProvider, JavaExecHandleFactory execHandleFactory, JvmVersionDetector jvmVersionDetector, OutputEventListener outputEventListener) {
+                                       File gradleUserHomeDir, TemporaryFileProvider temporaryFileProvider, JavaExecHandleFactory execHandleFactory, JvmVersionDetector jvmVersionDetector, OutputEventListener outputEventListener, MemoryManager memoryManager) {
         this.workerLogLevel = workerLogLevel;
         this.server = server;
         this.idGenerator = idGenerator;
@@ -50,6 +52,7 @@ public class DefaultWorkerProcessFactory implements WorkerProcessFactory {
         this.execHandleFactory = execHandleFactory;
         this.outputEventListener = outputEventListener;
         this.workerFactory = new ApplicationClassesInSystemClassLoaderWorkerFactory(classPathRegistry, temporaryFileProvider, jvmVersionDetector);
+        this.memoryManager = memoryManager;
     }
 
     public void setConnectTimeoutSeconds(int connectTimeoutSeconds) {
@@ -65,7 +68,7 @@ public class DefaultWorkerProcessFactory implements WorkerProcessFactory {
     }
 
     private DefaultWorkerProcessBuilder newWorker() {
-        DefaultWorkerProcessBuilder workerProcessBuilder = new DefaultWorkerProcessBuilder(execHandleFactory, server, idGenerator, workerFactory, outputEventListener);
+        DefaultWorkerProcessBuilder workerProcessBuilder = new DefaultWorkerProcessBuilder(execHandleFactory, server, idGenerator, workerFactory, outputEventListener, memoryManager);
         workerProcessBuilder.setLogLevel(workerLogLevel);
         workerProcessBuilder.setGradleUserHomeDir(gradleUserHomeDir);
         workerProcessBuilder.setConnectTimeoutSeconds(connectTimeoutSeconds);
