@@ -15,9 +15,13 @@
  */
 package org.gradle.api.internal.artifacts;
 
+import org.gradle.api.attributes.AttributesSchema;
+import org.gradle.api.internal.artifacts.ivyservice.resolveengine.excludes.ModuleExclusions;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.DependencyGraphVisitor;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.DependencyArtifactsVisitor;
 import org.gradle.api.internal.artifacts.repositories.ResolutionAwareRepository;
+import org.gradle.api.specs.Spec;
+import org.gradle.internal.component.model.DependencyMetadata;
 
 import java.util.List;
 
@@ -25,6 +29,9 @@ public interface ArtifactDependencyResolver {
     void resolve(ResolveContext resolveContext,
                  List<? extends ResolutionAwareRepository> repositories,
                  GlobalDependencyResolutionRules metadataHandler,
+                 Spec<? super DependencyMetadata> edgeFilter,
                  DependencyGraphVisitor graphVisitor,
-                 DependencyArtifactsVisitor artifactsVisitor);
+                 DependencyArtifactsVisitor artifactsVisitor,
+                 AttributesSchema attributesSchema,
+                 ImmutableModuleIdentifierFactory moduleIdentifierFactory, ModuleExclusions moduleExclusions);
 }
