@@ -22,6 +22,9 @@ import groovy.lang.Closure;
 import org.gradle.api.Action;
 import org.gradle.api.DomainObjectSet;
 import org.gradle.api.InvalidUserDataException;
+import org.gradle.api.Nullable;
+import org.gradle.api.artifacts.ArtifactCollection;
+import org.gradle.api.artifacts.ArtifactView;
 import org.gradle.api.artifacts.ConfigurablePublishArtifact;
 import org.gradle.api.artifacts.Configuration;
 import org.gradle.api.artifacts.ConfigurationPublications;
@@ -41,37 +44,43 @@ import org.gradle.api.artifacts.result.ResolutionResult;
 import org.gradle.api.artifacts.result.ResolvedArtifactResult;
 import org.gradle.api.attributes.Attribute;
 import org.gradle.api.attributes.AttributeContainer;
-import org.gradle.api.attributes.AttributesSchema;
 import org.gradle.api.file.FileCollection;
-import org.gradle.api.internal.ClosureBackedAction;
 import org.gradle.api.internal.CompositeDomainObjectSet;
 import org.gradle.api.internal.DefaultDomainObjectSet;
 import org.gradle.api.internal.artifacts.ConfigurationResolver;
 import org.gradle.api.internal.artifacts.DefaultDependencySet;
 import org.gradle.api.internal.artifacts.DefaultExcludeRule;
-import org.gradle.api.internal.artifacts.DefaultModuleVersionIdentifier;
 import org.gradle.api.internal.artifacts.DefaultPublishArtifactSet;
 import org.gradle.api.internal.artifacts.DefaultResolverResults;
 import org.gradle.api.internal.artifacts.ExcludeRuleNotationConverter;
+import org.gradle.api.internal.artifacts.ImmutableModuleIdentifierFactory;
 import org.gradle.api.internal.artifacts.Module;
 import org.gradle.api.internal.artifacts.ResolverResults;
 import org.gradle.api.internal.artifacts.component.ComponentIdentifierFactory;
 import org.gradle.api.internal.artifacts.dsl.dependencies.ProjectFinder;
 import org.gradle.api.internal.artifacts.ivyservice.moduleconverter.ConfigurationComponentMetaDataBuilder;
+import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.SelectedArtifactSet;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.projectresult.ResolvedProjectConfiguration;
 import org.gradle.api.internal.attributes.AttributeContainerInternal;
-import org.gradle.api.internal.attributes.DefaultAttributeContainer;
+import org.gradle.api.internal.attributes.AttributesSchemaInternal;
+import org.gradle.api.internal.attributes.DefaultMutableAttributeContainer;
+import org.gradle.api.internal.attributes.ImmutableAttributes;
+import org.gradle.api.internal.attributes.ImmutableAttributesFactory;
 import org.gradle.api.internal.file.AbstractFileCollection;
 import org.gradle.api.internal.file.FileCollectionFactory;
 import org.gradle.api.internal.file.FileCollectionInternal;
 import org.gradle.api.internal.file.FileSystemSubset;
 import org.gradle.api.internal.project.ProjectInternal;
-import org.gradle.api.internal.tasks.TaskDependencies;
+import org.gradle.api.internal.tasks.AbstractTaskDependency;
+import org.gradle.api.internal.tasks.TaskDependencyResolveContext;
 import org.gradle.api.specs.Spec;
 import org.gradle.api.specs.Specs;
 import org.gradle.api.tasks.TaskDependency;
 import org.gradle.initialization.ProjectAccessListener;
 import org.gradle.internal.Cast;
+import org.gradle.internal.Factories;
+import org.gradle.internal.Factory;
+import org.gradle.internal.ImmutableActionSet;
 import org.gradle.internal.component.local.model.DefaultLocalComponentMetadata;
 import org.gradle.internal.component.model.ComponentResolveMetadata;
 import org.gradle.internal.event.ListenerBroadcast;
@@ -88,13 +97,14 @@ import org.gradle.util.WrapUtil;
 import java.io.File;
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.Iterator;
 import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
 import static org.gradle.api.internal.artifacts.configurations.ConfigurationInternal.InternalState.*;
-import static org.gradle.internal.Cast.uncheckedCast;
+import static org.gradle.util.ConfigureUtil.configure;
 
 public class DefaultConfiguration extends AbstractFileCollection implements ConfigurationInternal, MutationValidator {
 
@@ -104,7 +114,7 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
     private final DefaultDependencySet dependencies;
     private final CompositeDomainObjectSet<Dependency> inheritedDependencies;
     private final DefaultDependencySet allDependencies;
-    private final List<Action<? super DependencySet>> defaultDependencyActions = new ArrayList<Action<? super DependencySet>>();
+    private ImmutableActionSet<DependencySet> defaultDependencyActions = ImmutableActionSet.empty();
     private final DefaultPublishArtifactSet artifacts;
     private final CompositeDomainObjectSet<PublishArtifact> inheritedArtifacts;
     private final DefaultPublishArtifactSet allArtifacts;
@@ -115,7 +125,8 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
     private final NotationParser<Object, ConfigurablePublishArtifact> artifactNotationParser;
     private final ProjectAccessListener projectAccessListener;
     private final ProjectFinder projectFinder;
-    private final ResolutionStrategyInternal resolutionStrategy;
+    private Factory<ResolutionStrategyInternal> resolutionStrategyFactory;
+    private ResolutionStrategyInternal resolutionStrategy;
     private final ConfigurationComponentMetaDataBuilder configurationComponentMetaDataBuilder;
     private final FileCollectionFactory fileCollectionFactory;
     private final ComponentIdentifierFactory componentIdentifierFactory;
@@ -127,6 +138,7 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
             DefaultConfiguration.this.validateParentMutation(type);
         }
     };
+    private final ImmutableModuleIdentifierFactory moduleIdentifierFactory;
 
     private final Path identityPath;
     // These fields are not covered by mutation lock
@@ -151,14 +163,16 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
     private boolean dependenciesModified;
     private boolean canBeConsumed = true;
     private boolean canBeResolved = true;
-    private final DefaultAttributeContainer configurationAttributes = new DefaultAttributeContainer();
+    private AttributeContainerInternal configurationAttributes;
+    private final ImmutableAttributesFactory attributesFactory;
+    private final FileCollection intrinsicFiles = new ConfigurationFileCollection(Specs.<Dependency>satisfyAll());
 
     public DefaultConfiguration(Path identityPath, Path path, String name,
                                 ConfigurationsProvider configurationsProvider,
                                 ConfigurationResolver resolver,
                                 ListenerManager listenerManager,
                                 DependencyMetaDataProvider metaDataProvider,
-                                ResolutionStrategyInternal resolutionStrategy,
+                                Factory<ResolutionStrategyInternal> resolutionStrategyFactory,
                                 ProjectAccessListener projectAccessListener,
                                 ProjectFinder projectFinder,
                                 ConfigurationComponentMetaDataBuilder configurationComponentMetaDataBuilder,
@@ -166,7 +180,9 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
                                 ComponentIdentifierFactory componentIdentifierFactory,
                                 BuildOperationExecutor buildOperationExecutor,
                                 Instantiator instantiator,
-                                NotationParser<Object, ConfigurablePublishArtifact> artifactNotationParser) {
+                                NotationParser<Object, ConfigurablePublishArtifact> artifactNotationParser,
+                                ImmutableAttributesFactory attributesFactory,
+                                ImmutableModuleIdentifierFactory moduleIdentifierFactory) {
         this.identityPath = identityPath;
         this.path = path;
         this.name = name;
@@ -174,7 +190,7 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
         this.resolver = resolver;
         this.listenerManager = listenerManager;
         this.metaDataProvider = metaDataProvider;
-        this.resolutionStrategy = resolutionStrategy;
+        this.resolutionStrategyFactory = resolutionStrategyFactory;
         this.projectAccessListener = projectAccessListener;
         this.projectFinder = projectFinder;
         this.configurationComponentMetaDataBuilder = configurationComponentMetaDataBuilder;
@@ -185,6 +201,9 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
         this.buildOperationExecutor = buildOperationExecutor;
         this.instantiator = instantiator;
         this.artifactNotationParser = artifactNotationParser;
+        this.moduleIdentifierFactory = moduleIdentifierFactory;
+        this.attributesFactory = attributesFactory;
+        this.configurationAttributes = new DefaultMutableAttributeContainer(attributesFactory);
 
         DefaultDomainObjectSet<Dependency> ownDependencies = new DefaultDomainObjectSet<Dependency>(Dependency.class);
         ownDependencies.beforeChange(validateMutationType(this, MutationType.DEPENDENCIES));
@@ -201,14 +220,13 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
         inheritedArtifacts = CompositeDomainObjectSet.create(PublishArtifact.class, ownArtifacts);
         allArtifacts = new DefaultPublishArtifactSet(displayName + " all artifacts", inheritedArtifacts, fileCollectionFactory);
 
-        resolutionStrategy.setMutationValidator(this);
-        outgoing = instantiator.newInstance(DefaultConfigurationPublications.class, artifacts, configurationAttributes, instantiator, artifactNotationParser, fileCollectionFactory);
+        outgoing = instantiator.newInstance(DefaultConfigurationPublications.class, artifacts, allArtifacts, configurationAttributes, instantiator, artifactNotationParser, fileCollectionFactory, attributesFactory);
     }
 
-    private static Runnable validateMutationType(final MutationValidator mutationValidator, final MutationType type) {
-        return new Runnable() {
+    private static Action<Void> validateMutationType(final MutationValidator mutationValidator, final MutationType type) {
+        return new Action<Void>() {
             @Override
-            public void run() {
+            public void execute(Void arg) {
                 mutationValidator.validateMutation(type);
             }
         };
@@ -321,24 +339,26 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
     }
 
     @Override
-    public Configuration defaultDependencies(Action<? super DependencySet> action) {
+    public Configuration defaultDependencies(final Action<? super DependencySet> action) {
         validateMutation(MutationType.DEPENDENCIES);
-        this.defaultDependencyActions.add(action);
+        defaultDependencyActions = defaultDependencyActions.add(new Action<DependencySet>() {
+            @Override
+            public void execute(DependencySet dependencies) {
+                if (dependencies.isEmpty()) {
+                    action.execute(dependencies);
+                }
+            }
+        });
         return this;
     }
 
     @Override
     public void triggerWhenEmptyActionsIfNecessary() {
-        if (!defaultDependencyActions.isEmpty()) {
-            for (Action<? super DependencySet> action : defaultDependencyActions) {
-                if (!dependencies.isEmpty()) {
-                    break;
-                }
-                action.execute(dependencies);
-            }
+        if (dependencies.isEmpty()) {
+            defaultDependencyActions.execute(dependencies);
         }
         // Discard actions
-        defaultDependencyActions.clear();
+        defaultDependencyActions = ImmutableActionSet.empty();
         for (Configuration superConfig : extendsFrom) {
             ((ConfigurationInternal) superConfig).triggerWhenEmptyActionsIfNecessary();
         }
@@ -353,7 +373,7 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
     }
 
     public Set<File> getFiles() {
-        return doGetFiles(Specs.<Dependency>satisfyAll(), configurationAttributes);
+        return intrinsicFiles.getFiles();
     }
 
     public Set<File> files(Dependency... dependencies) {
@@ -432,13 +452,13 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
         if (resolvedState != UNRESOLVED) {
             throw new IllegalStateException("Graph resolution already performed");
         }
-
         buildOperationExecutor.run("Resolve dependencies " + identityPath, new Action<BuildOperationContext>() {
             @Override
             public void execute(BuildOperationContext buildOperationContext) {
+                lockAttributes();
+
                 ResolvableDependencies incoming = getIncoming();
                 performPreResolveActions(incoming);
-
                 cachedResolverResults = new DefaultResolverResults();
                 resolver.resolveGraph(DefaultConfiguration.this, cachedResolverResults);
                 dependenciesModified = false;
@@ -483,41 +503,13 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
         if (resolvedState != GRAPH_RESOLVED) {
             throw new IllegalStateException("Cannot resolve artifacts before graph has been resolved.");
         }
-        resolver.resolveArtifacts(this, cachedResolverResults);
+        resolver.resolveArtifacts(DefaultConfiguration.this, cachedResolverResults);
         resolvedState = ARTIFACTS_RESOLVED;
     }
 
     public TaskDependency getBuildDependencies() {
         assertResolvingAllowed();
-        return doGetTaskDependency(Specs.<Dependency>satisfyAll(), configurationAttributes);
-    }
-
-    private TaskDependency doGetTaskDependency(Spec<? super Dependency> dependencySpec, AttributeContainerInternal requestedAttributes) {
-        synchronized (resolutionLock) {
-            if (resolutionStrategy.resolveGraphToDetermineTaskDependencies()) {
-                // Force graph resolution as this is required to calculate build dependencies
-                resolveToStateOrLater(GRAPH_RESOLVED);
-            }
-            ResolverResults results;
-            if (getState() == State.UNRESOLVED) {
-                // Traverse graph
-                results = new DefaultResolverResults();
-                resolver.resolveBuildDependencies(this, results);
-            } else {
-                // Otherwise, already have a result, so reuse it
-                results = cachedResolverResults;
-            }
-            List<Object> buildDependencies = new ArrayList<Object>();
-            results.getVisitedArtifacts().select(dependencySpec, requestedAttributes).collectBuildDependencies(buildDependencies);
-            return TaskDependencies.of(buildDependencies);
-        }
-    }
-
-    private Set<File> doGetFiles(Spec<? super Dependency> dependencySpec, AttributeContainerInternal requestedAttributes) {
-        synchronized (resolutionLock) {
-            resolveToStateOrLater(ARTIFACTS_RESOLVED);
-            return cachedResolverResults.getVisitedArtifacts().select(dependencySpec, requestedAttributes).collectFiles(new LinkedHashSet<File>());
-        }
+        return intrinsicFiles.getBuildDependencies();
     }
 
     /**
@@ -588,6 +580,12 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
         return outgoing.convertToOutgoingVariant();
     }
 
+    @Override
+    public void lockAttributes() {
+        AttributeContainerInternal delegatee = configurationAttributes.asImmutable();
+        configurationAttributes = new AttributeContainerWithErrorMessage(delegatee);
+    }
+
     @Override
     public void outgoing(Action<? super ConfigurationPublications> action) {
         action.execute(outgoing);
@@ -614,8 +612,9 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
         String newName = name + "Copy";
         Path newIdentityPath = identityPath.getParent().child(newName);
         Path newPath = path.getParent().child(newName);
+        Factory<ResolutionStrategyInternal> childResolutionStrategy = resolutionStrategy != null ? Factories.constant(resolutionStrategy.copy()) : resolutionStrategyFactory;
         DefaultConfiguration copiedConfiguration = instantiator.newInstance(DefaultConfiguration.class, newIdentityPath, newPath, newName,
-            configurationsProvider, resolver, listenerManager, metaDataProvider, resolutionStrategy.copy(), projectAccessListener, projectFinder, configurationComponentMetaDataBuilder, fileCollectionFactory, componentIdentifierFactory, buildOperationExecutor, instantiator, artifactNotationParser);
+            configurationsProvider, resolver, listenerManager, metaDataProvider, childResolutionStrategy, projectAccessListener, projectFinder, configurationComponentMetaDataBuilder, fileCollectionFactory, componentIdentifierFactory, buildOperationExecutor, instantiator, artifactNotationParser, attributesFactory, moduleIdentifierFactory);
         configurationsProvider.setTheOnlyConfiguration(copiedConfiguration);
         // state, cachedResolvedConfiguration, and extendsFrom intentionally not copied - must re-resolve copy
         // copying extendsFrom could mess up dependencies when copy was re-resolved
@@ -624,17 +623,17 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
         copiedConfiguration.transitive = transitive;
         copiedConfiguration.description = description;
 
-        copiedConfiguration.defaultDependencyActions.addAll(defaultDependencyActions);
+        copiedConfiguration.defaultDependencyActions = defaultDependencyActions;
 
         copiedConfiguration.canBeConsumed = canBeConsumed;
         copiedConfiguration.canBeResolved = canBeResolved;
 
         copiedConfiguration.getArtifacts().addAll(getAllArtifacts());
 
-        if (hasAttributes()) {
+        if (!configurationAttributes.isEmpty()) {
             for (Attribute<?> attribute : configurationAttributes.keySet()) {
                 Object value = configurationAttributes.getAttribute(attribute);
-                copiedConfiguration.attribute(Cast.<Attribute<Object>>uncheckedCast(attribute), value);
+                copiedConfiguration.getAttributes().attribute(Cast.<Attribute<Object>>uncheckedCast(attribute), value);
             }
         }
 
@@ -669,15 +668,20 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
     }
 
     public ResolutionStrategyInternal getResolutionStrategy() {
+        if (resolutionStrategy == null) {
+            resolutionStrategy = resolutionStrategyFactory.create();
+            resolutionStrategy.setMutationValidator(this);
+            resolutionStrategyFactory = null;
+        }
         return resolutionStrategy;
     }
 
     public ComponentResolveMetadata toRootComponentMetaData() {
         Module module = getModule();
         ComponentIdentifier componentIdentifier = componentIdentifierFactory.createComponentIdentifier(module);
-        ModuleVersionIdentifier moduleVersionIdentifier = DefaultModuleVersionIdentifier.newId(module);
+        ModuleVersionIdentifier moduleVersionIdentifier = moduleIdentifierFactory.moduleWithVersion(module.getGroup(), module.getName(), module.getVersion());
         ProjectInternal project = projectFinder.findProject(module.getProjectPath());
-        AttributesSchema schema = project == null ? null : project.getAttributesSchema();
+        AttributesSchemaInternal schema = project == null ? null : (AttributesSchemaInternal) project.getDependencies().getAttributesSchema();
         DefaultLocalComponentMetadata metaData = new DefaultLocalComponentMetadata(moduleVersionIdentifier, componentIdentifier, module.getStatus(), schema);
         configurationComponentMetaDataBuilder.addConfigurations(metaData, configurationsProvider.getAll());
         return metaData;
@@ -689,12 +693,13 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
 
     @Override
     public Configuration resolutionStrategy(Closure closure) {
-        return resolutionStrategy(ClosureBackedAction.of(closure));
+        configure(closure, getResolutionStrategy());
+        return this;
     }
 
     @Override
     public Configuration resolutionStrategy(Action<? super ResolutionStrategy> action) {
-        action.execute(resolutionStrategy);
+        action.execute(getResolutionStrategy());
         return this;
     }
 
@@ -758,17 +763,21 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
     private class ConfigurationFileCollection extends AbstractFileCollection {
         private final Spec<? super Dependency> dependencySpec;
         private final AttributeContainerInternal viewAttributes;
+        private final Spec<? super ComponentIdentifier> componentSpec;
+        private SelectedArtifactSet selectedArtifacts;
 
         private ConfigurationFileCollection(Spec<? super Dependency> dependencySpec) {
             assertResolvingAllowed();
             this.dependencySpec = dependencySpec;
-            this.viewAttributes = DefaultConfiguration.this.configurationAttributes.asImmutable();
+            this.viewAttributes = ImmutableAttributes.EMPTY;
+            this.componentSpec = Specs.satisfyAll();
         }
 
-        private ConfigurationFileCollection(Spec<? super Dependency> dependencySpec, AttributeContainerInternal viewAttributes) {
-            assertResolvingAllowed();
+        private ConfigurationFileCollection(Spec<? super Dependency> dependencySpec, AttributeContainerInternal viewAttributes,
+                                            Spec<? super ComponentIdentifier> componentSpec) {
             this.dependencySpec = dependencySpec;
             this.viewAttributes = viewAttributes.asImmutable();
+            this.componentSpec = componentSpec;
         }
 
         private ConfigurationFileCollection(Closure dependencySpecClosure) {
@@ -785,7 +794,8 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
 
         @Override
         public TaskDependency getBuildDependencies() {
-            return doGetTaskDependency(dependencySpec, viewAttributes);
+            assertResolvingAllowed();
+            return new ConfigurationTaskDependency(dependencySpec, viewAttributes, componentSpec);
         }
 
         public Spec<? super Dependency> getDependencySpec() {
@@ -797,7 +807,16 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
         }
 
         public Set<File> getFiles() {
-            return doGetFiles(dependencySpec, viewAttributes);
+            return getSelectedArtifacts().collectFiles(new LinkedHashSet<File>());
+        }
+
+        private SelectedArtifactSet getSelectedArtifacts() {
+            assertResolvingAllowed();
+            if (selectedArtifacts == null) {
+                resolveToStateOrLater(ARTIFACTS_RESOLVED);
+                selectedArtifacts = cachedResolverResults.getVisitedArtifacts().select(dependencySpec, viewAttributes, componentSpec);
+            }
+            return selectedArtifacts;
         }
     }
 
@@ -818,57 +837,17 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
         super.registerWatchPoints(builder);
     }
 
-    @Override
-    public Configuration attribute(String key, String value) {
-        attribute(stringAttribute(key), value);
-        return this;
-    }
-
-    @Override
-    public <T> Configuration attribute(Attribute<T> key, T value) {
-        validateMutation(MutationType.ATTRIBUTES);
-        configurationAttributes.attribute(key, value);
-        return this;
-    }
-
     @Override
     public AttributeContainerInternal getAttributes() {
         return configurationAttributes;
     }
 
     @Override
-    public <T> T getAttribute(Attribute<T> key) {
-        return configurationAttributes.getAttribute(key);
-    }
-
-    @Override
-    public Configuration attributes(Map<?, ?> attributes) {
-        validateMutation(MutationType.ATTRIBUTES);
-        populateAttributesFromMap(attributes, configurationAttributes);
+    public Configuration attributes(Action<? super AttributeContainer> action) {
+        action.execute(configurationAttributes);
         return this;
     }
 
-    private void populateAttributesFromMap(Map<?, ?> attributes, AttributeContainer attributeContainer) {
-        for (Map.Entry<?, ?> entry : attributes.entrySet()) {
-            Object rawKey = entry.getKey();
-            Attribute<Object> key = uncheckedCast(asAttribute(rawKey));
-            Object value = entry.getValue();
-            attributeContainer.attribute(key, value);
-        }
-    }
-
-    private static Attribute<?> asAttribute(Object rawKey) {
-        if (rawKey instanceof Attribute) {
-            return (Attribute<?>) rawKey;
-        }
-        return stringAttribute(rawKey.toString());
-    }
-
-    @Override
-    public boolean hasAttributes() {
-        return !configurationAttributes.isEmpty();
-    }
-
     @Override
     public boolean isCanBeConsumed() {
         return canBeConsumed;
@@ -960,12 +939,6 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
             return new ConfigurationFileCollection(Specs.<Dependency>satisfyAll());
         }
 
-        public FileCollection getFiles(Map<?, ?> attributeMap) {
-            AttributeContainerInternal attributes = configurationAttributes.copy();
-            populateAttributesFromMap(attributeMap, attributes);
-            return new ConfigurationFileCollection(Specs.<Dependency>satisfyAll(), attributes);
-        }
-
         public DependencySet getDependencies() {
             return getAllDependencies();
         }
@@ -991,14 +964,193 @@ public class DefaultConfiguration extends AbstractFileCollection implements Conf
             return DefaultConfiguration.this.cachedResolverResults.getResolutionResult();
         }
 
-        public Set<ResolvedArtifactResult> getArtifacts() {
-            resolveToStateOrLater(ARTIFACTS_RESOLVED);
-            return cachedResolverResults.getVisitedArtifacts().select(Specs.<Dependency>satisfyAll(), configurationAttributes).collectArtifacts(new LinkedHashSet<ResolvedArtifactResult>());
+        @Override
+        public ArtifactCollection getArtifacts() {
+            return new ConfigurationArtifactCollection();
+        }
+
+        public ArtifactView artifactView() {
+            return new ConfigurationViewBuilder();
+        }
+
+        private class ConfigurationViewBuilder implements ArtifactView {
+            private AttributeContainerInternal viewAttributes;
+            private Spec<? super ComponentIdentifier> componentFilter;
+
+            @Override
+            public AttributeContainer getAttributes() {
+                if (viewAttributes == null) {
+                    viewAttributes = new DefaultMutableAttributeContainer(attributesFactory);
+                }
+                return viewAttributes;
+            }
+
+            @Override
+            public ArtifactView attributes(Action<? super AttributeContainer> action) {
+                action.execute(getAttributes());
+                return this;
+            }
+
+            @Override
+            public ArtifactView componentFilter(Spec<? super ComponentIdentifier> componentFilter) {
+                assertComponentFilterUnset();
+                this.componentFilter = componentFilter;
+                return this;
+            }
+
+            private void assertComponentFilterUnset() {
+                if (componentFilter != null) {
+                    throw new IllegalStateException("The component filter can only be set once before the view was computed");
+                }
+            }
+
+            @Override
+            public ArtifactCollection getArtifacts() {
+                return new ConfigurationArtifactCollection(lockViewAttributes(), lockComponentFilter());
+            }
+
+            @Override
+            public FileCollection getFiles() {
+                return new ConfigurationFileCollection(Specs.<Dependency>satisfyAll(), lockViewAttributes(), lockComponentFilter());
+            }
+
+            private Spec<? super ComponentIdentifier> lockComponentFilter() {
+                if (componentFilter == null) {
+                    componentFilter = Specs.satisfyAll();
+                }
+                return componentFilter;
+            }
+
+            private AttributeContainerInternal lockViewAttributes() {
+                if (viewAttributes == null) {
+                    viewAttributes = ImmutableAttributes.EMPTY;
+                } else {
+                    viewAttributes = viewAttributes.asImmutable();
+                }
+                return viewAttributes;
+            }
         }
     }
 
-    private static Attribute<String> stringAttribute(String name) {
-        return Attribute.of(name, String.class);
+    private class ConfigurationArtifactCollection implements ArtifactCollection {
+        private final ConfigurationFileCollection fileCollection;
+        private final AttributeContainerInternal viewAttributes;
+        private final Spec<? super ComponentIdentifier> componentFilter;
+
+        ConfigurationArtifactCollection() {
+            this(ImmutableAttributes.EMPTY, Specs.<ComponentIdentifier>satisfyAll());
+        }
+
+        ConfigurationArtifactCollection(AttributeContainerInternal attributes, Spec<? super ComponentIdentifier> componentFilter) {
+            assertResolvingAllowed();
+            this.viewAttributes = attributes.asImmutable();
+            this.componentFilter = componentFilter;
+            this.fileCollection = new ConfigurationFileCollection(Specs.<Dependency>satisfyAll(), viewAttributes, this.componentFilter);
+        }
+
+        @Override
+        public FileCollection getArtifactFiles() {
+            return fileCollection;
+        }
+
+        @Override
+        public Set<ResolvedArtifactResult> getArtifacts() {
+            return fileCollection.getSelectedArtifacts().collectArtifacts(new LinkedHashSet<ResolvedArtifactResult>());
+        }
+
+        @Override
+        public Iterator<ResolvedArtifactResult> iterator() {
+            return getArtifacts().iterator();
+        }
     }
 
+    private class ConfigurationTaskDependency extends AbstractTaskDependency {
+        private final Spec<? super Dependency> dependencySpec;
+        private final AttributeContainerInternal requestedAttributes;
+        private final Spec<? super ComponentIdentifier> componentIdentifierSpec;
+
+        ConfigurationTaskDependency(Spec<? super Dependency> dependencySpec, AttributeContainerInternal requestedAttributes, Spec<? super ComponentIdentifier> componentIdentifierSpec) {
+            this.dependencySpec = dependencySpec;
+            this.requestedAttributes = requestedAttributes;
+            this.componentIdentifierSpec = componentIdentifierSpec;
+        }
+
+        @Override
+        public void visitDependencies(TaskDependencyResolveContext context) {
+            synchronized (resolutionLock) {
+                if (getResolutionStrategy().resolveGraphToDetermineTaskDependencies()) {
+                    // Force graph resolution as this is required to calculate build dependencies
+                    resolveToStateOrLater(GRAPH_RESOLVED);
+                }
+                ResolverResults results;
+                if (getState() == State.UNRESOLVED) {
+                    // Traverse graph
+                    results = new DefaultResolverResults();
+                    resolver.resolveBuildDependencies(DefaultConfiguration.this, results);
+                } else {
+                    // Otherwise, already have a result, so reuse it
+                    results = cachedResolverResults;
+                }
+                List<Object> buildDependencies = new ArrayList<Object>();
+                results.getVisitedArtifacts().select(dependencySpec, requestedAttributes, componentIdentifierSpec).collectBuildDependencies(buildDependencies);
+                for (Object buildDependency : buildDependencies) {
+                    context.add(buildDependency);
+                }
+            }
+        }
+    }
+
+    private class AttributeContainerWithErrorMessage implements AttributeContainerInternal {
+        private final AttributeContainerInternal delegate;
+
+        AttributeContainerWithErrorMessage(AttributeContainerInternal delegate) {
+            this.delegate = delegate;
+        }
+
+        @Override
+        public String toString() {
+            return delegate.toString();
+        }
+
+        @Override
+        public ImmutableAttributes asImmutable() {
+            return delegate.asImmutable();
+        }
+
+        @Override
+        public AttributeContainerInternal copy() {
+            return delegate.copy();
+        }
+
+        @Override
+        public Set<Attribute<?>> keySet() {
+            return delegate.keySet();
+        }
+
+        @Override
+        public <T> AttributeContainer attribute(Attribute<T> key, T value) {
+            throw new IllegalArgumentException(String.format("Cannot change attributes of %s after it has been resolved", getDisplayName()));
+        }
+
+        @Nullable
+        @Override
+        public <T> T getAttribute(Attribute<T> key) {
+            return delegate.getAttribute(key);
+        }
+
+        @Override
+        public boolean isEmpty() {
+            return delegate.isEmpty();
+        }
+
+        @Override
+        public boolean contains(Attribute<?> key) {
+            return delegate.contains(key);
+        }
+
+        @Override
+        public AttributeContainer getAttributes() {
+            return delegate.getAttributes();
+        }
+    }
 }
