@@ -17,7 +17,6 @@
 package org.gradle.api.internal.artifacts.dsl;
 
 import groovy.lang.Closure;
-import groovy.lang.MissingMethodException;
 import org.gradle.api.Action;
 import org.gradle.api.artifacts.ConfigurablePublishArtifact;
 import org.gradle.api.artifacts.Configuration;
@@ -25,6 +24,9 @@ import org.gradle.api.artifacts.ConfigurationContainer;
 import org.gradle.api.artifacts.PublishArtifact;
 import org.gradle.api.artifacts.dsl.ArtifactHandler;
 import org.gradle.internal.Actions;
+import org.gradle.internal.metaobject.InvokeMethodResult;
+import org.gradle.internal.metaobject.MethodAccess;
+import org.gradle.internal.metaobject.MethodMixIn;
 import org.gradle.internal.typeconversion.NotationParser;
 import org.gradle.util.ConfigureUtil;
 import org.gradle.util.GUtil;
@@ -32,14 +34,16 @@ import org.gradle.util.GUtil;
 import java.util.Arrays;
 import java.util.List;
 
-public class DefaultArtifactHandler implements ArtifactHandler {
+public class DefaultArtifactHandler implements ArtifactHandler, MethodMixIn {
 
     private final ConfigurationContainer configurationContainer;
     private final NotationParser<Object, ConfigurablePublishArtifact> publishArtifactFactory;
+    private final DynamicMethods dynamicMethods;
 
     public DefaultArtifactHandler(ConfigurationContainer configurationContainer, NotationParser<Object, ConfigurablePublishArtifact> publishArtifactFactory) {
         this.configurationContainer = configurationContainer;
         this.publishArtifactFactory = publishArtifactFactory;
+        dynamicMethods = new DynamicMethods();
     }
 
     private PublishArtifact pushArtifact(org.gradle.api.artifacts.Configuration configuration, Object notation, Closure configureClosure) {
@@ -69,19 +73,35 @@ public class DefaultArtifactHandler implements ArtifactHandler {
         return pushArtifact(configurationContainer.getByName(configurationName), artifactNotation, configureClosure);
     }
 
-    public Object methodMissing(String name, Object arg) {
-        Object[] args = (Object[]) arg;
-        Configuration configuration = configurationContainer.findByName(name);
-        if (configuration == null) {
-            throw new MissingMethodException(name, this.getClass(), args);
+    @Override
+    public MethodAccess getAdditionalMethods() {
+        return dynamicMethods;
+    }
+
+    private class DynamicMethods implements MethodAccess {
+        @Override
+        public boolean hasMethod(String name, Object... arguments) {
+            return arguments.length > 0 && configurationContainer.findByName(name) != null;
         }
-        List<Object> normalizedArgs = GUtil.flatten(Arrays.asList(args), false);
-        if (normalizedArgs.size() == 2 && normalizedArgs.get(1) instanceof Closure) {
-            return pushArtifact(configuration, normalizedArgs.get(0), (Closure) normalizedArgs.get(1));
+
+        @Override
+        public void invokeMethod(String name, InvokeMethodResult result, Object... arguments) {
+            if (arguments.length == 0) {
+                return;
+            }
+            Configuration configuration = configurationContainer.findByName(name);
+            if (configuration == null) {
+                return;
+            }
+            List<Object> normalizedArgs = GUtil.flatten(Arrays.asList(arguments), false);
+            if (normalizedArgs.size() == 2 && normalizedArgs.get(1) instanceof Closure) {
+                result.result(pushArtifact(configuration, normalizedArgs.get(0), (Closure) normalizedArgs.get(1)));
+            } else {
+                for (Object notation : normalizedArgs) {
+                    pushArtifact(configuration, notation, Actions.doNothing());
+                }
+                result.result(null);
+            }
         }
-        for (Object notation : args) {
-            pushArtifact(configuration, notation, Actions.doNothing());
-        }
-        return null;
     }
 }
