@@ -16,7 +16,6 @@
 package org.gradle.api.internal.artifacts.ivyservice;
 
 import com.google.common.collect.Sets;
-import org.gradle.api.Nullable;
 import org.gradle.api.Transformer;
 import org.gradle.api.artifacts.Configuration;
 import org.gradle.api.artifacts.Dependency;
@@ -31,12 +30,13 @@ import org.gradle.api.artifacts.component.ComponentArtifactIdentifier;
 import org.gradle.api.artifacts.component.ComponentIdentifier;
 import org.gradle.api.artifacts.component.ModuleComponentIdentifier;
 import org.gradle.api.artifacts.result.ResolvedArtifactResult;
-import org.gradle.api.attributes.HasAttributes;
+import org.gradle.api.attributes.AttributeContainer;
 import org.gradle.api.component.Artifact;
 import org.gradle.api.internal.artifacts.DependencyGraphNodeResult;
 import org.gradle.api.internal.artifacts.configurations.ConfigurationInternal;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.ArtifactVisitor;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.ResolvedArtifactSet;
+import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.ResolvedVariant;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.SelectedArtifactResults;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.SelectedArtifactSet;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.SelectedFileDependencyResults;
@@ -46,13 +46,12 @@ import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.Visit
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.oldresult.TransientConfigurationResults;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.oldresult.TransientConfigurationResultsLoader;
 import org.gradle.api.internal.artifacts.result.DefaultResolvedArtifactResult;
-import org.gradle.api.internal.artifacts.transform.ArtifactTransformer;
+import org.gradle.api.internal.artifacts.transform.ArtifactTransforms;
 import org.gradle.api.internal.attributes.AttributeContainerInternal;
+import org.gradle.api.internal.attributes.ImmutableAttributes;
 import org.gradle.api.specs.Spec;
 import org.gradle.api.specs.Specs;
 import org.gradle.internal.Factory;
-import org.gradle.internal.component.local.model.ComponentFileArtifactIdentifier;
-import org.gradle.internal.component.local.model.OpaqueComponentArtifactIdentifier;
 import org.gradle.internal.graph.CachingDirectedGraphWalker;
 import org.gradle.internal.graph.DirectedGraphWithEdgeValues;
 import org.gradle.util.CollectionUtils;
@@ -75,39 +74,63 @@ public class DefaultLenientConfiguration implements LenientConfiguration, Visite
     private final VisitedArtifactsResults artifactResults;
     private final VisitedFileDependencyResults fileDependencyResults;
     private final TransientConfigurationResultsLoader transientConfigurationResultsFactory;
-    private final ArtifactTransformer artifactTransformer;
-    // Selected for the configuration
-    private final SelectedArtifactResults selectedArtifacts;
-    private final SelectedFileDependencyResults selectedFileDependencies;
+    private final ArtifactTransforms artifactTransforms;
 
-    public DefaultLenientConfiguration(ConfigurationInternal configuration, CacheLockingManager cacheLockingManager, Set<UnresolvedDependency> unresolvedDependencies, VisitedArtifactsResults artifactResults, VisitedFileDependencyResults fileDependencyResults, TransientConfigurationResultsLoader transientConfigurationResultsLoader, ArtifactTransformer artifactTransformer) {
+    // Selected for the configuration
+    private SelectedArtifactResults artifactsForThisConfiguration;
+    private SelectedFileDependencyResults filesForThisConfiguration;
+
+    public DefaultLenientConfiguration(ConfigurationInternal configuration, CacheLockingManager cacheLockingManager, Set<UnresolvedDependency> unresolvedDependencies, VisitedArtifactsResults artifactResults, VisitedFileDependencyResults fileDependencyResults, TransientConfigurationResultsLoader transientConfigurationResultsLoader, ArtifactTransforms artifactTransforms) {
         this.configuration = configuration;
         this.cacheLockingManager = cacheLockingManager;
         this.unresolvedDependencies = unresolvedDependencies;
         this.artifactResults = artifactResults;
         this.fileDependencyResults = fileDependencyResults;
         this.transientConfigurationResultsFactory = transientConfigurationResultsLoader;
-        this.artifactTransformer = artifactTransformer;
-        selectedArtifacts = artifactResults.select(artifactTransformer.variantSelector(configuration.getAttributes()));
-        selectedFileDependencies = fileDependencyResults.select(artifactTransformer.variantSelector(configuration.getAttributes()));
+        this.artifactTransforms = artifactTransforms;
+    }
+
+    private SelectedArtifactResults getSelectedArtifacts() {
+        if (artifactsForThisConfiguration == null) {
+            artifactsForThisConfiguration = artifactResults.select(Specs.<ComponentIdentifier>satisfyAll(), artifactTransforms.variantSelector(ImmutableAttributes.EMPTY));
+        }
+        return artifactsForThisConfiguration;
+    }
+
+    private SelectedFileDependencyResults getSelectedFiles() {
+        if (filesForThisConfiguration == null) {
+            filesForThisConfiguration = fileDependencyResults.select(Specs.<ComponentIdentifier>satisfyAll(), artifactTransforms.variantSelector(ImmutableAttributes.EMPTY));
+        }
+        return filesForThisConfiguration;
+    }
+
+    public SelectedArtifactSet select() {
+        return select(Specs.<Dependency>satisfyAll(), ImmutableAttributes.EMPTY, Specs.<ComponentIdentifier>satisfyAll());
+    }
+
+    public SelectedArtifactSet select(final Spec<? super Dependency> dependencySpec) {
+        return select(dependencySpec, ImmutableAttributes.EMPTY, Specs.<ComponentIdentifier>satisfyAll());
     }
 
     @Override
-    public SelectedArtifactSet select(final Spec<? super Dependency> dependencySpec, final AttributeContainerInternal requestedAttributes) {
-        Transformer<HasAttributes, Collection<? extends HasAttributes>> selector = artifactTransformer.variantSelector(requestedAttributes);
-        final SelectedArtifactResults artifactResults = this.artifactResults.select(selector);
-        final SelectedFileDependencyResults fileDependencyResults = this.fileDependencyResults.select(selector);
+    public SelectedArtifactSet select(final Spec<? super Dependency> dependencySpec, final AttributeContainerInternal requestedAttributes, final Spec<? super ComponentIdentifier> componentSpec) {
+        final SelectedArtifactResults artifactResults;
+        final SelectedFileDependencyResults fileDependencyResults;
+        Transformer<ResolvedArtifactSet, Collection<? extends ResolvedVariant>> selector = artifactTransforms.variantSelector(requestedAttributes);
+        artifactResults = this.artifactResults.select(componentSpec, selector);
+        fileDependencyResults = this.fileDependencyResults.select(componentSpec, selector);
+
         return new SelectedArtifactSet() {
             @Override
             public <T extends Collection<Object>> T collectBuildDependencies(T dest) {
                 artifactResults.getArtifacts().collectBuildDependencies(dest);
-                fileDependencyResults.getFiles().collectBuildDependencies(dest);
+                fileDependencyResults.getArtifacts().collectBuildDependencies(dest);
                 return dest;
             }
 
             @Override
             public void visitArtifacts(ArtifactVisitor visitor) {
-                DefaultLenientConfiguration.this.visitArtifacts(dependencySpec, requestedAttributes, artifactResults, fileDependencyResults, visitor);
+                DefaultLenientConfiguration.this.visitArtifacts(dependencySpec, artifactResults, fileDependencyResults, visitor);
             }
 
             /**
@@ -118,7 +141,7 @@ public class DefaultLenientConfiguration implements LenientConfiguration, Visite
                 rethrowFailure();
                 ResolvedFilesCollectingVisitor visitor = new ResolvedFilesCollectingVisitor(dest);
                 try {
-                    DefaultLenientConfiguration.this.visitArtifacts(dependencySpec, requestedAttributes, artifactResults, fileDependencyResults, visitor);
+                    DefaultLenientConfiguration.this.visitArtifacts(dependencySpec, artifactResults, fileDependencyResults, visitor);
                     // The visitor adds file dependencies directly to the destination collection however defers adding the artifacts.
                     // This is to ensure a fixed order regardless of whether the first level dependencies are filtered or not
                     // File dependencies and artifacts are currently treated separately as a migration step
@@ -140,7 +163,7 @@ public class DefaultLenientConfiguration implements LenientConfiguration, Visite
                 rethrowFailure();
                 ResolvedArtifactCollectingVisitor visitor = new ResolvedArtifactCollectingVisitor(dest);
                 try {
-                    DefaultLenientConfiguration.this.visitArtifacts(dependencySpec, requestedAttributes, artifactResults, fileDependencyResults, visitor);
+                    DefaultLenientConfiguration.this.visitArtifacts(dependencySpec, artifactResults, fileDependencyResults, visitor);
                 } catch (Throwable t) {
                     visitor.failures.add(t);
                 }
@@ -184,7 +207,7 @@ public class DefaultLenientConfiguration implements LenientConfiguration, Visite
 
     private Set<DependencyGraphNodeResult> getFirstLevelNodes(Spec<? super Dependency> dependencySpec) {
         Set<DependencyGraphNodeResult> matches = new LinkedHashSet<DependencyGraphNodeResult>();
-        TransientConfigurationResults graphResults = loadTransientGraphResults(selectedArtifacts);
+        TransientConfigurationResults graphResults = loadTransientGraphResults(getSelectedArtifacts());
         for (Map.Entry<ModuleDependency, DependencyGraphNodeResult> entry : graphResults.getFirstLevelDependencies().entrySet()) {
             if (dependencySpec.isSatisfiedBy(entry.getKey())) {
                 matches.add(entry.getValue());
@@ -196,7 +219,7 @@ public class DefaultLenientConfiguration implements LenientConfiguration, Visite
     public Set<ResolvedDependency> getAllModuleDependencies() {
         Set<ResolvedDependency> resolvedElements = new LinkedHashSet<ResolvedDependency>();
         Deque<ResolvedDependency> workQueue = new LinkedList<ResolvedDependency>();
-        workQueue.addAll(loadTransientGraphResults(selectedArtifacts).getRootNode().getPublicView().getChildren());
+        workQueue.addAll(loadTransientGraphResults(getSelectedArtifacts()).getRootNode().getPublicView().getChildren());
         while (!workQueue.isEmpty()) {
             ResolvedDependency item = workQueue.removeFirst();
             if (resolvedElements.add(item)) {
@@ -220,7 +243,7 @@ public class DefaultLenientConfiguration implements LenientConfiguration, Visite
     public Set<File> getFiles(Spec<? super Dependency> dependencySpec) {
         Set<File> files = Sets.newLinkedHashSet();
         FilesAndArtifactCollectingVisitor visitor = new FilesAndArtifactCollectingVisitor(files);
-        visitArtifacts(dependencySpec, configuration.getAttributes(), selectedArtifacts, selectedFileDependencies, visitor);
+        visitArtifacts(dependencySpec, getSelectedArtifacts(), getSelectedFiles(), visitor);
         files.addAll(getFiles(filterUnresolved(visitor.artifacts)));
         return files;
     }
@@ -235,12 +258,12 @@ public class DefaultLenientConfiguration implements LenientConfiguration, Visite
      */
     public Set<ResolvedArtifact> getArtifacts(Spec<? super Dependency> dependencySpec) {
         ArtifactCollectingVisitor visitor = new ArtifactCollectingVisitor();
-        visitArtifacts(dependencySpec, configuration.getAttributes(), selectedArtifacts, selectedFileDependencies, visitor);
+        visitArtifacts(dependencySpec, getSelectedArtifacts(), getSelectedFiles(), visitor);
         return filterUnresolved(visitor.artifacts);
     }
 
     private Set<ResolvedArtifact> filterUnresolved(final Set<ResolvedArtifact> artifacts) {
-        return cacheLockingManager.useCache("retrieve artifacts from " + configuration, new Factory<Set<ResolvedArtifact>>() {
+        return cacheLockingManager.useCache(new Factory<Set<ResolvedArtifact>>() {
             public Set<ResolvedArtifact> create() {
                 return CollectionUtils.filter(artifacts, new IgnoreMissingExternalArtifacts());
             }
@@ -249,7 +272,7 @@ public class DefaultLenientConfiguration implements LenientConfiguration, Visite
 
     private Set<File> getFiles(final Set<ResolvedArtifact> artifacts) {
         final Set<File> files = new LinkedHashSet<File>();
-        cacheLockingManager.useCache("resolve files from " + configuration, new Runnable() {
+        cacheLockingManager.useCache(new Runnable() {
             public void run() {
                 for (ResolvedArtifact artifact : artifacts) {
                     File depFile = artifact.getFile();
@@ -267,32 +290,29 @@ public class DefaultLenientConfiguration implements LenientConfiguration, Visite
      *
      * @param dependencySpec dependency spec
      */
-    private void visitArtifacts(Spec<? super Dependency> dependencySpec, AttributeContainerInternal requestedAttributes, SelectedArtifactResults artifactResults, SelectedFileDependencyResults fileDependencyResults, ArtifactVisitor visitor) {
-        ArtifactVisitor transformingVisitor = artifactTransformer.visitor(visitor, requestedAttributes);
-
+    private void visitArtifacts(Spec<? super Dependency> dependencySpec, SelectedArtifactResults artifactResults, SelectedFileDependencyResults fileDependencyResults, ArtifactVisitor visitor) {
         //this is not very nice might be good enough until we get rid of ResolvedConfiguration and friends
         //avoid traversing the graph causing the full ResolvedDependency graph to be loaded for the most typical scenario
         if (dependencySpec == Specs.SATISFIES_ALL) {
-            if (transformingVisitor.includeFiles()) {
-                fileDependencyResults.getFiles().visit(transformingVisitor);
+            if (visitor.includeFiles()) {
+                fileDependencyResults.getArtifacts().visit(visitor);
             }
-            artifactResults.getArtifacts().visit(transformingVisitor);
+            artifactResults.getArtifacts().visit(visitor);
             return;
         }
 
-        if (transformingVisitor.includeFiles()) {
-            for (Map.Entry<FileCollectionDependency, ResolvedArtifactSet> entry: fileDependencyResults.getFirstLevelFiles().entrySet()) {
+        if (visitor.includeFiles()) {
+            for (Map.Entry<FileCollectionDependency, ResolvedArtifactSet> entry : fileDependencyResults.getFirstLevelFiles().entrySet()) {
                 if (dependencySpec.isSatisfiedBy(entry.getKey())) {
-                    entry.getValue().visit(transformingVisitor);
+                    entry.getValue().visit(visitor);
                 }
             }
         }
 
-        CachingDirectedGraphWalker<DependencyGraphNodeResult, ResolvedArtifact> walker = new CachingDirectedGraphWalker<DependencyGraphNodeResult, ResolvedArtifact>(new ResolvedDependencyArtifactsGraph(transformingVisitor, fileDependencyResults));
-
+        CachingDirectedGraphWalker<DependencyGraphNodeResult, ResolvedArtifact> walker = new CachingDirectedGraphWalker<DependencyGraphNodeResult, ResolvedArtifact>(new ResolvedDependencyArtifactsGraph(visitor, fileDependencyResults));
         DependencyGraphNodeResult rootNode = loadTransientGraphResults(artifactResults).getRootNode();
         for (DependencyGraphNodeResult node : getFirstLevelNodes(dependencySpec)) {
-            node.getArtifactsForIncomingEdge(rootNode).visit(transformingVisitor);
+            node.getArtifactsForIncomingEdge(rootNode).visit(visitor);
             walker.add(node);
         }
         walker.findValues();
@@ -303,7 +323,7 @@ public class DefaultLenientConfiguration implements LenientConfiguration, Visite
     }
 
     public Set<ResolvedDependency> getFirstLevelModuleDependencies() {
-        return loadTransientGraphResults(selectedArtifacts).getRootNode().getPublicView().getChildren();
+        return loadTransientGraphResults(getSelectedArtifacts()).getRootNode().getPublicView().getChildren();
     }
 
     private static class ResolvedFilesCollectingVisitor implements ArtifactVisitor {
@@ -316,25 +336,24 @@ public class DefaultLenientConfiguration implements LenientConfiguration, Visite
         }
 
         @Override
-        public void visitArtifact(ResolvedArtifact artifact) {
+        public void visitArtifact(AttributeContainer variant, ResolvedArtifact artifact) {
             // Defer adding the artifacts until after all the file dependencies have been visited
             this.artifacts.add(artifact);
         }
 
+        @Override
+        public void visitFailure(Throwable failure) {
+            failures.add(failure);
+        }
+
         @Override
         public boolean includeFiles() {
             return true;
         }
 
         @Override
-        public void visitFiles(@Nullable ComponentIdentifier componentIdentifier, Iterable<File> files) {
-            try {
-                for (File file : files) {
-                    this.files.add(file);
-                }
-            } catch (Throwable t) {
-                failures.add(t);
-            }
+        public void visitFile(ComponentArtifactIdentifier artifactIdentifier, AttributeContainer variant, File file) {
+            this.files.add(file);
         }
 
         public void addArtifacts() {
@@ -351,7 +370,6 @@ public class DefaultLenientConfiguration implements LenientConfiguration, Visite
     private static class ResolvedArtifactCollectingVisitor implements ArtifactVisitor {
         private final Collection<? super ResolvedArtifactResult> artifacts;
         private final Set<ComponentArtifactIdentifier> seenArtifacts = new HashSet<ComponentArtifactIdentifier>();
-        private final Set<File> seenFiles = new HashSet<File>();
         private final List<Throwable> failures = new ArrayList<Throwable>();
 
         ResolvedArtifactCollectingVisitor(Collection<? super ResolvedArtifactResult> artifacts) {
@@ -359,12 +377,17 @@ public class DefaultLenientConfiguration implements LenientConfiguration, Visite
         }
 
         @Override
-        public void visitArtifact(ResolvedArtifact artifact) {
+        public void visitFailure(Throwable failure) {
+            failures.add(failure);
+        }
+
+        @Override
+        public void visitArtifact(AttributeContainer variant, ResolvedArtifact artifact) {
             try {
                 if (seenArtifacts.add(artifact.getId())) {
                     // Trigger download of file, if required
                     File file = artifact.getFile();
-                    this.artifacts.add(new DefaultResolvedArtifactResult(artifact.getId(), Artifact.class, file));
+                    this.artifacts.add(new DefaultResolvedArtifactResult(artifact.getId(), variant, Artifact.class, file));
                 }
             } catch (Throwable t) {
                 failures.add(t);
@@ -377,21 +400,9 @@ public class DefaultLenientConfiguration implements LenientConfiguration, Visite
         }
 
         @Override
-        public void visitFiles(@Nullable ComponentIdentifier componentIdentifier, Iterable<File> files) {
-            try {
-                for (File file : files) {
-                    if (seenFiles.add(file)) {
-                        ComponentArtifactIdentifier artifactIdentifier;
-                        if (componentIdentifier == null) {
-                            artifactIdentifier = new OpaqueComponentArtifactIdentifier(file);
-                        } else {
-                            artifactIdentifier = new ComponentFileArtifactIdentifier(componentIdentifier, file.getName());
-                        }
-                        artifacts.add(new DefaultResolvedArtifactResult(artifactIdentifier, Artifact.class, file));
-                    }
-                }
-            } catch (Throwable t) {
-                failures.add(t);
+        public void visitFile(ComponentArtifactIdentifier artifactIdentifier, AttributeContainer variant, File file) {
+            if (seenArtifacts.add(artifactIdentifier)) {
+                artifacts.add(new DefaultResolvedArtifactResult(artifactIdentifier, variant, Artifact.class, file));
             }
         }
     }
@@ -409,8 +420,8 @@ public class DefaultLenientConfiguration implements LenientConfiguration, Visite
         }
 
         @Override
-        public void visitFiles(@Nullable ComponentIdentifier componentIdentifier, Iterable<File> files) {
-            CollectionUtils.addAll(this.files, files);
+        public void visitFile(ComponentArtifactIdentifier artifactIdentifier, AttributeContainer variant, File file) {
+            this.files.add(file);
         }
     }
 
@@ -444,7 +455,7 @@ public class DefaultLenientConfiguration implements LenientConfiguration, Visite
         public void getNodeValues(DependencyGraphNodeResult node, Collection<? super ResolvedArtifact> values, Collection<? super DependencyGraphNodeResult> connectedNodes) {
             connectedNodes.addAll(node.getOutgoingEdges());
             if (artifactsVisitor.includeFiles()) {
-                fileDependencyResults.getFiles(node.getNodeId()).visit(artifactsVisitor);
+                fileDependencyResults.getArtifacts(node.getNodeId()).visit(artifactsVisitor);
             }
         }
 
