@@ -17,11 +17,11 @@
 package org.gradle.api.internal.artifacts.ivyservice.ivyresolve.parser;
 
 import org.apache.ivy.core.module.descriptor.DefaultModuleDescriptor;
-import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy.ResolverStrategy;
+import org.gradle.api.internal.artifacts.ImmutableModuleIdentifierFactory;
 
 public class DownloadedIvyModuleDescriptorParser extends IvyXmlModuleDescriptorParser {
-    public DownloadedIvyModuleDescriptorParser(ResolverStrategy resolverStrategy) {
-        super(resolverStrategy);
+    public DownloadedIvyModuleDescriptorParser(IvyModuleDescriptorConverter moduleDescriptorConverter, ImmutableModuleIdentifierFactory moduleIdentifierFactory) {
+        super(moduleDescriptorConverter, moduleIdentifierFactory);
     }
 
     @Override
