@@ -21,12 +21,13 @@ import org.apache.ivy.core.IvyPatternHelper;
 import org.gradle.api.Transformer;
 import org.gradle.api.artifacts.ModuleIdentifier;
 import org.gradle.api.artifacts.ModuleVersionIdentifier;
-import org.gradle.api.internal.artifacts.DefaultModuleIdentifier;
 import org.gradle.api.internal.artifacts.DefaultModuleVersionIdentifier;
+import org.gradle.api.internal.artifacts.ImmutableModuleIdentifierFactory;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.parser.data.MavenDependencyKey;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.parser.data.PomDependencyMgt;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.parser.data.PomProfile;
 import org.gradle.internal.UncheckedException;
+import org.gradle.internal.classloader.ClassLoaderUtils;
 import org.gradle.internal.resource.local.LocallyAvailableExternalResource;
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
@@ -50,7 +51,11 @@ import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
 
-import static org.gradle.api.internal.artifacts.ivyservice.ivyresolve.parser.PomDomParser.*;
+import static org.gradle.api.internal.artifacts.ivyservice.ivyresolve.parser.PomDomParser.AddDTDFilterInputStream;
+import static org.gradle.api.internal.artifacts.ivyservice.ivyresolve.parser.PomDomParser.getAllChilds;
+import static org.gradle.api.internal.artifacts.ivyservice.ivyresolve.parser.PomDomParser.getFirstChildElement;
+import static org.gradle.api.internal.artifacts.ivyservice.ivyresolve.parser.PomDomParser.getFirstChildText;
+import static org.gradle.api.internal.artifacts.ivyservice.ivyresolve.parser.PomDomParser.getTextContent;
 
 /**
  * Copied from org.apache.ivy.plugins.parser.m2.PomReader.
@@ -98,7 +103,7 @@ public class PomReader implements PomParent {
         // Set the context classloader the bootstrap classloader, to work around the way that JAXP locates implementation classes
         // This should ensure that the JAXP classes provided by the JVM are used, rather than some other implementation
         ClassLoader original = Thread.currentThread().getContextClassLoader();
-        Thread.currentThread().setContextClassLoader(ClassLoader.getSystemClassLoader().getParent());
+        Thread.currentThread().setContextClassLoader(ClassLoaderUtils.getPlatformClassLoader());
         try {
             DOCUMENT_BUILDER_FACTORY = DocumentBuilderFactory.newInstance();
             DOCUMENT_BUILDER_FACTORY.setValidating(false);
@@ -124,11 +129,13 @@ public class PomReader implements PomParent {
     private Map<MavenDependencyKey, PomDependencyMgt> resolvedDependencyMgts;
     private final Map<MavenDependencyKey, PomDependencyMgt> importedDependencyMgts = new LinkedHashMap<MavenDependencyKey, PomDependencyMgt>();
     private Map<MavenDependencyKey, PomDependencyData> resolvedDependencies;
+    private final ImmutableModuleIdentifierFactory moduleIdentifierFactory;
 
     private final Element projectElement;
     private final Element parentElement;
 
-    public PomReader(final LocallyAvailableExternalResource resource) throws SAXException {
+    public PomReader(final LocallyAvailableExternalResource resource, ImmutableModuleIdentifierFactory moduleIdentifierFactory) throws SAXException {
+        this.moduleIdentifierFactory = moduleIdentifierFactory;
         final String systemId = resource.getLocalResource().getFile().toURI().toASCIIString();
         Document pomDomDoc = resource.withContent(new Transformer<Document, InputStream>() {
             public Document transform(InputStream inputStream) {
@@ -157,13 +164,13 @@ public class PomReader implements PomParent {
     }
 
     private void maybeSetGavProperties(GavProperty gavProperty, String propertyValue) {
-        for(String name : gavProperty.getNames()) {
+        for (String name : gavProperty.getNames()) {
             maybeSetProperty(name, propertyValue);
         }
     }
 
     private void setPomProperties() {
-        for(Map.Entry<String, String> pomProperty : getPomProperties().entrySet()) {
+        for (Map.Entry<String, String> pomProperty : getPomProperties().entrySet()) {
             maybeSetProperty(pomProperty.getKey(), pomProperty.getValue());
         }
     }
@@ -172,8 +179,8 @@ public class PomReader implements PomParent {
      * Sets properties for all active profiles. Properties from an active profile override existing POM properties.
      */
     private void setActiveProfileProperties() {
-        for(PomProfile activePomProfile : parseActivePomProfiles()) {
-            for(Map.Entry<String, String> property : activePomProfile.getProperties().entrySet()) {
+        for (PomProfile activePomProfile : parseActivePomProfiles()) {
+            for (Map.Entry<String, String> property : activePomProfile.getProperties().entrySet()) {
                 properties.put(property.getKey(), property.getValue());
             }
         }
@@ -186,7 +193,7 @@ public class PomReader implements PomParent {
 
     private void setPomParentProperties() {
         Map<String, String> parentPomProps = pomParent.getProperties();
-        for(Map.Entry<String, String> entry : parentPomProps.entrySet()) {
+        for (Map.Entry<String, String> entry : parentPomProps.entrySet()) {
             maybeSetProperty(entry.getKey(), entry.getValue());
         }
     }
@@ -242,7 +249,7 @@ public class PomReader implements PomParent {
         // Set the context classloader the bootstrap classloader, to work around the way that JAXP locates implementation classes
         // This should ensure that the JAXP classes provided by the JVM are used, rather than some other implementation
         ClassLoader original = Thread.currentThread().getContextClassLoader();
-        Thread.currentThread().setContextClassLoader(ClassLoader.getSystemClassLoader().getParent());
+        Thread.currentThread().setContextClassLoader(ClassLoaderUtils.getPlatformClassLoader());
         try {
             InputStream dtdStream = new AddDTDFilterInputStream(stream);
             return getDocBuilder(M2_ENTITY_RESOLVER).parse(dtdStream, systemId);
@@ -278,7 +285,7 @@ public class PomReader implements PomParent {
     }
 
     public String getGroupId() {
-        String groupId = getFirstChildText(projectElement , GROUP_ID);
+        String groupId = getFirstChildText(projectElement, GROUP_ID);
         if (groupId == null) {
             groupId = getFirstChildText(parentElement, GROUP_ID);
         }
@@ -287,7 +294,7 @@ public class PomReader implements PomParent {
     }
 
     public String getParentGroupId() {
-        String groupId = getFirstChildText(parentElement , GROUP_ID);
+        String groupId = getFirstChildText(parentElement, GROUP_ID);
         if (groupId == null) {
             groupId = getFirstChildText(projectElement, GROUP_ID);
         }
@@ -296,7 +303,7 @@ public class PomReader implements PomParent {
     }
 
     public String getArtifactId() {
-        String val = getFirstChildText(projectElement , ARTIFACT_ID);
+        String val = getFirstChildText(projectElement, ARTIFACT_ID);
         if (val == null) {
             val = getFirstChildText(parentElement, ARTIFACT_ID);
         }
@@ -305,7 +312,7 @@ public class PomReader implements PomParent {
     }
 
     public String getParentArtifactId() {
-        String val = getFirstChildText(parentElement , ARTIFACT_ID);
+        String val = getFirstChildText(parentElement, ARTIFACT_ID);
         if (val == null) {
             val = getFirstChildText(projectElement, ARTIFACT_ID);
         }
@@ -314,7 +321,7 @@ public class PomReader implements PomParent {
     }
 
     public String getVersion() {
-        String val = getFirstChildText(projectElement , VERSION);
+        String val = getFirstChildText(projectElement, VERSION);
         if (val == null) {
             val = getFirstChildText(parentElement, VERSION);
         }
@@ -322,7 +329,7 @@ public class PomReader implements PomParent {
     }
 
     public String getParentVersion() {
-        String val = getFirstChildText(parentElement , VERSION);
+        String val = getFirstChildText(parentElement, VERSION);
         if (val == null) {
             val = getFirstChildText(projectElement, VERSION);
         }
@@ -330,7 +337,7 @@ public class PomReader implements PomParent {
     }
 
     public String getPackaging() {
-        String val = getFirstChildText(projectElement , PACKAGING);
+        String val = getFirstChildText(projectElement, PACKAGING);
         if (val == null) {
             val = "jar";
         }
@@ -347,7 +354,7 @@ public class PomReader implements PomParent {
 
     public ModuleVersionIdentifier getRelocation() {
         Element distrMgt = getFirstChildElement(projectElement, DISTRIBUTION_MGT);
-        Element relocation = getFirstChildElement(distrMgt , RELOCATION);
+        Element relocation = getFirstChildElement(distrMgt, RELOCATION);
         if (relocation == null) {
             return null;
         } else {
@@ -374,7 +381,7 @@ public class PomReader implements PomParent {
     private Map<MavenDependencyKey, PomDependencyData> resolveDependencies() {
         Map<MavenDependencyKey, PomDependencyData> dependencies = new LinkedHashMap<MavenDependencyKey, PomDependencyData>();
 
-        for(PomDependencyData dependency : getDependencyData(projectElement)) {
+        for (PomDependencyData dependency : getDependencyData(projectElement)) {
             dependencies.put(dependency.getId(), dependency);
         }
 
@@ -385,8 +392,8 @@ public class PomReader implements PomParent {
             }
         }
 
-        for(PomProfile pomProfile : parseActivePomProfiles()) {
-            for(PomDependencyData dependency : pomProfile.getDependencies()) {
+        for (PomProfile pomProfile : parseActivePomProfiles()) {
+            for (PomDependencyData dependency : pomProfile.getDependencies()) {
                 dependencies.put(dependency.getId(), dependency);
             }
         }
@@ -414,7 +421,7 @@ public class PomReader implements PomParent {
      * Returns all dependency management elements for this POM, including those inherited from parent and imported POMs.
      */
     public Map<MavenDependencyKey, PomDependencyMgt> getDependencyMgt() {
-        if(resolvedDependencyMgts == null) {
+        if (resolvedDependencyMgts == null) {
             resolvedDependencyMgts = resolveDependencyMgt();
         }
         return resolvedDependencyMgts;
@@ -424,7 +431,7 @@ public class PomReader implements PomParent {
         Map<MavenDependencyKey, PomDependencyMgt> dependencies = new LinkedHashMap<MavenDependencyKey, PomDependencyMgt>();
         dependencies.putAll(pomParent.getDependencyMgt());
         dependencies.putAll(importedDependencyMgts);
-        for(PomDependencyMgt dependencyMgt : parseDependencyMgt()) {
+        for (PomDependencyMgt dependencyMgt : parseDependencyMgt()) {
             dependencies.put(dependencyMgt.getId(), dependencyMgt);
         }
         return dependencies;
@@ -436,11 +443,11 @@ public class PomReader implements PomParent {
      * @return Parsed dependency management elements
      */
     public List<PomDependencyMgt> parseDependencyMgt() {
-        if(declaredDependencyMgts == null) {
+        if (declaredDependencyMgts == null) {
             List<PomDependencyMgt> dependencyMgts = getDependencyMgt(projectElement);
 
-            for(PomProfile pomProfile : parseActivePomProfiles()) {
-                for(PomDependencyMgt dependencyMgt : pomProfile.getDependencyMgts()) {
+            for (PomProfile pomProfile : parseActivePomProfiles()) {
+                for (PomDependencyMgt dependencyMgt : pomProfile.getDependencyMgts()) {
                     dependencyMgts.add(dependencyMgt);
                 }
             }
@@ -480,7 +487,7 @@ public class PomReader implements PomParent {
     }
 
     private void setGavPropertyValue(GavProperty gavProperty, String propertyValue) {
-        for(String name : gavProperty.getNames()) {
+        for (String name : gavProperty.getNames()) {
             properties.put(name, propertyValue);
         }
     }
@@ -518,20 +525,20 @@ public class PomReader implements PomParent {
          * @see org.apache.ivy.plugins.parser.m2.PomDependencyMgt#getVersion()
          */
         public String getVersion() {
-            String val = getFirstChildText(depElement , VERSION);
+            String val = getFirstChildText(depElement, VERSION);
             return replaceProps(val);
         }
 
         public String getScope() {
-            String val = getFirstChildText(depElement , SCOPE);
+            String val = getFirstChildText(depElement, SCOPE);
             return replaceProps(val);
         }
 
         public String getType() {
-            String val = getFirstChildText(depElement , TYPE);
+            String val = getFirstChildText(depElement, TYPE);
             val = replaceProps(val);
 
-            if(val == null) {
+            if (val == null) {
                 val = "jar";
             }
 
@@ -539,7 +546,7 @@ public class PomReader implements PomParent {
         }
 
         public String getClassifier() {
-            String val = getFirstChildText(depElement , CLASSIFIER);
+            String val = getFirstChildText(depElement, CLASSIFIER);
             return replaceProps(val);
         }
 
@@ -554,7 +561,7 @@ public class PomReader implements PomParent {
                         String groupId = getFirstChildText((Element) node, GROUP_ID);
                         String artifactId = getFirstChildText((Element) node, ARTIFACT_ID);
                         if ((groupId != null) && (artifactId != null)) {
-                            exclusions.add(DefaultModuleIdentifier.newId(groupId, artifactId));
+                            exclusions.add(moduleIdentifierFactory.module(groupId, artifactId));
                         }
                     }
                 }
@@ -566,6 +573,7 @@ public class PomReader implements PomParent {
 
     public class PomDependencyData extends PomDependencyMgtElement {
         private final Element depElement;
+
         PomDependencyData(Element depElement) {
             super(depElement);
             this.depElement = depElement;
@@ -595,7 +603,7 @@ public class PomReader implements PomParent {
         }
 
         public List<PomDependencyMgt> getDependencyMgts() {
-            if(declaredDependencyMgts == null) {
+            if (declaredDependencyMgts == null) {
                 declaredDependencyMgts = getDependencyMgt(element);
             }
 
@@ -603,7 +611,7 @@ public class PomReader implements PomParent {
         }
 
         public List<PomDependencyData> getDependencies() {
-            if(declaredDependencies == null) {
+            if (declaredDependencies == null) {
                 declaredDependencies = getDependencyData(element);
             }
 
@@ -617,26 +625,26 @@ public class PomReader implements PomParent {
      * @return Active POM profiles
      */
     private List<PomProfile> parseActivePomProfiles() {
-        if(declaredActivePomProfiles == null) {
+        if (declaredActivePomProfiles == null) {
             List<PomProfile> activeByDefaultPomProfiles = new ArrayList<PomProfile>();
             List<PomProfile> activeByAbsenceOfPropertyPomProfiles = new ArrayList<PomProfile>();
             Element profilesElement = getFirstChildElement(projectElement, PROFILES);
 
-            if(profilesElement != null) {
-                for(Element profileElement : getAllChilds(profilesElement)) {
-                    if(PROFILE.equals(profileElement.getNodeName())) {
+            if (profilesElement != null) {
+                for (Element profileElement : getAllChilds(profilesElement)) {
+                    if (PROFILE.equals(profileElement.getNodeName())) {
                         Element activationElement = getFirstChildElement(profileElement, PROFILE_ACTIVATION);
 
-                        if(activationElement != null) {
+                        if (activationElement != null) {
                             String activeByDefault = getFirstChildText(activationElement, PROFILE_ACTIVATION_ACTIVE_BY_DEFAULT);
 
-                            if(activeByDefault != null && "true".equals(activeByDefault)) {
+                            if (activeByDefault != null && "true".equals(activeByDefault)) {
                                 activeByDefaultPomProfiles.add(new PomProfileElement(profileElement));
                             } else {
                                 Element propertyElement = getFirstChildElement(activationElement, PROFILE_ACTIVATION_PROPERTY);
 
-                                if(propertyElement != null) {
-                                    if(isActivationPropertyActivated(propertyElement)) {
+                                if (propertyElement != null) {
+                                    if (isActivationPropertyActivated(propertyElement)) {
                                         activeByAbsenceOfPropertyPomProfiles.add(new PomProfileElement(profileElement));
                                     }
                                 }
