@@ -18,10 +18,13 @@ package org.gradle.api.internal.artifacts.ivyservice.modulecache;
 import com.google.common.collect.LinkedHashMultimap;
 import com.google.common.collect.Lists;
 import com.google.common.collect.SetMultimap;
+import org.gradle.api.artifacts.ModuleVersionIdentifier;
 import org.gradle.api.artifacts.ModuleVersionSelector;
 import org.gradle.api.artifacts.component.ModuleComponentIdentifier;
 import org.gradle.api.internal.artifacts.DefaultModuleVersionSelector;
+import org.gradle.api.internal.artifacts.ImmutableModuleIdentifierFactory;
 import org.gradle.api.internal.artifacts.ivyservice.NamespaceId;
+import org.gradle.api.internal.artifacts.ivyservice.resolveengine.excludes.ModuleExclusions;
 import org.gradle.internal.component.external.descriptor.Artifact;
 import org.gradle.internal.component.external.descriptor.Configuration;
 import org.gradle.internal.component.external.descriptor.DefaultExclude;
@@ -57,8 +60,8 @@ public class ModuleMetadataSerializer {
     private static final byte TYPE_IVY = 1;
     private static final byte TYPE_MAVEN = 2;
 
-    public MutableModuleComponentResolveMetadata read(Decoder decoder) throws IOException {
-        return new Reader(decoder).read();
+    public MutableModuleComponentResolveMetadata read(Decoder decoder, ImmutableModuleIdentifierFactory moduleIdentifierFactory, ModuleExclusions moduleExclusions) throws IOException {
+        return new Reader(decoder, moduleIdentifierFactory).read();
     }
 
     public void write(Encoder encoder, ModuleComponentResolveMetadata metadata) throws IOException {
@@ -182,7 +185,7 @@ public class ModuleMetadataSerializer {
                 encoder.writeByte(TYPE_IVY);
                 writeDependencyConfigurationMapping(ivyDependency);
                 writeArtifacts(ivyDependency.getDependencyArtifacts());
-                writeExcludeRules(ivyDependency.getDependencyExcludes());
+                writeExcludeRules(ivyDependency.getExcludes());
                 writeString(ivyDependency.getDynamicConstraintVersion());
                 writeBoolean(ivyDependency.isForce());
                 writeBoolean(ivyDependency.isChanging());
@@ -191,7 +194,7 @@ public class ModuleMetadataSerializer {
                 MavenDependencyMetadata mavenDependency = (MavenDependencyMetadata) dep;
                 encoder.writeByte(TYPE_MAVEN);
                 writeArtifacts(mavenDependency.getDependencyArtifacts());
-                writeExcludeRules(mavenDependency.getDependencyExcludes());
+                writeExcludeRules(mavenDependency.getExcludes());
                 encoder.writeSmallInt(mavenDependency.getScope().ordinal());
                 encoder.writeBoolean(mavenDependency.isOptional());
             } else {
@@ -274,11 +277,14 @@ public class ModuleMetadataSerializer {
 
     private static class Reader {
         private final Decoder decoder;
+        private final ImmutableModuleIdentifierFactory moduleIdentifierFactory;
         private MutableModuleDescriptorState md;
         private ModuleComponentIdentifier id;
+        private ModuleVersionIdentifier mvi;
 
-        private Reader(Decoder decoder) {
+        private Reader(Decoder decoder, ImmutableModuleIdentifierFactory moduleIdentifierFactory) {
             this.decoder = decoder;
+            this.moduleIdentifierFactory = moduleIdentifierFactory;
         }
 
         public MutableModuleComponentResolveMetadata read() throws IOException {
@@ -305,7 +311,7 @@ public class ModuleMetadataSerializer {
             String snapshotTimestamp = readNullableString();
             String packaging = readNullableString();
             boolean relocated = readBoolean();
-            DefaultMutableMavenModuleResolveMetadata metadata = new DefaultMutableMavenModuleResolveMetadata(id, md, packaging, relocated, dependencies);
+            DefaultMutableMavenModuleResolveMetadata metadata = new DefaultMutableMavenModuleResolveMetadata(mvi, id, md, packaging, relocated, dependencies);
             metadata.setSnapshotTimestamp(snapshotTimestamp);
             return metadata;
         }
@@ -315,7 +321,7 @@ public class ModuleMetadataSerializer {
             List<Configuration> configurations = readConfigurations();
             List<DependencyMetadata> dependencies = readDependencies();
             readSharedInfo();
-            return new DefaultMutableIvyModuleResolveMetadata(id, md, configurations, dependencies);
+            return new DefaultMutableIvyModuleResolveMetadata(mvi, id, md, configurations, dependencies);
         }
 
         private void readInfoSection() throws IOException {
@@ -330,6 +336,7 @@ public class ModuleMetadataSerializer {
             md.setDescription(readNullableString());
             md.setPublicationDate(readNullableDate());
             md.setBranch(readNullableString());
+            mvi = moduleIdentifierFactory.moduleWithVersion(componentIdentifier.getGroup(), componentIdentifier.getModule(), componentIdentifier.getVersion());
 
             readExtraInfo();
         }
@@ -446,7 +453,7 @@ public class ModuleMetadataSerializer {
             String ext = readString();
             String[] confs = readStringArray();
             String matcher = readString();
-            return new DefaultExclude(moduleOrg, moduleName, artifact, type, ext, confs, matcher);
+            return new DefaultExclude(moduleIdentifierFactory.module(moduleOrg, moduleName), artifact, type, ext, confs, matcher);
         }
 
         private void readAllExcludes() throws IOException {
