@@ -16,14 +16,22 @@
 package org.gradle.api.internal.artifacts.ivyservice.moduleconverter.dependencies;
 
 import org.gradle.api.artifacts.ExcludeRule;
+import org.gradle.api.internal.artifacts.ImmutableModuleIdentifierFactory;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.excludes.PatternMatchers;
 import org.gradle.internal.component.external.descriptor.DefaultExclude;
 import org.gradle.util.GUtil;
 
 public class DefaultExcludeRuleConverter implements ExcludeRuleConverter {
+    private final ImmutableModuleIdentifierFactory moduleIdentifierFactory;
+
+    public DefaultExcludeRuleConverter(ImmutableModuleIdentifierFactory moduleIdentifierFactory) {
+        this.moduleIdentifierFactory = moduleIdentifierFactory;
+    }
+
     public DefaultExclude convertExcludeRule(String configurationName, ExcludeRule excludeRule) {
         String org = GUtil.elvis(excludeRule.getGroup(), PatternMatchers.ANY_EXPRESSION);
         String module = GUtil.elvis(excludeRule.getModule(), PatternMatchers.ANY_EXPRESSION);
-        return new DefaultExclude(org, module, new String[] {configurationName}, PatternMatchers.EXACT);
+        String[] configurationNames = GUtil.isTrue(configurationName) ? new String[]{configurationName} : new String[0];
+        return new DefaultExclude(moduleIdentifierFactory.module(org, module), configurationNames, PatternMatchers.EXACT);
     }
 }
