@@ -16,10 +16,16 @@
 
 package org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact;
 
-import java.util.Set;
+import org.gradle.api.Transformer;
+import org.gradle.api.artifacts.component.ComponentIdentifier;
+import org.gradle.api.specs.Spec;
 
+import java.util.Collection;
+
+/**
+ * Represents a container of artifacts, possibly made up of several different variants.
+ */
 public interface ArtifactSet {
-
     long getId();
 
     /**
@@ -27,5 +33,8 @@ public interface ArtifactSet {
      */
     ArtifactSet snapshot();
 
-    Set<? extends ResolvedVariant> getVariants();
+    /**
+     * Selects the artifacts of this set that meet the given criteria. Implementation should be eager where possible, so that selection happens immediately, but may be lazy.
+     */
+    ResolvedArtifactSet select(Spec<? super ComponentIdentifier> componentFilter, Transformer<ResolvedArtifactSet, Collection<? extends ResolvedVariant>> selector);
 }
