@@ -17,7 +17,11 @@
 package org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact;
 
 import org.gradle.api.attributes.HasAttributes;
+import org.gradle.api.internal.attributes.AttributeContainerInternal;
 
 public interface ResolvedVariant extends HasAttributes {
+    @Override
+    AttributeContainerInternal getAttributes();
+
     ResolvedArtifactSet getArtifacts();
 }
