@@ -17,13 +17,14 @@
 package org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact;
 
 import org.gradle.api.Transformer;
-import org.gradle.api.attributes.HasAttributes;
+import org.gradle.api.artifacts.component.ComponentIdentifier;
+import org.gradle.api.specs.Spec;
 
 import java.util.Collection;
 
 public interface VisitedArtifactsResults {
     /**
-     * Selects the artifacts for the matching variant of each node seen during traversal.
+     * Selects the artifacts for the matching variant of each node seen during traversal. The implementation should attempt to select artifacts eagerly, but may be lazy where the selection cannot happen until the results are queried.
      */
-    SelectedArtifactResults select(Transformer<HasAttributes, Collection<? extends HasAttributes>> selector);
+    SelectedArtifactResults select(Spec<? super ComponentIdentifier> componentFilter, Transformer<ResolvedArtifactSet, Collection<? extends ResolvedVariant>> selector);
 }
