@@ -16,7 +16,6 @@
 package org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph;
 
 import org.gradle.api.Action;
-import org.gradle.api.attributes.AttributesSchema;
 import org.gradle.api.artifacts.ModuleDependency;
 import org.gradle.api.artifacts.ModuleIdentifier;
 import org.gradle.api.artifacts.ModuleVersionIdentifier;
@@ -24,7 +23,8 @@ import org.gradle.api.artifacts.ModuleVersionSelector;
 import org.gradle.api.artifacts.component.ComponentIdentifier;
 import org.gradle.api.artifacts.component.ComponentSelector;
 import org.gradle.api.artifacts.result.ComponentSelectionReason;
-import org.gradle.api.internal.artifacts.DefaultModuleIdentifier;
+import org.gradle.api.attributes.AttributesSchema;
+import org.gradle.api.internal.artifacts.ImmutableModuleIdentifierFactory;
 import org.gradle.api.internal.artifacts.ResolveContext;
 import org.gradle.api.internal.artifacts.ResolvedConfigurationIdentifier;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.ComponentResolutionState;
@@ -43,6 +43,7 @@ import org.gradle.internal.component.model.ComponentResolveMetadata;
 import org.gradle.internal.component.model.ConfigurationMetadata;
 import org.gradle.internal.component.model.DefaultComponentOverrideMetadata;
 import org.gradle.internal.component.model.DependencyMetadata;
+import org.gradle.internal.component.model.Exclude;
 import org.gradle.internal.id.IdGenerator;
 import org.gradle.internal.id.LongIdGenerator;
 import org.gradle.internal.resolve.ModuleVersionResolveException;
@@ -75,16 +76,21 @@ public class DependencyGraphBuilder {
     private final DependencyToComponentIdResolver idResolver;
     private final ComponentMetaDataResolver metaDataResolver;
     private final AttributesSchema attributesSchema;
+    private final ImmutableModuleIdentifierFactory moduleIdentifierFactory;
+    private final ModuleExclusions moduleExclusions;
 
     public DependencyGraphBuilder(DependencyToComponentIdResolver componentIdResolver, ComponentMetaDataResolver componentMetaDataResolver,
                                   ResolveContextToComponentResolver resolveContextToComponentResolver,
-                                  ConflictHandler conflictHandler, Spec<? super DependencyMetadata> edgeFilter, AttributesSchema attributesSchema) {
+                                  ConflictHandler conflictHandler, Spec<? super DependencyMetadata> edgeFilter, AttributesSchema attributesSchema,
+                                  ImmutableModuleIdentifierFactory moduleIdentifierFactory, ModuleExclusions moduleExclusions) {
         this.idResolver = componentIdResolver;
         this.metaDataResolver = componentMetaDataResolver;
         this.moduleResolver = resolveContextToComponentResolver;
         this.conflictHandler = conflictHandler;
         this.edgeFilter = edgeFilter;
         this.attributesSchema = attributesSchema;
+        this.moduleIdentifierFactory = moduleIdentifierFactory;
+        this.moduleExclusions = moduleExclusions;
     }
 
     public void resolve(ResolveContext resolveContext, DependencyGraphVisitor modelVisitor) {
@@ -92,7 +98,7 @@ public class DependencyGraphBuilder {
         DefaultBuildableComponentResolveResult rootModule = new DefaultBuildableComponentResolveResult();
         moduleResolver.resolve(resolveContext, rootModule);
 
-        ResolveState resolveState = new ResolveState(idGenerator, rootModule, resolveContext.getName(), idResolver, metaDataResolver, edgeFilter, attributesSchema);
+        ResolveState resolveState = new ResolveState(idGenerator, rootModule, resolveContext.getName(), idResolver, metaDataResolver, edgeFilter, attributesSchema, moduleIdentifierFactory, moduleExclusions);
         conflictHandler.registerResolver(new DirectDependencyForcingResolver(resolveState.root.moduleRevision));
 
         traverseGraph(resolveState, conflictHandler);
@@ -181,23 +187,29 @@ public class DependencyGraphBuilder {
     /**
      * Populates the result from the graph traversal state.
      */
-    private void assembleResult(ResolveState resolveState, DependencyGraphVisitor listener) {
-        listener.start(resolveState.root);
+    private void assembleResult(ResolveState resolveState, DependencyGraphVisitor visitor) {
+        visitor.start(resolveState.root);
 
         // Visit the nodes
         for (ConfigurationNode resolvedConfiguration : resolveState.getConfigurationNodes()) {
             if (resolvedConfiguration.isSelected()) {
-                listener.visitNode(resolvedConfiguration);
-            }
-        }
-        // Visit the edges
-        for (ConfigurationNode resolvedConfiguration : resolveState.getConfigurationNodes()) {
-            if (resolvedConfiguration.isSelected()) {
-                listener.visitEdge(resolvedConfiguration);
+                visitor.visitNode(resolvedConfiguration);
             }
         }
 
-        listener.finish(resolveState.root);
+        // Visit the selectors
+        for (DependencyGraphSelector selector : resolveState.getSelectors()) {
+            visitor.visitSelector(selector);
+        }
+
+        // Visit the edges
+        for (ConfigurationNode resolvedConfiguration : resolveState.getConfigurationNodes()) {
+            if (resolvedConfiguration.isSelected()) {
+                visitor.visitEdges(resolvedConfiguration);
+            }
+        }
+
+        visitor.finish(resolveState.root);
     }
 
     /**
@@ -231,6 +243,11 @@ public class DependencyGraphBuilder {
             return from;
         }
 
+        @Override
+        public DependencyGraphSelector getSelector() {
+            return selector;
+        }
+
         /**
          * @return The resolved module version
          */
@@ -290,10 +307,18 @@ public class DependencyGraphBuilder {
             }
         }
 
+        public ModuleExclusion toExclusions(DependencyMetadata md, ConfigurationMetadata from) {
+            List<Exclude> excludes = md.getExcludes(from.getHierarchy());
+            if (excludes.isEmpty()) {
+                return ModuleExclusions.excludeNone();
+            }
+            return resolveState.moduleExclusions.excludeAny(excludes);
+        }
+
         @Override
-        public ModuleExclusion getExclusions() {
-            ModuleExclusion edgeExclusions = dependencyMetadata.getExclusions(from.metaData);
-            return ModuleExclusions.intersect(edgeExclusions, moduleExclusion);
+        public ModuleExclusion getExclusions(ModuleExclusions moduleExclusions) {
+            ModuleExclusion edgeExclusions = toExclusions(dependencyMetadata, from.metaData);
+            return resolveState.moduleExclusions.intersect(edgeExclusions, moduleExclusion);
         }
 
         @Override
@@ -329,6 +354,11 @@ public class DependencyGraphBuilder {
             return null;
         }
 
+        @Override
+        public Iterable<? extends DependencyGraphNode> getTargets() {
+            return targetConfigurations;
+        }
+
         @Override
         public Set<ComponentArtifactMetadata> getArtifacts(ConfigurationMetadata metaData1) {
             return dependencyMetadata.getArtifacts(from.metaData, metaData1);
@@ -350,14 +380,19 @@ public class DependencyGraphBuilder {
         private final Set<ConfigurationNode> queued = new HashSet<ConfigurationNode>();
         private final LinkedList<ConfigurationNode> queue = new LinkedList<ConfigurationNode>();
         private final AttributesSchema attributesSchema;
+        private final ImmutableModuleIdentifierFactory moduleIdentifierFactory;
+        private final ModuleExclusions moduleExclusions;
 
         public ResolveState(IdGenerator<Long> idGenerator, ComponentResolveResult rootResult, String rootConfigurationName, DependencyToComponentIdResolver idResolver,
-                            ComponentMetaDataResolver metaDataResolver, Spec<? super DependencyMetadata> edgeFilter, AttributesSchema attributesSchema) {
+                            ComponentMetaDataResolver metaDataResolver, Spec<? super DependencyMetadata> edgeFilter, AttributesSchema attributesSchema,
+                            ImmutableModuleIdentifierFactory moduleIdentifierFactory, ModuleExclusions moduleExclusions) {
             this.idGenerator = idGenerator;
             this.idResolver = idResolver;
             this.metaDataResolver = metaDataResolver;
             this.edgeFilter = edgeFilter;
             this.attributesSchema = attributesSchema;
+            this.moduleIdentifierFactory = moduleIdentifierFactory;
+            this.moduleExclusions = moduleExclusions;
             ModuleVersionResolveState rootVersion = getRevision(rootResult.getId());
             rootVersion.setMetaData(rootResult.getMetaData());
             root = new RootConfigurationNode(idGenerator.generateId(), rootVersion, new ResolvedConfigurationIdentifier(rootVersion.id, rootConfigurationName), this);
@@ -392,11 +427,15 @@ public class DependencyGraphBuilder {
             return configuration;
         }
 
+        public Collection<ModuleVersionSelectorResolveState> getSelectors() {
+            return selectors.values();
+        }
+
         public ModuleVersionSelectorResolveState getSelector(DependencyMetadata dependencyMetadata) {
             ModuleVersionSelector requested = dependencyMetadata.getRequested();
             ModuleVersionSelectorResolveState resolveState = selectors.get(requested);
             if (resolveState == null) {
-                resolveState = new ModuleVersionSelectorResolveState(dependencyMetadata, idResolver, this);
+                resolveState = new ModuleVersionSelectorResolveState(idGenerator.generateId(), dependencyMetadata, idResolver, this);
                 selectors.put(requested, resolveState);
             }
             return resolveState;
@@ -796,7 +835,7 @@ public class DependencyGraphBuilder {
                 LOGGER.debug("{} is filtered.", dependency);
                 return true;
             }
-            ModuleIdentifier targetModuleId = DefaultModuleIdentifier.newId(dependency.getRequested().getGroup(), dependency.getRequested().getName());
+            ModuleIdentifier targetModuleId = resolveState.moduleIdentifierFactory.module(dependency.getRequested().getGroup(), dependency.getRequested().getName());
             if (selector.excludeModule(targetModuleId)) {
                 LOGGER.debug("{} is excluded from {}.", targetModuleId, this);
                 return true;
@@ -821,16 +860,17 @@ public class DependencyGraphBuilder {
 
         private ModuleExclusion getModuleResolutionFilter(List<DependencyEdge> transitiveEdges) {
             ModuleExclusion resolutionFilter;
+            ModuleExclusions moduleExclusions = resolveState.moduleExclusions;
             if (transitiveEdges.isEmpty()) {
                 resolutionFilter = ModuleExclusions.excludeNone();
             } else {
-                resolutionFilter = transitiveEdges.get(0).getExclusions();
+                resolutionFilter = transitiveEdges.get(0).getExclusions(moduleExclusions);
                 for (int i = 1; i < transitiveEdges.size(); i++) {
                     DependencyEdge dependencyEdge = transitiveEdges.get(i);
-                    resolutionFilter = ModuleExclusions.union(resolutionFilter, dependencyEdge.getExclusions());
+                    resolutionFilter = moduleExclusions.union(resolutionFilter, dependencyEdge.getExclusions(moduleExclusions));
                 }
             }
-            resolutionFilter = ModuleExclusions.intersect(resolutionFilter, metaData.getExclusions());
+            resolutionFilter = moduleExclusions.intersect(resolutionFilter, metaData.getExclusions(moduleExclusions));
             return resolutionFilter;
         }
 
@@ -879,7 +919,8 @@ public class DependencyGraphBuilder {
     /**
      * Resolution state for a given module version selector.
      */
-    private static class ModuleVersionSelectorResolveState {
+    private static class ModuleVersionSelectorResolveState implements DependencyGraphSelector {
+        final Long id;
         final DependencyMetadata dependencyMetadata;
         final DependencyToComponentIdResolver resolver;
         final ResolveState resolveState;
@@ -888,11 +929,17 @@ public class DependencyGraphBuilder {
         ModuleVersionResolveState targetModuleRevision;
         BuildableComponentIdResolveResult idResolveResult;
 
-        private ModuleVersionSelectorResolveState(DependencyMetadata dependencyMetadata, DependencyToComponentIdResolver resolver, ResolveState resolveState) {
+        private ModuleVersionSelectorResolveState(Long id, DependencyMetadata dependencyMetadata, DependencyToComponentIdResolver resolver, ResolveState resolveState) {
+            this.id = id;
             this.dependencyMetadata = dependencyMetadata;
             this.resolver = resolver;
             this.resolveState = resolveState;
-            targetModule = resolveState.getModule(new DefaultModuleIdentifier(dependencyMetadata.getRequested().getGroup(), dependencyMetadata.getRequested().getName()));
+            targetModule = resolveState.getModule(resolveState.moduleIdentifierFactory.module(dependencyMetadata.getRequested().getGroup(), dependencyMetadata.getRequested().getName()));
+        }
+
+        @Override
+        public Long getResultId() {
+            return id;
         }
 
         @Override
@@ -900,6 +947,11 @@ public class DependencyGraphBuilder {
             return dependencyMetadata.toString();
         }
 
+        @Override
+        public ComponentSelector getRequested() {
+            return dependencyMetadata.getSelector();
+        }
+
         private ModuleVersionResolveException getFailure() {
             return failure != null ? failure : targetModuleRevision.getFailure();
         }
