@@ -20,21 +20,24 @@ import org.gradle.api.Nullable;
 import org.gradle.api.artifacts.ModuleDependency;
 import org.gradle.api.artifacts.ModuleVersionSelector;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.excludes.ModuleExclusion;
+import org.gradle.api.internal.artifacts.ivyservice.resolveengine.excludes.ModuleExclusions;
 import org.gradle.internal.component.model.ComponentArtifactMetadata;
 import org.gradle.internal.component.model.ConfigurationMetadata;
 
 import java.util.Set;
 
 /**
- * An edge in the dependency graph, between 2 configurations.
+ * An edge in the dependency graph, between 2 nodes.
  */
 public interface DependencyGraphEdge extends DependencyResult {
     DependencyGraphNode getFrom();
 
+    DependencyGraphSelector getSelector();
+
     // TODO This should be replaced by getRequested()
     ModuleVersionSelector getRequestedModuleVersion();
 
-    ModuleExclusion getExclusions();
+    ModuleExclusion getExclusions(ModuleExclusions moduleExclusions);
 
     boolean isTransitive();
 
@@ -42,4 +45,6 @@ public interface DependencyGraphEdge extends DependencyResult {
 
     @Nullable
     ModuleDependency getModuleDependency();
+
+    Iterable<? extends DependencyGraphNode> getTargets();
 }
