@@ -19,10 +19,12 @@ package org.gradle.api.internal.artifacts.ivyservice.resolveengine.result;
 import org.gradle.api.artifacts.component.ComponentSelector;
 import org.gradle.api.artifacts.result.ResolutionResult;
 import org.gradle.api.artifacts.result.ResolvedComponentResult;
+import org.gradle.api.internal.artifacts.ImmutableModuleIdentifierFactory;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.ComponentResult;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.DependencyGraphComponent;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.DependencyGraphEdge;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.DependencyGraphNode;
+import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.DependencyGraphSelector;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.DependencyGraphVisitor;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.DependencyResult;
 import org.gradle.api.internal.artifacts.result.DefaultResolutionResult;
@@ -34,7 +36,8 @@ import org.gradle.internal.Factory;
 import org.gradle.internal.resolve.ModuleVersionResolveException;
 import org.gradle.internal.serialize.Decoder;
 import org.gradle.internal.serialize.Encoder;
-import org.gradle.util.Clock;
+import org.gradle.internal.time.Timer;
+import org.gradle.internal.time.Timers;
 
 import java.io.IOException;
 import java.util.ArrayList;
@@ -49,23 +52,26 @@ import static org.gradle.internal.UncheckedException.throwAsUncheckedException;
 public class StreamingResolutionResultBuilder implements DependencyGraphVisitor {
     private final static byte ROOT = 1;
     private final static byte COMPONENT = 2;
-    private final static byte DEPENDENCY = 3;
+    private final static byte SELECTOR = 4;
+    private final static byte DEPENDENCY = 5;
 
     private final Map<ComponentSelector, ModuleVersionResolveException> failures = new HashMap<ComponentSelector, ModuleVersionResolveException>();
     private final BinaryStore store;
-    private final ComponentResultSerializer componentResultSerializer = new ComponentResultSerializer();
+    private final ComponentResultSerializer componentResultSerializer;
     private final Store<ResolvedComponentResult> cache;
+    private final ComponentSelectorSerializer componentSelectorSerializer = new ComponentSelectorSerializer();
     private final DependencyResultSerializer dependencyResultSerializer = new DependencyResultSerializer();
     private final Set<Long> visitedComponents = new HashSet<Long>();
 
-    public StreamingResolutionResultBuilder(BinaryStore store, Store<ResolvedComponentResult> cache) {
+    public StreamingResolutionResultBuilder(BinaryStore store, Store<ResolvedComponentResult> cache, ImmutableModuleIdentifierFactory moduleIdentifierFactory) {
+        this.componentResultSerializer = new ComponentResultSerializer(moduleIdentifierFactory);
         this.store = store;
         this.cache = cache;
     }
 
     public ResolutionResult complete() {
         BinaryStore.BinaryData data = store.done();
-        RootFactory rootSource = new RootFactory(data, failures, cache);
+        RootFactory rootSource = new RootFactory(data, failures, cache, componentSelectorSerializer, dependencyResultSerializer, componentResultSerializer);
         return new DefaultResolutionResult(rootSource);
     }
 
@@ -97,7 +103,19 @@ public class StreamingResolutionResultBuilder implements DependencyGraphVisitor
     }
 
     @Override
-    public void visitEdge(DependencyGraphNode resolvedConfiguration) {
+    public void visitSelector(final DependencyGraphSelector selector) {
+        store.write(new BinaryStore.WriteAction() {
+            @Override
+            public void write(Encoder encoder) throws IOException {
+                encoder.writeByte(SELECTOR);
+                encoder.writeSmallLong(selector.getResultId());
+                componentSelectorSerializer.write(encoder, selector.getRequested());
+            }
+        });
+    }
+
+    @Override
+    public void visitEdges(DependencyGraphNode resolvedConfiguration) {
         final Long fromComponent = resolvedConfiguration.getOwner().getResultId();
         final Set<? extends DependencyGraphEdge> dependencies = resolvedConfiguration.getOutgoingEdges();
         if (!dependencies.isEmpty()) {
@@ -106,10 +124,10 @@ public class StreamingResolutionResultBuilder implements DependencyGraphVisitor
                     encoder.writeByte(DEPENDENCY);
                     encoder.writeSmallLong(fromComponent);
                     encoder.writeSmallInt(dependencies.size());
-                    for (DependencyResult dependency : dependencies) {
+                    for (DependencyGraphEdge dependency : dependencies) {
                         dependencyResultSerializer.write(encoder, dependency);
                         if (dependency.getFailure() != null) {
-                            //by keying the failures only be 'requested' we lose some precision
+                            //by keying the failures only by 'requested' we lose some precision
                             //at edge case we'll lose info about a different exception if we have different failure for the same requested version
                             failures.put(dependency.getRequested(), dependency.getFailure());
                         }
@@ -122,19 +140,22 @@ public class StreamingResolutionResultBuilder implements DependencyGraphVisitor
     private static class RootFactory implements Factory<ResolvedComponentResult> {
 
         private final static Logger LOG = Logging.getLogger(RootFactory.class);
-        private final ComponentResultSerializer componentResultSerializer = new ComponentResultSerializer();
+        private final ComponentResultSerializer componentResultSerializer;
 
         private final BinaryStore.BinaryData data;
         private final Map<ComponentSelector, ModuleVersionResolveException> failures;
         private final Store<ResolvedComponentResult> cache;
         private final Object lock = new Object();
-        private final DependencyResultSerializer dependencyResultSerializer = new DependencyResultSerializer();
+        private final ComponentSelectorSerializer componentSelectorSerializer;
+        private final DependencyResultSerializer dependencyResultSerializer;
 
-        public RootFactory(BinaryStore.BinaryData data, Map<ComponentSelector, ModuleVersionResolveException> failures,
-                           Store<ResolvedComponentResult> cache) {
+        RootFactory(BinaryStore.BinaryData data, Map<ComponentSelector, ModuleVersionResolveException> failures, Store<ResolvedComponentResult> cache, ComponentSelectorSerializer componentSelectorSerializer, DependencyResultSerializer dependencyResultSerializer, ComponentResultSerializer componentResultSerializer) {
             this.data = data;
             this.failures = failures;
             this.cache = cache;
+            this.componentResultSerializer = componentResultSerializer;
+            this.componentSelectorSerializer = componentSelectorSerializer;
+            this.dependencyResultSerializer = dependencyResultSerializer;
         }
 
         public ResolvedComponentResult create() {
@@ -162,9 +183,10 @@ public class StreamingResolutionResultBuilder implements DependencyGraphVisitor
         private ResolvedComponentResult deserialize(Decoder decoder) {
             int valuesRead = 0;
             byte type = -1;
-            Clock clock = new Clock();
+            Timer clock = Timers.startTimer();
             try {
                 DefaultResolutionResultBuilder builder = new DefaultResolutionResultBuilder();
+                Map<Long, ComponentSelector> selectors = new HashMap<Long, ComponentSelector>();
                 while (true) {
                     type = decoder.readByte();
                     valuesRead++;
@@ -173,18 +195,23 @@ public class StreamingResolutionResultBuilder implements DependencyGraphVisitor
                             // Last entry, complete the result
                             Long rootId = decoder.readSmallLong();
                             ResolvedComponentResult root = builder.complete(rootId).getRoot();
-                            LOG.debug("Loaded resolution results ({}) from {}", clock.getTime(), data);
+                            LOG.debug("Loaded resolution results ({}) from {}", clock.getElapsed(), data);
                             return root;
                         case COMPONENT:
                             ComponentResult component = componentResultSerializer.read(decoder);
                             builder.visitComponent(component);
                             break;
+                        case SELECTOR:
+                            Long id = decoder.readSmallLong();
+                            ComponentSelector selector = componentSelectorSerializer.read(decoder);
+                            selectors.put(id, selector);
+                            break;
                         case DEPENDENCY:
                             Long fromId = decoder.readSmallLong();
                             int size = decoder.readSmallInt();
                             List<DependencyResult> deps = new ArrayList<DependencyResult>(size);
                             for (int i = 0; i < size; i++) {
-                                deps.add(dependencyResultSerializer.read(decoder, failures));
+                                deps.add(dependencyResultSerializer.read(decoder, selectors, failures));
                             }
                             builder.visitOutgoingEdges(fromId, deps);
                             break;
@@ -193,7 +220,7 @@ public class StreamingResolutionResultBuilder implements DependencyGraphVisitor
                     }
                 }
             } catch (IOException e) {
-                throw new RuntimeException("Problems loading the resolution results (" + clock.getTime() + "). "
+                throw new RuntimeException("Problems loading the resolution results (" + clock.getElapsed() + "). "
                         + "Read " + valuesRead + " values, last was: " + type, e);
             }
         }
