@@ -17,11 +17,16 @@
 package org.gradle.api.internal.artifacts.transform;
 
 import org.gradle.api.Transformer;
-import org.gradle.api.attributes.AttributeContainer;
+import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.ResolvedArtifactSet;
+import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.ResolvedVariant;
+import org.gradle.api.internal.attributes.AttributeContainerInternal;
 
-import java.io.File;
-import java.util.List;
+import java.util.Collection;
 
-interface ArtifactTransforms {
-    Transformer<List<File>, File> getTransform(AttributeContainer from, AttributeContainer to);
+// TODO:DAZ Rename
+public interface ArtifactTransforms {
+    /**
+     * Returns a selector that selects the variant matching the supplied attributes, or which can be transformed to match.
+     */
+    Transformer<ResolvedArtifactSet, Collection<? extends ResolvedVariant>> variantSelector(AttributeContainerInternal attributes);
 }
