@@ -24,6 +24,8 @@ import org.gradle.api.artifacts.component.ModuleComponentIdentifier;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.excludes.ModuleExclusion;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.excludes.ModuleExclusions;
 import org.gradle.api.internal.attributes.AttributeContainerInternal;
+import org.gradle.api.internal.attributes.AttributesSchemaInternal;
+import org.gradle.api.internal.attributes.ImmutableAttributes;
 import org.gradle.internal.component.external.descriptor.Artifact;
 import org.gradle.internal.component.external.descriptor.Configuration;
 import org.gradle.internal.component.external.descriptor.ModuleDescriptorState;
@@ -97,47 +99,68 @@ abstract class AbstractModuleComponentResolveMetadata implements ModuleComponent
         configurations = metadata.configurations;
     }
 
+    @Nullable
+    @Override
+    public AttributesSchemaInternal getAttributesSchema() {
+        return null;
+    }
+
+    @Override
     public ModuleDescriptorState getDescriptor() {
         return descriptor;
     }
 
+    @Override
     public boolean isChanging() {
         return changing;
     }
 
+    @Override
     public boolean isGenerated() {
         return descriptor.isGenerated();
     }
 
+    @Override
     public String getStatus() {
         return status;
     }
 
+    @Override
     public List<String> getStatusScheme() {
         return statusScheme;
     }
 
+    @Override
     public ModuleComponentIdentifier getComponentId() {
         return componentIdentifier;
     }
 
+    @Override
     public ModuleVersionIdentifier getId() {
         return moduleVersionIdentifier;
     }
 
+    @Override
     public ModuleSource getSource() {
         return moduleSource;
     }
 
+    @Override
     public Set<String> getConfigurationNames() {
         return configurations.keySet();
     }
 
+    @Override
+    public List<? extends ConfigurationMetadata> getConsumableConfigurationsHavingAttributes() {
+        return Collections.emptyList();
+    }
+
     @Override
     public String toString() {
         return componentIdentifier.getDisplayName();
     }
 
+    @Override
     public ModuleComponentArtifactMetadata artifact(String type, @Nullable String extension, @Nullable String classifier) {
         IvyArtifactName ivyArtifactName = new DefaultIvyArtifactName(getId().getName(), type, extension, classifier);
         return new DefaultModuleComponentArtifactMetadata(getComponentId(), ivyArtifactName);
@@ -168,6 +191,7 @@ abstract class AbstractModuleComponentResolveMetadata implements ModuleComponent
         return artifacts;
     }
 
+    @Override
     public List<? extends DependencyMetadata> getDependencies() {
         return dependencies;
     }
@@ -177,6 +201,7 @@ abstract class AbstractModuleComponentResolveMetadata implements ModuleComponent
         return configurationDefinitions;
     }
 
+    @Override
     public DefaultConfigurationMetadata getConfiguration(final String name) {
         return configurations.get(name);
     }
@@ -241,10 +266,11 @@ abstract class AbstractModuleComponentResolveMetadata implements ModuleComponent
         private final List<DefaultConfigurationMetadata> parents;
         private final List<DependencyMetadata> configDependencies = new ArrayList<DependencyMetadata>();
         private final Set<ComponentArtifactMetadata> artifacts = new LinkedHashSet<ComponentArtifactMetadata>();
-        private final ModuleExclusion exclusions;
         private final boolean transitive;
         private final boolean visible;
         private final Set<String> hierarchy;
+        private final List<Exclude> excludes;
+        private ModuleExclusion exclusions;
 
         private DefaultConfigurationMetadata(ModuleComponentIdentifier componentId, String name, boolean transitive, boolean visible, List<DefaultConfigurationMetadata> parents, List<Exclude> excludes) {
             this.componentId = componentId;
@@ -253,7 +279,7 @@ abstract class AbstractModuleComponentResolveMetadata implements ModuleComponent
             this.transitive = transitive;
             this.visible = visible;
             this.hierarchy = calculateHierarchy();
-            this.exclusions = filterExcludes(excludes);
+            this.excludes = excludes;
         }
 
         private DefaultConfigurationMetadata(ModuleComponentIdentifier componentId, String name, boolean transitive, boolean visible, List<Exclude> excludes) {
@@ -265,6 +291,7 @@ abstract class AbstractModuleComponentResolveMetadata implements ModuleComponent
             return componentId + ":" + name;
         }
 
+        @Override
         public String getName() {
             return name;
         }
@@ -278,7 +305,7 @@ abstract class AbstractModuleComponentResolveMetadata implements ModuleComponent
             if (parents == null) {
                 return Collections.singleton(name);
             }
-            Set<String> hierarchy = new LinkedHashSet<String>(1+parents.size());
+            Set<String> hierarchy = new LinkedHashSet<String>(1 + parents.size());
             populateHierarchy(hierarchy);
             return hierarchy;
         }
@@ -292,17 +319,19 @@ abstract class AbstractModuleComponentResolveMetadata implements ModuleComponent
             }
         }
 
+        @Override
         public boolean isTransitive() {
             return transitive;
         }
 
+        @Override
         public boolean isVisible() {
             return visible;
         }
 
         @Override
         public AttributeContainerInternal getAttributes() {
-            return AttributeContainerInternal.EMPTY;
+            return ImmutableAttributes.EMPTY;
         }
 
         @Override
@@ -315,6 +344,7 @@ abstract class AbstractModuleComponentResolveMetadata implements ModuleComponent
             return false;
         }
 
+        @Override
         public List<DependencyMetadata> getDependencies() {
             return configDependencies;
         }
@@ -349,11 +379,15 @@ abstract class AbstractModuleComponentResolveMetadata implements ModuleComponent
             return false;
         }
 
-        public ModuleExclusion getExclusions() {
+        @Override
+        public ModuleExclusion getExclusions(ModuleExclusions moduleExclusions) {
+            if (exclusions == null) {
+                exclusions = filterExcludes(moduleExclusions, excludes);
+            }
             return exclusions;
         }
 
-        private ModuleExclusion filterExcludes(Iterable<Exclude> excludes) {
+        private ModuleExclusion filterExcludes(ModuleExclusions exclusions, Iterable<Exclude> excludes) {
             Set<String> hierarchy = getHierarchy();
             List<Exclude> filtered = Lists.newArrayList();
             for (Exclude exclude : excludes) {
@@ -364,9 +398,10 @@ abstract class AbstractModuleComponentResolveMetadata implements ModuleComponent
                     }
                 }
             }
-            return ModuleExclusions.excludeAny(filtered);
+            return exclusions.excludeAny(filtered);
         }
 
+        @Override
         public Set<ComponentArtifactMetadata> getArtifacts() {
             return artifacts;
         }
@@ -376,6 +411,7 @@ abstract class AbstractModuleComponentResolveMetadata implements ModuleComponent
             return ImmutableSet.of(new DefaultVariantMetadata(getAttributes(), getArtifacts()));
         }
 
+        @Override
         public ModuleComponentArtifactMetadata artifact(IvyArtifactName artifact) {
             return new DefaultModuleComponentArtifactMetadata(componentId, artifact);
         }
