@@ -16,7 +16,11 @@
 
 package org.gradle.internal.component.local.model;
 
+import org.gradle.api.Nullable;
 import org.gradle.internal.component.model.ComponentResolveMetadata;
 
 public interface LocalComponentMetadata extends ComponentResolveMetadata {
+    @Nullable
+    @Override
+    LocalConfigurationMetadata getConfiguration(String name);
 }
