@@ -271,7 +271,7 @@ public class EclipsePlugin extends IdePlugin {
     }
 
     private static void configureJavaClasspath(final Project project, GenerateEclipseClasspath task) {
-        task.getClasspath().setPlusConfigurations(Lists.newArrayList(project.getConfigurations().getByName("testRuntime"), project.getConfigurations().getByName("compileClasspath"), project.getConfigurations().getByName("testCompileClasspath")));
+        task.getClasspath().setPlusConfigurations(Lists.newArrayList(project.getConfigurations().getByName("compileClasspath"), project.getConfigurations().getByName("runtimeClasspath"), project.getConfigurations().getByName("testCompileClasspath"), project.getConfigurations().getByName("testRuntimeClasspath")));
         ((IConventionAware) task.getClasspath()).getConventionMapping().map("classFolders", new Callable<List<File>>() {
             @Override
             public List<File> call() {
@@ -353,7 +353,7 @@ public class EclipsePlugin extends IdePlugin {
                         conventionMapping.map("javaRuntimeName", new Callable<String>() {
                             @Override
                             public String call() {
-                                return "JavaSE-" + project.getConvention().getPlugin(JavaPluginConvention.class).getTargetCompatibility();
+                                return eclipseJavaRuntimeNameFor(project.getConvention().getPlugin(JavaPluginConvention.class).getTargetCompatibility());
                             }
 
                         });
@@ -365,6 +365,22 @@ public class EclipsePlugin extends IdePlugin {
         });
     }
 
+    private static String eclipseJavaRuntimeNameFor(JavaVersion version) {
+        // Default Eclipse JRE paths:
+        // https://github.com/eclipse/eclipse.jdt.debug/blob/master/org.eclipse.jdt.launching/plugin.xml#L241-L303
+        switch (version) {
+            case VERSION_1_1:
+                return "JRE-1.1";
+            case VERSION_1_2:
+            case VERSION_1_3:
+            case VERSION_1_4:
+            case VERSION_1_5:
+                return "J2SE-" + version;
+            default:
+                return "JavaSE-" + version;
+        }
+    }
+
     private void applyEclipseWtpPluginOnWebProjects(Project project) {
         Action<Plugin<Project>> action = createActionApplyingEclipseWtpPlugin();
         project.getPlugins().withType(WarPlugin.class, action);
