@@ -17,7 +17,7 @@
 package org.gradle.plugins.ide.eclipse.model;
 
 import groovy.lang.Closure;
-import groovy.lang.GroovyObjectSupport;
+import org.gradle.api.Action;
 import org.gradle.api.Incubating;
 import org.gradle.api.JavaVersion;
 import org.gradle.plugins.ide.api.PropertiesFileContentMerger;
@@ -58,7 +58,7 @@ import org.gradle.util.ConfigureUtil;
  * }
  * </pre>
  */
-public class EclipseJdt extends GroovyObjectSupport {
+public class EclipseJdt {
 
     private JavaVersion sourceCompatibility = JavaVersion.current();
 
@@ -120,7 +120,7 @@ public class EclipseJdt extends GroovyObjectSupport {
     }
 
     /**
-     * See {@link #file(Closure) }
+     * See {@link #file(Action) }
      */
     public PropertiesFileContentMerger getFile() {
         return file;
@@ -139,4 +139,20 @@ public class EclipseJdt extends GroovyObjectSupport {
     public void file(Closure closure) {
         ConfigureUtil.configure(closure, file);
     }
+
+    /**
+     * Enables advanced configuration like affecting the way existing jdt file content
+     * is merged with gradle build information
+     * <p>
+     * The object passed to whenMerged{} and beforeMerged{} actions is of type {@link Jdt}
+     * <p>
+     * The object passed to withProperties{} actions is of type {@link java.util.Properties}
+     * <p>
+     * For example see docs for {@link EclipseJdt}
+     *
+     * @since 3.5
+     */
+    public void file(Action<? super PropertiesFileContentMerger> action) {
+        action.execute(file);
+    }
 }
