@@ -18,6 +18,7 @@ package org.gradle.plugins.ide.eclipse.model;
 import com.google.common.collect.Iterables;
 import com.google.common.collect.Lists;
 import groovy.lang.Closure;
+import org.gradle.api.Action;
 import org.gradle.plugins.ide.api.XmlFileContentMerger;
 import org.gradle.util.ConfigureUtil;
 
@@ -25,6 +26,8 @@ import java.util.Collections;
 import java.util.List;
 import java.util.Map;
 
+import static org.gradle.util.ConfigureUtil.configure;
+
 /**
  * Enables fine-tuning wtp facet details of the Eclipse plugin
  * <p>
@@ -79,7 +82,7 @@ public class EclipseWtpFacet {
     }
 
     /**
-     * See {@link #file(Closure) }
+     * See {@link #file(Action) }
      */
     public XmlFileContentMerger getFile() {
         return file;
@@ -95,7 +98,20 @@ public class EclipseWtpFacet {
      * For example see docs for {@link EclipseWtpFacet}
      */
     public void file(Closure closure) {
-        ConfigureUtil.configure(closure, file);
+        configure(closure, file);
+    }
+
+    /**
+     * Enables advanced configuration like tinkering with the output XML
+     * or affecting the way existing wtp facet file content is merged with gradle build information.
+     * <p>
+     *
+     * For example see docs for {@link EclipseWtpFacet}
+     *
+     * @since 3.5
+     */
+    public void file(Action<? super XmlFileContentMerger> action) {
+        action.execute(file);
     }
 
     /**
