@@ -25,12 +25,11 @@ import org.gradle.api.Transformer;
 import org.gradle.api.UncheckedIOException;
 import org.gradle.api.internal.ClosureBackedAction;
 import org.gradle.api.internal.initialization.DefaultClassLoaderScope;
-import org.gradle.api.internal.initialization.loadercache.ClassLoaderCache;
 import org.gradle.api.logging.Logger;
 import org.gradle.api.logging.Logging;
 import org.gradle.integtests.fixtures.daemon.DaemonLogsAnalyzer;
+import org.gradle.internal.ImmutableActionSet;
 import org.gradle.internal.UncheckedException;
-import org.gradle.internal.concurrent.CompositeStoppable;
 import org.gradle.internal.featurelifecycle.LoggingDeprecatedFeatureHandler;
 import org.gradle.internal.jvm.Jvm;
 import org.gradle.internal.logging.services.LoggingServiceRegistry;
@@ -40,7 +39,7 @@ import org.gradle.internal.service.ServiceRegistryBuilder;
 import org.gradle.internal.service.scopes.GlobalScopeServices;
 import org.gradle.launcher.daemon.configuration.DaemonParameters;
 import org.gradle.launcher.daemon.configuration.GradleProperties;
-import org.gradle.listener.ActionBroadcast;
+import org.gradle.internal.MutableActionSet;
 import org.gradle.process.internal.streams.SafeStreams;
 import org.gradle.test.fixtures.file.TestDirectoryProvider;
 import org.gradle.test.fixtures.file.TestFile;
@@ -59,15 +58,14 @@ import java.util.Arrays;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.LinkedHashMap;
-import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Set;
 
-import static org.gradle.internal.service.scopes.DefaultGradleUserHomeScopeServiceRegistry.REUSE_USER_HOME_SERVICES;
 import static org.gradle.integtests.fixtures.executer.AbstractGradleExecuter.CliDaemonArgument.*;
 import static org.gradle.integtests.fixtures.executer.OutputScrapingExecutionResult.STACK_TRACE_ELEMENT;
+import static org.gradle.internal.service.scopes.DefaultGradleUserHomeScopeServiceRegistry.REUSE_USER_HOME_SERVICES;
 import static org.gradle.util.CollectionUtils.collect;
 import static org.gradle.util.CollectionUtils.join;
 
@@ -124,13 +122,14 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
     private final List<String> commandLineJvmOpts = new ArrayList<String>();
     private boolean useOnlyRequestedJvmOpts;
     private boolean requiresGradleDistribution;
+    private boolean useOwnUserHomeServices;
 
     private int expectedDeprecationWarnings;
     private boolean eagerClassLoaderCreationChecksOn = true;
     private boolean stackTraceChecksOn = true;
 
-    private final ActionBroadcast<GradleExecuter> beforeExecute = new ActionBroadcast<GradleExecuter>();
-    private final Set<Action<? super GradleExecuter>> afterExecute = new LinkedHashSet<Action<? super GradleExecuter>>();
+    private final MutableActionSet<GradleExecuter> beforeExecute = new MutableActionSet<GradleExecuter>();
+    private ImmutableActionSet<GradleExecuter> afterExecute = ImmutableActionSet.empty();
 
     private final TestDirectoryProvider testDirectoryProvider;
     protected final GradleVersion gradleVersion;
@@ -221,11 +220,11 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
     }
 
     public void afterExecute(Action<? super GradleExecuter> action) {
-        afterExecute.add(action);
+        afterExecute = afterExecute.add(action);
     }
 
     public void afterExecute(@DelegatesTo(GradleExecuter.class) Closure action) {
-        afterExecute.add(new ClosureBackedAction<GradleExecuter>(action));
+        afterExecute(new ClosureBackedAction<GradleExecuter>(action));
     }
 
     public GradleExecuter inDirectory(File directory) {
@@ -300,7 +299,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         executer.withCommandLineGradleOpts(commandLineJvmOpts);
         executer.withBuildJvmOpts(buildJvmOpts);
         if (useOnlyRequestedJvmOpts) {
-            executer.useDefaultBuildJvmArgs();
+            executer.useOnlyRequestedJvmOpts();
         }
         executer.noExtraLogging();
 
@@ -316,6 +315,9 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         if (requiresGradleDistribution) {
             executer.requireGradleDistribution();
         }
+        if (useOwnUserHomeServices) {
+            executer.withOwnUserHomeServices();
+        }
         if (requireDaemon) {
             executer.requireDaemon();
         }
@@ -602,7 +604,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return this;
     }
 
-    public GradleExecuter useDefaultBuildJvmArgs() {
+    public GradleExecuter useOnlyRequestedJvmOpts() {
         useOnlyRequestedJvmOpts = true;
         return this;
     }
@@ -616,6 +618,11 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return withDaemonBaseDir(testDirectoryProvider.getTestDirectory().file("daemon"));
     }
 
+    @Override
+    public GradleExecuter withWorkerDaemonsExpirationDisabled() {
+        return withArgument("-Dorg.gradle.workers.internal.disable-daemons-expiration=true");
+    }
+
     public boolean usesSharedDaemons() {
         return isSharedDaemons();
     }
@@ -642,6 +649,12 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return requireDaemon;
     }
 
+    @Override
+    public GradleExecuter withOwnUserHomeServices() {
+        useOwnUserHomeServices = true;
+        return this;
+    }
+
     /**
      * Performs cleanup at completion of the test.
      */
@@ -659,12 +672,6 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         }
     }
 
-    // gets called by reflection from AbstractTestDirectoryProvider#closeCachedClassLoaders method
-    public static void cleanupCachedClassLoaders() {
-        ClassLoaderCache classLoaderCache = GLOBAL_SERVICES.get(ClassLoaderCache.class);
-        CompositeStoppable.stoppable(classLoaderCache).stop();
-    }
-
     enum CliDaemonArgument {
         NOT_DEFINED,
         DAEMON,
@@ -776,7 +783,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         }
         properties.put(LoggingDeprecatedFeatureHandler.ORG_GRADLE_DEPRECATION_TRACE_PROPERTY_NAME, Boolean.toString(fullDeprecationStackTrace));
 
-        if (gradleUserHomeDir != null && !gradleUserHomeDir.equals(buildContext.getGradleUserHomeDir())) {
+        if (useOwnUserHomeServices || (gradleUserHomeDir != null && !gradleUserHomeDir.equals(buildContext.getGradleUserHomeDir()))) {
             properties.put(REUSE_USER_HOME_SERVICES, "false");
         }
         if (!noExplicitTmpDir) {
@@ -837,7 +844,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
 
     private void finished() {
         try {
-            new ActionBroadcast<GradleExecuter>(afterExecute).execute(this);
+            afterExecute.execute(this);
         } finally {
             reset();
         }
@@ -898,12 +905,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
 
     @Override
     public GradleExecuter withBuildCacheEnabled() {
-        return withArgument("-Dorg.gradle.cache.tasks=true");
-    }
-
-    @Override
-    public GradleExecuter withLocalBuildCache(File cacheDir) {
-        return withBuildCacheEnabled().withArgument("-Dorg.gradle.cache.tasks.directory=" + cacheDir.getAbsolutePath());
+        return withArgument("--build-cache");
     }
 
     protected Action<ExecutionResult> getResultAssertion() {
