@@ -23,7 +23,6 @@ import org.gradle.api.Task;
 import org.gradle.api.execution.TaskExecutionGraph;
 import org.gradle.api.execution.TaskExecutionGraphListener;
 import org.gradle.api.execution.TaskExecutionListener;
-import org.gradle.api.internal.changedetection.state.InMemoryTaskArtifactCache;
 import org.gradle.api.internal.classpath.ModuleRegistry;
 import org.gradle.api.internal.file.TestFiles;
 import org.gradle.api.logging.StandardOutputListener;
@@ -262,11 +261,6 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
         parametersConverter.configure(parser);
         final Parameters parameters = new Parameters(startParameter);
         parametersConverter.convert(parser.parse(getAllArgs()), parameters);
-        if (parameters.getDaemonParameters().isStop()) {
-            // --stop should simulate stopping the daemon
-            cleanupCachedClassLoaders();
-            GLOBAL_SERVICES.get(InMemoryTaskArtifactCache.class).invalidateAll();
-        }
 
         BuildActionExecuter<BuildActionParameters> actionExecuter = GLOBAL_SERVICES.get(BuildActionExecuter.class);
 
