@@ -34,10 +34,12 @@ public class OutputScrapingExecutionFailure extends OutputScrapingExecutionResul
     private static final Pattern DESCRIPTION_PATTERN = Pattern.compile("(?ms)^\\* What went wrong:$(.+?)^\\* Try:$");
     private static final Pattern LOCATION_PATTERN = Pattern.compile("(?ms)^\\* Where:((.+)'.+') line: (\\d+)$");
     private static final Pattern RESOLUTION_PATTERN = Pattern.compile("(?ms)^\\* Try:$(.+?)^\\* Exception is:$");
+    private static final Pattern EXCEPTION_PATTERN = Pattern.compile("(?ms)^\\* Exception is:$(.+?):(.+?)$");
     private final String description;
     private final String lineNumber;
     private final String fileName;
     private final String resolution;
+    private final Exception exception;
     // with normalized line endings
     private final List<String> causes = new ArrayList<String>();
 
@@ -81,6 +83,15 @@ public class OutputScrapingExecutionFailure extends OutputScrapingExecutionResul
         } else {
             resolution = matcher.group(1).trim();
         }
+
+        matcher = EXCEPTION_PATTERN.matcher(error);
+        if (!matcher.find()) {
+            exception = null;
+        } else {
+            String exceptionClass = matcher.group(1).trim();
+            String exceptionMessage = matcher.group(2).trim();
+            exception = recreateException(exceptionClass, exceptionMessage);
+        }
     }
 
     private Problem extract(String problem) {
@@ -108,6 +119,14 @@ public class OutputScrapingExecutionFailure extends OutputScrapingExecutionResul
         return new Problem(description, causes);
     }
 
+    private Exception recreateException(String className, String message) {
+        try {
+            return (Exception) Class.forName(className).getConstructor(String.class).newInstance(message);
+        } catch (Exception e) {
+            return new Exception(message);
+        }
+    }
+
     private String toPrefixPattern(int prefix) {
         StringBuilder builder = new StringBuilder("(?m)^");
         for (int i = 0; i < prefix; i++) {
@@ -170,6 +189,10 @@ public class OutputScrapingExecutionFailure extends OutputScrapingExecutionResul
         return new DependencyResolutionFailure(this, configurationPath);
     }
 
+    public Exception getException() {
+        return exception;
+    }
+
     private static class Problem {
         final String description;
         final List<String> causes;
