@@ -20,26 +20,24 @@ import com.google.common.base.Joiner;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
 import org.gradle.internal.UncheckedException;
-import org.gradle.performance.measure.DataAmount;
 import org.gradle.performance.measure.Duration;
 import org.gradle.performance.measure.MeasuredOperation;
 import org.gradle.util.GradleVersion;
 import org.joda.time.LocalDate;
 
 import java.io.Closeable;
-import java.math.BigDecimal;
 import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
 import java.sql.Timestamp;
-import java.sql.Types;
 import java.text.DateFormat;
 import java.text.ParseException;
 import java.text.SimpleDateFormat;
 import java.util.ArrayList;
 import java.util.Comparator;
+import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -54,6 +52,7 @@ import static org.gradle.performance.results.ResultsStoreHelper.toArray;
 public class CrossVersionResultsStore implements DataReporter<CrossVersionPerformanceResults>, ResultsStore, Closeable {
     private final long ignoreV17Before;
     private final PerformanceDatabase db;
+    private final Map<String, GradleVersion> gradleVersionCache = new HashMap<String, GradleVersion>();
 
     public CrossVersionResultsStore() {
         this("results");
@@ -106,7 +105,7 @@ public class CrossVersionResultsStore implements DataReporter<CrossVersionPerfor
                         closeResultSet(keys);
                     }
                     try {
-                        statement = connection.prepareStatement("insert into testOperation(testExecution, version, totalTime, configurationTime, executionTime, heapUsageBytes, totalHeapUsageBytes, maxHeapUsageBytes, maxUncollectedHeapBytes, maxCommittedHeapBytes, compileTotalTime, gcTotalTime) values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
+                        statement = connection.prepareStatement("insert into testOperation(testExecution, version, totalTime) values (?, ?, ?)");
                         addOperations(statement, testId, null, results.getCurrent());
                         for (BaselineVersion baselineVersion : results.getBaselineVersions()) {
                             addOperations(statement, testId, baselineVersion.getVersion(), baselineVersion.getResults());
@@ -138,23 +137,6 @@ public class CrossVersionResultsStore implements DataReporter<CrossVersionPerfor
             statement.setLong(1, testId);
             statement.setString(2, version);
             statement.setBigDecimal(3, operation.getTotalTime().toUnits(Duration.MILLI_SECONDS).getValue());
-            statement.setBigDecimal(4, operation.getConfigurationTime().toUnits(Duration.MILLI_SECONDS).getValue());
-            statement.setBigDecimal(5, operation.getExecutionTime().toUnits(Duration.MILLI_SECONDS).getValue());
-            statement.setBigDecimal(6, operation.getTotalMemoryUsed().toUnits(DataAmount.BYTES).getValue());
-            statement.setBigDecimal(7, operation.getTotalHeapUsage().toUnits(DataAmount.BYTES).getValue());
-            statement.setBigDecimal(8, operation.getMaxHeapUsage().toUnits(DataAmount.BYTES).getValue());
-            statement.setBigDecimal(9, operation.getMaxUncollectedHeap().toUnits(DataAmount.BYTES).getValue());
-            statement.setBigDecimal(10, operation.getMaxCommittedHeap().toUnits(DataAmount.BYTES).getValue());
-            if (operation.getCompileTotalTime() != null) {
-                statement.setBigDecimal(11, operation.getCompileTotalTime().toUnits(Duration.MILLI_SECONDS).getValue());
-            } else {
-                statement.setNull(11, Types.DECIMAL);
-            }
-            if (operation.getGcTotalTime() != null) {
-                statement.setBigDecimal(12, operation.getGcTotalTime().toUnits(Duration.MILLI_SECONDS).getValue());
-            } else {
-                statement.setNull(12, Types.DECIMAL);
-            }
             statement.addBatch();
         }
     }
@@ -192,6 +174,7 @@ public class CrossVersionResultsStore implements DataReporter<CrossVersionPerfor
         return getTestResults(testName, Integer.MAX_VALUE, Integer.MAX_VALUE, channel);
     }
 
+
     @Override
     public CrossVersionPerformanceTestHistory getTestResults(final String testName, final int mostRecentN, final int maxDaysOld, final String channel) {
         try {
@@ -200,7 +183,7 @@ public class CrossVersionResultsStore implements DataReporter<CrossVersionPerfor
                     Map<Long, CrossVersionPerformanceResults> results = Maps.newLinkedHashMap();
                     Set<String> allVersions = new TreeSet<String>(new Comparator<String>() {
                         public int compare(String o1, String o2) {
-                            return GradleVersion.version(o1).compareTo(GradleVersion.version(o2));
+                            return resolveGradleVersion(o1).compareTo(resolveGradleVersion(o2));
                         }
                     });
                     Set<String> allBranches = new TreeSet<String>();
@@ -241,9 +224,13 @@ public class CrossVersionResultsStore implements DataReporter<CrossVersionPerfor
                             allBranches.add(performanceResults.getVcsBranch());
                         }
 
-                        operationsForExecution = connection.prepareStatement("select version, testExecution, totalTime, configurationTime, executionTime, heapUsageBytes, totalHeapUsageBytes, maxHeapUsageBytes, maxUncollectedHeapBytes, maxCommittedHeapBytes, compileTotalTime, gcTotalTime from testOperation where testExecution in (select * from table(x int = ?))");
+                        operationsForExecution = connection.prepareStatement("select version, testExecution, totalTime from testOperation "
+                                + "where testExecution in (select top ? id from testExecution where testId = ? and startTime >= ? and channel = ? order by startTime desc)");
                         operationsForExecution.setFetchSize(10 * results.size());
-                        operationsForExecution.setObject(1, results.keySet().toArray());
+                        operationsForExecution.setInt(1, mostRecentN);
+                        operationsForExecution.setString(2, testName);
+                        operationsForExecution.setTimestamp(3, minDate);
+                        operationsForExecution.setString(4, channel);
 
                         operations = operationsForExecution.executeQuery();
                         while (operations.next()) {
@@ -255,21 +242,6 @@ public class CrossVersionResultsStore implements DataReporter<CrossVersionPerfor
                             }
                             MeasuredOperation operation = new MeasuredOperation();
                             operation.setTotalTime(Duration.millis(operations.getBigDecimal(3)));
-                            operation.setConfigurationTime(Duration.millis(operations.getBigDecimal(4)));
-                            operation.setExecutionTime(Duration.millis(operations.getBigDecimal(5)));
-                            operation.setTotalMemoryUsed(DataAmount.bytes(operations.getBigDecimal(6)));
-                            operation.setTotalHeapUsage(DataAmount.bytes(operations.getBigDecimal(7)));
-                            operation.setMaxHeapUsage(DataAmount.bytes(operations.getBigDecimal(8)));
-                            operation.setMaxUncollectedHeap(DataAmount.bytes(operations.getBigDecimal(9)));
-                            operation.setMaxCommittedHeap(DataAmount.bytes(operations.getBigDecimal(10)));
-                            BigDecimal compileTotalTime = operations.getBigDecimal(11);
-                            if (compileTotalTime != null) {
-                                operation.setCompileTotalTime(Duration.millis(compileTotalTime));
-                            }
-                            BigDecimal gcTotalTime = operations.getBigDecimal(12);
-                            if (gcTotalTime != null) {
-                                operation.setGcTotalTime(Duration.millis(gcTotalTime));
-                            }
 
                             if (version == null) {
                                 result.getCurrent().add(operation);
@@ -295,6 +267,15 @@ public class CrossVersionResultsStore implements DataReporter<CrossVersionPerfor
         }
     }
 
+    protected GradleVersion resolveGradleVersion(String version) {
+        GradleVersion gradleVersion = gradleVersionCache.get(version);
+        if (gradleVersion == null) {
+            gradleVersion = GradleVersion.version(version);
+            gradleVersionCache.put(version, gradleVersion);
+        }
+        return gradleVersion;
+    }
+
     public void close() {
         db.close();
     }
@@ -306,63 +287,39 @@ public class CrossVersionResultsStore implements DataReporter<CrossVersionPerfor
 
             try {
                 statement = connection.createStatement();
-                statement.execute("create table if not exists testExecution (id bigint identity not null, testId varchar not null, executionTime timestamp not null, targetVersion varchar not null, testProject varchar not null, tasks array not null, args array not null, operatingSystem varchar not null, jvm varchar not null)");
-                statement.execute("create table if not exists testOperation (testExecution bigint not null, version varchar, executionTimeMs decimal not null, heapUsageBytes decimal not null, foreign key(testExecution) references testExecution(id))");
+                statement.execute("create table if not exists testExecution (id bigint identity not null, testId varchar not null, startTime timestamp not null, targetVersion varchar not null, testProject varchar not null, tasks array not null, args array not null, operatingSystem varchar not null, jvm varchar not null)");
+                statement.execute("create table if not exists testOperation (testExecution bigint not null, version varchar, totalTime decimal not null, foreign key(testExecution) references testExecution(id))");
                 statement.execute("alter table testExecution add column if not exists vcsBranch varchar not null default 'master'");
                 statement.execute("alter table testExecution add column if not exists vcsCommit varchar");
                 statement.execute("alter table testExecution add column if not exists gradleOpts array");
                 statement.execute("alter table testExecution add column if not exists daemon boolean");
-                statement.execute("alter table testOperation add column if not exists totalHeapUsageBytes decimal");
-                statement.execute("alter table testOperation add column if not exists maxHeapUsageBytes decimal");
-                statement.execute("alter table testOperation add column if not exists maxUncollectedHeapBytes decimal");
-                statement.execute("alter table testOperation add column if not exists maxCommittedHeapBytes decimal");
-                statement.execute("alter table testOperation add column if not exists compileTotalTime decimal");
-                statement.execute("alter table testOperation add column if not exists gcTotalTime decimal");
-                if (columnExists(connection, "TESTOPERATION", "EXECUTIONTIMEMS")) {
+                if (DataBaseSchemaUtil.columnExists(connection, "TESTOPERATION", "EXECUTIONTIMEMS")) {
                     statement.execute("alter table testOperation alter column executionTimeMs rename to totalTime");
-                    statement.execute("alter table testOperation add column executionTime decimal");
-                    statement.execute("update testOperation set executionTime = 0");
-                    statement.execute("alter table testOperation alter column executionTime set not null");
-                    statement.execute("alter table testOperation add column configurationTime decimal");
-                    statement.execute("update testOperation set configurationTime = 0");
-                    statement.execute("alter table testOperation alter column configurationTime set not null");
                 }
-                statement.execute("create index if not exists testExecution_testId on testExecution (testId)");
-                statement.execute("create index if not exists testExecution_executionTime on testExecution (executionTime desc)");
-                if (columnExists(connection, "TESTEXECUTION", "EXECUTIONTIME")) {
+                if (DataBaseSchemaUtil.columnExists(connection, "TESTEXECUTION", "EXECUTIONTIME")) {
                     statement.execute("alter table testExecution alter column executionTime rename to startTime");
                 }
-                if (!columnExists(connection, "TESTEXECUTION", "ENDTIME")) {
+                if (!DataBaseSchemaUtil.columnExists(connection, "TESTEXECUTION", "ENDTIME")) {
                     statement.execute("alter table testExecution add column endTime timestamp");
                     statement.execute("update testExecution set endTime = startTime");
                     statement.execute("alter table testExecution alter column endTime set not null");
                 }
-                if (!columnExists(connection, "TESTEXECUTION", "CHANNEL")) {
+                if (!DataBaseSchemaUtil.columnExists(connection, "TESTEXECUTION", "CHANNEL")) {
                     statement.execute("alter table testExecution add column if not exists channel varchar");
                     statement.execute("update testExecution set channel='commits'");
                     statement.execute("alter table testExecution alter column channel set not null");
                     statement.execute("create index if not exists testExecution_channel on testExecution (channel)");
                 }
+                statement.execute("create index if not exists testExecution_testId on testExecution (testId)");
+                statement.execute("create index if not exists testExecution_executionTime on testExecution (startTime desc)");
+
+                DataBaseSchemaUtil.removeOutdatedColumnsFromTestDB(connection, statement);
             } finally {
                 closeStatement(statement);
             }
 
             return null;
         }
-
-        private boolean columnExists(Connection connection, String table, String column) throws SQLException {
-            ResultSet columns = null;
-            boolean exists;
-
-            try {
-                columns = connection.getMetaData().getColumns(null, null, table, column);
-                exists = columns.next();
-            } finally {
-                closeResultSet(columns);
-            }
-
-            return exists;
-        }
     }
 
     private void closeStatement(Statement statement) throws SQLException {
