@@ -44,13 +44,9 @@ import org.gradle.language.base.internal.compile.Compiler;
 import org.gradle.util.VersionNumber;
 
 import java.io.File;
-import java.io.IOException;
 import java.io.Serializable;
 import java.lang.reflect.Method;
-import java.net.URL;
 import java.util.Arrays;
-import java.util.Collections;
-import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
@@ -93,9 +89,9 @@ public class ApiGroovyCompiler implements org.gradle.language.base.internal.comp
         VersionNumber version = parseGroovyVersion();
         if (version.compareTo(VersionNumber.parse("2.0")) < 0) {
             // using a transforming classloader is only required for older buggy Groovy versions
-            classPathLoader = new GroovyCompileTransformingClassLoader(getExtClassLoader(), new DefaultClassPath(spec.getClasspath()));
+            classPathLoader = new GroovyCompileTransformingClassLoader(getExtClassLoader(), new DefaultClassPath(spec.getCompileClasspath()));
         } else {
-            classPathLoader = new DefaultClassLoaderFactory().createIsolatedClassLoader(new DefaultClassPath(spec.getClasspath()));
+            classPathLoader = new DefaultClassLoaderFactory().createIsolatedClassLoader(new DefaultClassPath(spec.getCompileClasspath()));
         }
         GroovyClassLoader compileClasspathClassLoader = new GroovyClassLoader(classPathLoader, null);
         GroovySystemLoader compileClasspathLoader = groovySystemLoaderFactory.forClassLoader(classPathLoader);
@@ -115,7 +111,7 @@ public class ApiGroovyCompiler implements org.gradle.language.base.internal.comp
         // can't delegate to compileClasspathLoader because this would result in ASTTransformation interface
         // (which is implemented by the transform class) being loaded by compileClasspathClassLoader (which is
         // where the transform class is loaded from)
-        for (File file : spec.getClasspath()) {
+        for (File file : spec.getCompileClasspath()) {
             astTransformClassLoader.addClasspath(file.getPath());
         }
         JavaAwareCompilationUnit unit = new JavaAwareCompilationUnit(configuration, compileClasspathClassLoader) {
@@ -125,17 +121,21 @@ public class ApiGroovyCompiler implements org.gradle.language.base.internal.comp
             }
         };
 
-        final boolean shouldProcessAnnotations = shouldProcessAnnotations(astTransformClassLoader, spec);
+        final boolean shouldProcessAnnotations = shouldProcessAnnotations(spec);
         if (shouldProcessAnnotations) {
             // If an annotation processor is detected, we need to force Java stub generation, so the we can process annotations on Groovy classes
             // We are forcing stub generation by tricking the groovy compiler into thinking there are java files to compile.
             // All java files are just passed to the compile method of the JavaCompiler and aren't processed internally by the Groovy Compiler.
-            // Since we're maintaining our own list of Java files independent what's passed by the Groovy compiler, adding a non-existant java file
+            // Since we're maintaining our own list of Java files independent of what's passed by the Groovy compiler, adding a non-existent java file
             // to the sources won't cause any issues.
             unit.addSources(new File[]{new File("ForceStubGeneration.java")});
         }
 
-        unit.addSources(Iterables.toArray(spec.getSource(), File.class));
+        // Sort source files to work around https://issues.apache.org/jira/browse/GROOVY-7966
+        File[] sortedSourceFiles = Iterables.toArray(spec.getSource(), File.class);
+        Arrays.sort(sortedSourceFiles);
+        unit.addSources(sortedSourceFiles);
+
         unit.setCompilerFactory(new JavaCompilerFactory() {
             public JavaCompiler createCompiler(final CompilerConfiguration config) {
                 return new JavaCompiler() {
@@ -188,28 +188,8 @@ public class ApiGroovyCompiler implements org.gradle.language.base.internal.comp
         return new SimpleWorkResult(true);
     }
 
-    private boolean shouldProcessAnnotations(ClassLoader classLoader, GroovyJavaJointCompileSpec spec) {
-        return !isAnnotationProcessingDisabled(spec)
-            && (isAnnotationProcessorOnClasspath(classLoader) || isDefaultAnnotationProcessorDiscoveryOverridden(spec));
-    }
-
-    private boolean isAnnotationProcessingDisabled(GroovyJavaJointCompileSpec spec) {
-        List<String> compilerArgs = spec.getCompileOptions().getCompilerArgs();
-        return !spec.getGroovyCompileOptions().isJavaAnnotationProcessing() || compilerArgs.contains("-proc:none");
-    }
-
-    private boolean isAnnotationProcessorOnClasspath(ClassLoader classLoader) {
-        try {
-            Enumeration<URL> processorEntries = classLoader.getResources("META-INF/services/javax.annotation.processing.Processor");
-            return processorEntries.hasMoreElements();
-        } catch (IOException e) {
-            throw new GradleException("Failed to retrieve annotation processor metadata from classpath", e);
-        }
-    }
-
-    private boolean isDefaultAnnotationProcessorDiscoveryOverridden(GroovyJavaJointCompileSpec spec) {
-        List<String> compilerArgs = spec.getCompileOptions().getCompilerArgs();
-        return !Collections.disjoint(compilerArgs, Arrays.asList("-processorpath", "-processor"));
+    private boolean shouldProcessAnnotations(GroovyJavaJointCompileSpec spec) {
+        return spec.getGroovyCompileOptions().isJavaAnnotationProcessing() && !spec.getAnnotationProcessorPath().isEmpty();
     }
 
     private void applyConfigurationScript(File configScript, CompilerConfiguration configuration) {
