@@ -16,8 +16,10 @@
 
 package org.gradle.api.internal.tasks.compile.incremental.analyzer;
 
-import java.io.File;
+import com.google.common.hash.HashCode;
+import org.gradle.api.file.FileTreeElement;
+import org.gradle.api.internal.tasks.compile.incremental.deps.ClassAnalysis;
 
 public interface ClassDependenciesAnalyzer {
-    ClassAnalysis getClassAnalysis(String className, File classFile);
+    ClassAnalysis getClassAnalysis(HashCode classFileHash, FileTreeElement classFile);
 }
