@@ -16,17 +16,24 @@
 
 package org.gradle.api.tasks.compile;
 
+import com.google.common.collect.ImmutableList;
 import org.gradle.api.AntBuilder;
 import org.gradle.api.Incubating;
+import org.gradle.api.JavaVersion;
 import org.gradle.api.Task;
+import org.gradle.api.file.FileCollection;
 import org.gradle.api.file.FileTree;
 import org.gradle.api.internal.changedetection.changes.IncrementalTaskInputsInternal;
 import org.gradle.api.internal.changedetection.state.CachingFileHasher;
 import org.gradle.api.internal.file.FileOperations;
+import org.gradle.api.internal.tasks.CurrentJvmJavaToolChain;
+import org.gradle.api.internal.tasks.JavaHomeBasedJavaToolChain;
+import org.gradle.api.internal.tasks.compile.AnnotationProcessorDetector;
 import org.gradle.api.internal.tasks.compile.CleaningJavaCompiler;
 import org.gradle.api.internal.tasks.compile.DefaultJavaCompileSpec;
 import org.gradle.api.internal.tasks.compile.DefaultJavaCompileSpecFactory;
 import org.gradle.api.internal.tasks.compile.JavaCompileSpec;
+import org.gradle.api.internal.tasks.compile.JavaCompilerFactory;
 import org.gradle.api.internal.tasks.compile.incremental.IncrementalCompilerFactory;
 import org.gradle.api.internal.tasks.compile.incremental.analyzer.ClassAnalysisCache;
 import org.gradle.api.internal.tasks.compile.incremental.cache.CompileCaches;
@@ -36,6 +43,8 @@ import org.gradle.api.internal.tasks.compile.incremental.jar.JarSnapshotCache;
 import org.gradle.api.internal.tasks.compile.incremental.jar.LocalJarClasspathSnapshotStore;
 import org.gradle.api.specs.Spec;
 import org.gradle.api.tasks.CacheableTask;
+import org.gradle.api.tasks.Classpath;
+import org.gradle.api.tasks.CompileClasspath;
 import org.gradle.api.tasks.Internal;
 import org.gradle.api.tasks.Nested;
 import org.gradle.api.tasks.ParallelizableTask;
@@ -46,14 +55,15 @@ import org.gradle.api.tasks.WorkResult;
 import org.gradle.api.tasks.incremental.IncrementalTaskInputs;
 import org.gradle.cache.CacheRepository;
 import org.gradle.internal.Factory;
+import org.gradle.internal.jvm.inspection.JvmVersionDetector;
 import org.gradle.jvm.internal.toolchain.JavaToolChainInternal;
 import org.gradle.jvm.platform.JavaPlatform;
 import org.gradle.jvm.platform.internal.DefaultJavaPlatform;
 import org.gradle.jvm.toolchain.JavaToolChain;
 import org.gradle.language.base.internal.compile.Compiler;
 import org.gradle.language.base.internal.compile.CompilerUtil;
+import org.gradle.process.internal.ExecActionFactory;
 import org.gradle.util.DeprecationLogger;
-import org.gradle.util.SingleMessageLogger;
 
 import javax.inject.Inject;
 import java.io.File;
@@ -63,7 +73,8 @@ import java.io.File;
  *
  * <pre autoTested=''>
  *     apply plugin: 'java'
- *     compileJava {
+ *
+ *     tasks.withType(JavaCompile) {
  *         //enable compilation in a separate daemon process
  *         options.fork = true
  *
@@ -77,9 +88,10 @@ import java.io.File;
 public class JavaCompile extends AbstractCompile {
     private File dependencyCacheDir;
     private final CompileOptions compileOptions = new CompileOptions();
+    private JavaToolChain toolChain;
 
     public JavaCompile() {
-        getOutputs().doNotCacheIf(new Spec<Task>() {
+        getOutputs().doNotCacheIf("Use depend is enabled", new Spec<Task>() {
             @Override
             public boolean isSatisfiedBy(Task task) {
                 return DeprecationLogger.whileDisabled(new Factory<Boolean>() {
@@ -107,10 +119,20 @@ public class JavaCompile extends AbstractCompile {
      *
      * @return The tool chain.
      */
-    @Incubating @Inject
+    @Nested
+    @Incubating
     public JavaToolChain getToolChain() {
-        // Implementation is generated
-        throw new UnsupportedOperationException();
+        if (toolChain != null) {
+            return toolChain;
+        }
+        if (getOptions().isFork()) {
+            ForkOptions forkOptions = getOptions().getForkOptions();
+            File javaHome = forkOptions.getJavaHome();
+            if (javaHome != null) {
+                return new JavaHomeBasedJavaToolChain(javaHome, getJavaCompilerFactory(), getExecActionFactory(), getJvmVersionDetector());
+            }
+        }
+        return new CurrentJvmJavaToolChain(getJavaCompilerFactory(), getExecActionFactory());
     }
 
     /**
@@ -120,8 +142,7 @@ public class JavaCompile extends AbstractCompile {
      */
     @Incubating
     public void setToolChain(JavaToolChain toolChain) {
-        // Implementation is generated
-        throw new UnsupportedOperationException();
+        this.toolChain = toolChain;
     }
 
     @TaskAction
@@ -131,17 +152,19 @@ public class JavaCompile extends AbstractCompile {
             return;
         }
 
-        SingleMessageLogger.incubatingFeatureUsed("Incremental java compilation");
-
         DefaultJavaCompileSpec spec = createSpec();
-        final CacheRepository cacheRepository = getCacheRepository();
-        final GeneralCompileCaches generalCompileCaches = getGeneralCompileCaches();
-
-        CompileCaches compileCaches = new CompileCaches() {
-            private final CacheRepository repository = cacheRepository;
-            private final JavaCompile javaCompile = JavaCompile.this;
-            private final GeneralCompileCaches generalCaches = generalCompileCaches;
+        CompileCaches compileCaches = createCompileCaches();
+        IncrementalCompilerFactory factory = new IncrementalCompilerFactory(
+            getFileOperations(), getCachingFileHasher(), getPath(), createCompiler(spec), source, compileCaches, (IncrementalTaskInputsInternal) inputs, getEffectiveAnnotationProcessorPath());
+        Compiler<JavaCompileSpec> compiler = factory.createCompiler();
+        performCompilation(spec, compiler);
+    }
 
+    private CompileCaches createCompileCaches() {
+        final GeneralCompileCaches generalCaches = getGeneralCompileCaches();
+        final LocalClassSetAnalysisStore localClassSetAnalysisStore = generalCaches.createLocalClassSetAnalysisStore(getPath());
+        final LocalJarClasspathSnapshotStore localJarClasspathSnapshotStore = generalCaches.createLocalJarClasspathSnapshotStore(getPath());
+        return new CompileCaches() {
             public ClassAnalysisCache getClassAnalysisCache() {
                 return generalCaches.getClassAnalysisCache();
             }
@@ -151,17 +174,13 @@ public class JavaCompile extends AbstractCompile {
             }
 
             public LocalJarClasspathSnapshotStore getLocalJarClasspathSnapshotStore() {
-                return new LocalJarClasspathSnapshotStore(repository, javaCompile);
+                return localJarClasspathSnapshotStore;
             }
 
             public LocalClassSetAnalysisStore getLocalClassSetAnalysisStore() {
-                return new LocalClassSetAnalysisStore(repository, javaCompile);
+                return localClassSetAnalysisStore;
             }
         };
-        IncrementalCompilerFactory factory = new IncrementalCompilerFactory(
-            getFileOperations(), getCachingFileHasher(), getPath(), createCompiler(spec), source, compileCaches, (IncrementalTaskInputsInternal) inputs);
-        Compiler<JavaCompileSpec> compiler = factory.createCompiler();
-        performCompilation(spec, compiler);
     }
 
     @Inject
@@ -169,15 +188,38 @@ public class JavaCompile extends AbstractCompile {
         throw new UnsupportedOperationException();
     }
 
-    @Inject protected FileOperations getFileOperations() {
+    @Inject
+    protected FileOperations getFileOperations() {
         throw new UnsupportedOperationException();
     }
 
-    @Inject protected GeneralCompileCaches getGeneralCompileCaches() {
+    @Inject
+    protected GeneralCompileCaches getGeneralCompileCaches() {
         throw new UnsupportedOperationException();
     }
 
-    @Inject protected CacheRepository getCacheRepository() {
+    @Inject
+    protected CacheRepository getCacheRepository() {
+        throw new UnsupportedOperationException();
+    }
+
+    @Inject
+    protected JavaCompilerFactory getJavaCompilerFactory() {
+        throw new UnsupportedOperationException();
+    }
+
+    @Inject
+    protected Factory<AntBuilder> getAntBuilderFactory() {
+        throw new UnsupportedOperationException();
+    }
+
+    @Inject
+    protected JvmVersionDetector getJvmVersionDetector() {
+        throw new UnsupportedOperationException();
+    }
+
+    @Inject
+    protected ExecActionFactory getExecActionFactory() {
         throw new UnsupportedOperationException();
     }
 
@@ -187,10 +229,6 @@ public class JavaCompile extends AbstractCompile {
         performCompilation(spec, createCompiler(spec));
     }
 
-    @Inject
-    protected Factory<AntBuilder> getAntBuilderFactory() {
-        throw new UnsupportedOperationException();
-    }
 
     private CleaningJavaCompiler createCompiler(JavaCompileSpec spec) {
         Compiler<JavaCompileSpec> javaCompiler = CompilerUtil.castCompiler(((JavaToolChainInternal) getToolChain()).select(getPlatform()).newCompiler(spec.getClass()));
@@ -199,7 +237,7 @@ public class JavaCompile extends AbstractCompile {
 
     @Nested
     protected JavaPlatform getPlatform() {
-        return DefaultJavaPlatform.current();
+        return new DefaultJavaPlatform(JavaVersion.toVersion(getTargetCompatibility()));
     }
 
     private void performCompilation(JavaCompileSpec spec, Compiler<JavaCompileSpec> compiler) {
@@ -214,8 +252,9 @@ public class JavaCompile extends AbstractCompile {
         spec.setDestinationDir(getDestinationDir());
         spec.setWorkingDir(getProject().getProjectDir());
         spec.setTempDir(getTemporaryDir());
-        spec.setClasspath(getClasspath());
-        final File dependencyCacheDir = DeprecationLogger.whileDisabled(new Factory<File>() {
+        spec.setCompileClasspath(ImmutableList.copyOf(getClasspath()));
+        spec.setAnnotationProcessorPath(ImmutableList.copyOf(getEffectiveAnnotationProcessorPath()));
+        File dependencyCacheDir = DeprecationLogger.whileDisabled(new Factory<File>() {
             @Override
             @SuppressWarnings("deprecation")
             public File create() {
@@ -251,4 +290,26 @@ public class JavaCompile extends AbstractCompile {
     public CompileOptions getOptions() {
         return compileOptions;
     }
+
+    @Override
+    @CompileClasspath
+    public FileCollection getClasspath() {
+        return super.getClasspath();
+    }
+
+    /**
+     * Returns the path to use for annotation processor discovery. Returns an empty collection when no processing should be performed, for example when no annotation processors are present in the compile classpath or annotation processing has been disabled.
+     *
+     * <p>You can specify this path using {@link CompileOptions#setAnnotationProcessorPath(FileCollection)} or {@link CompileOptions#setCompilerArgs(java.util.List)}. When not explicitly set using one of the methods on {@link CompileOptions}, the compile classpath will be used when there are annotation processors present in the compile classpath. Otherwise this path will be empty.
+     *
+     * <p>This path is always empty when annotation processing is disabled.</p>
+     *
+     * @since 3.4
+     */
+    @Incubating
+    @Classpath
+    public FileCollection getEffectiveAnnotationProcessorPath() {
+        AnnotationProcessorDetector annotationProcessorDetector = getServices().get(AnnotationProcessorDetector.class);
+        return annotationProcessorDetector.getEffectiveAnnotationProcessorClasspath(compileOptions, getClasspath());
+    }
 }
