@@ -25,6 +25,7 @@ import java.io.File;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.util.List;
+import java.util.Map;
 import java.util.concurrent.TimeUnit;
 
 /**
@@ -55,6 +56,12 @@ public interface ProviderOperationParameters {
     @Nullable
     List<String> getJvmArguments();
 
+    /**
+     * @return When null, use the provider's default environment variables. When empty, use no environment variables.
+     */
+    @Nullable
+    Map<String, String> getEnvironmentVariables();
+
     /**
      * @since 1.0-milestone-3
      */
