@@ -20,24 +20,29 @@ import net.jcip.annotations.ThreadSafe;
 import org.gradle.api.logging.LogLevel;
 import org.gradle.api.logging.StandardOutputListener;
 import org.gradle.api.logging.configuration.ConsoleOutput;
-import org.gradle.internal.TrueTimeProvider;
 import org.gradle.internal.event.ListenerBroadcast;
 import org.gradle.internal.logging.config.LoggingRouter;
 import org.gradle.internal.logging.console.AnsiConsole;
+import org.gradle.internal.logging.console.BuildStatusRenderer;
 import org.gradle.internal.logging.console.ColorMap;
 import org.gradle.internal.logging.console.Console;
-import org.gradle.internal.logging.console.ConsoleBackedProgressRenderer;
+import org.gradle.internal.logging.console.ConsoleLayoutCalculator;
 import org.gradle.internal.logging.console.DefaultColorMap;
-import org.gradle.internal.logging.console.DefaultStatusBarFormatter;
+import org.gradle.internal.logging.console.DefaultWorkInProgressFormatter;
 import org.gradle.internal.logging.console.StyledTextOutputBackedRenderer;
+import org.gradle.internal.logging.console.ThrottlingOutputEventListener;
+import org.gradle.internal.logging.console.WorkInProgressRenderer;
+import org.gradle.internal.logging.events.BatchOutputEventListener;
 import org.gradle.internal.logging.events.EndOutputEvent;
 import org.gradle.internal.logging.events.LogLevelChangeEvent;
+import org.gradle.internal.logging.events.MaxWorkerCountChangeEvent;
 import org.gradle.internal.logging.events.OutputEvent;
 import org.gradle.internal.logging.events.OutputEventListener;
 import org.gradle.internal.logging.text.StreamBackedStandardOutputListener;
 import org.gradle.internal.logging.text.StreamingStyledTextOutput;
 import org.gradle.internal.nativeintegration.console.ConsoleMetaData;
 import org.gradle.internal.nativeintegration.console.FallbackConsoleMetaData;
+import org.gradle.internal.time.TrueTimeProvider;
 
 import java.io.OutputStream;
 import java.io.OutputStreamWriter;
@@ -54,6 +59,7 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
     private final Object lock = new Object();
     private final DefaultColorMap colourMap = new DefaultColorMap();
     private LogLevel logLevel = LogLevel.LIFECYCLE;
+    private int maxWorkerCount;
     private final ConsoleConfigureAction consoleConfigureAction;
     private OutputStream originalStdOut;
     private OutputStream originalStdErr;
@@ -73,7 +79,7 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
     public Snapshot snapshot() {
         synchronized (lock) {
             // Currently only snapshot the console output listener. Should snapshot all output listeners, and cleanup in restore()
-            return new SnapshotImpl(logLevel, console);
+            return new SnapshotImpl(logLevel, console, maxWorkerCount);
         }
     }
 
@@ -84,6 +90,10 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
             if (snapshot.logLevel != logLevel) {
                 configure(snapshot.logLevel);
             }
+
+            if (snapshot.maxWorkerCount != maxWorkerCount) {
+                configureMaxWorkerCount(snapshot.maxWorkerCount);
+            }
             // TODO - also close console when it is replaced
             // TODO - remove console from formatters
             if (snapshot.console != console) {
@@ -118,9 +128,10 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
 
     public void attachAnsiConsole(OutputStream outputStream) {
         synchronized (lock) {
+            ConsoleMetaData consoleMetaData = new FallbackConsoleMetaData();
             OutputStreamWriter writer = new OutputStreamWriter(outputStream);
-            Console console = new AnsiConsole(writer, writer, colourMap, true);
-            addConsole(console, true, true, new FallbackConsoleMetaData());
+            Console console = new AnsiConsole(writer, writer, colourMap, consoleMetaData, true);
+            addConsole(console, true, true, consoleMetaData);
         }
     }
 
@@ -182,11 +193,13 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
     }
 
     public OutputEventRenderer addConsole(Console console, boolean stdout, boolean stderr, ConsoleMetaData consoleMetaData) {
-        final OutputEventListener consoleChain = new ConsoleBackedProgressRenderer(
-            new ProgressLogEventGenerator(
-                new StyledTextOutputBackedRenderer(console.getMainArea()), true),
-            console,
-            new DefaultStatusBarFormatter(consoleMetaData),
+        final OutputEventListener consoleChain = new ThrottlingOutputEventListener(
+             new BuildStatusRenderer(
+                new WorkInProgressRenderer(
+                    new ProgressLogEventGenerator(
+                        new StyledTextOutputBackedRenderer(console.getBuildOutputArea()), true),
+                    console.getBuildProgressArea(), new DefaultWorkInProgressFormatter(consoleMetaData), new ConsoleLayoutCalculator(consoleMetaData)),
+                console.getStatusBar(), console, consoleMetaData),
             new TrueTimeProvider());
         synchronized (lock) {
             if (stdout && stderr) {
@@ -201,13 +214,14 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
                 removeStandardErrorListener();
             }
             consoleChain.onOutput(new LogLevelChangeEvent(logLevel));
+            consoleChain.onOutput(new MaxWorkerCountChangeEvent(maxWorkerCount));
             formatters.add(this.console);
         }
         return this;
     }
 
     private OutputEventListener onError(final OutputEventListener listener) {
-        return new OutputEventListener() {
+        return new BatchOutputEventListener() {
             public void onOutput(OutputEvent event) {
                 if (event.getLogLevel() == LogLevel.ERROR || event.getLogLevel() == null) {
                     listener.onOutput(event);
@@ -217,7 +231,7 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
     }
 
     private OutputEventListener onNonError(final OutputEventListener listener) {
-        return new OutputEventListener() {
+        return new BatchOutputEventListener() {
             public void onOutput(OutputEvent event) {
                 if (event.getLogLevel() != LogLevel.ERROR || event.getLogLevel() == null) {
                     listener.onOutput(event);
@@ -263,6 +277,12 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
         onOutput(new LogLevelChangeEvent(logLevel));
     }
 
+    @Override
+    public void configureMaxWorkerCount(int maxWorkerCount) {
+        onOutput(new MaxWorkerCountChangeEvent(maxWorkerCount));
+    }
+
+    @Override
     public void onOutput(OutputEvent event) {
         synchronized (lock) {
             if (event.getLogLevel() != null && event.getLogLevel().compareTo(logLevel) < 0) {
@@ -275,6 +295,13 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
                     return;
                 }
                 this.logLevel = newLogLevel;
+            } else if (event instanceof MaxWorkerCountChangeEvent) {
+                MaxWorkerCountChangeEvent changeEvent = (MaxWorkerCountChangeEvent) event;
+                int newMaxWorkerCount = changeEvent.getNewMaxWorkerCount();
+                if (newMaxWorkerCount == this.maxWorkerCount) {
+                    return;
+                }
+                this.maxWorkerCount = newMaxWorkerCount;
             }
             formatters.getSource().onOutput(event);
         }
@@ -283,10 +310,12 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
     private class SnapshotImpl implements Snapshot {
         private final LogLevel logLevel;
         private final OutputEventListener console;
+        private final int maxWorkerCount;
 
-        SnapshotImpl(LogLevel logLevel, OutputEventListener console) {
+        SnapshotImpl(LogLevel logLevel, OutputEventListener console, int maxWorkerCount) {
             this.logLevel = logLevel;
             this.console = console;
+            this.maxWorkerCount = maxWorkerCount;
         }
     }
 }
