@@ -22,11 +22,10 @@ import org.gradle.api.Incubating;
 import org.gradle.api.Transformer;
 import org.gradle.api.UncheckedIOException;
 import org.gradle.api.file.FileCollection;
-import org.gradle.api.internal.hash.DefaultFileHasher;
-import org.gradle.api.internal.initialization.loadercache.HashClassPathSnapshotter;
 import org.gradle.api.tasks.Classpath;
 import org.gradle.api.tasks.OutputDirectory;
 import org.gradle.api.tasks.TaskAction;
+import org.gradle.internal.classloader.ClasspathHasher;
 import org.gradle.internal.classpath.DefaultClassPath;
 import org.gradle.util.CollectionUtils;
 import org.gradle.util.GUtil;
@@ -94,8 +93,8 @@ public class PluginUnderTestMetadata extends DefaultTask {
 
             // As these files are inputs into this task, they have just been snapshotted by the task up-to-date checking.
             // We should be reusing those persistent snapshots to avoid reading into memory again.
-            HashClassPathSnapshotter classPathSnapshotter = new HashClassPathSnapshotter(new DefaultFileHasher());
-            String hash = classPathSnapshotter.snapshot(new DefaultClassPath(getPluginClasspath())).getStrongHash().toString();
+            ClasspathHasher classpathHasher = getServices().get(ClasspathHasher.class);
+            String hash = classpathHasher.hash(new DefaultClassPath(getPluginClasspath())).toString();
             properties.setProperty(IMPLEMENTATION_CLASSPATH_HASH_PROP_KEY, hash);
         }
 
