@@ -24,7 +24,6 @@ import org.gradle.api.GradleException;
 import org.gradle.api.Nullable;
 import org.gradle.api.Transformer;
 import org.gradle.api.artifacts.dsl.RepositoryHandler;
-import org.gradle.api.artifacts.repositories.ArtifactRepository;
 import org.gradle.api.artifacts.repositories.MavenArtifactRepository;
 import org.gradle.api.internal.initialization.ClassLoaderScope;
 import org.gradle.api.internal.initialization.ScriptHandlerInternal;
@@ -39,17 +38,19 @@ import org.gradle.api.specs.Spec;
 import org.gradle.internal.classpath.CachedClasspathTransformer;
 import org.gradle.internal.classpath.ClassPath;
 import org.gradle.internal.exceptions.LocationAwareException;
-import org.gradle.plugin.repository.internal.PluginRepositoryRegistry;
-import org.gradle.plugin.internal.PluginId;
+import org.gradle.plugin.management.internal.PluginRequestInternal;
+import org.gradle.plugin.management.internal.PluginResolutionStrategyInternal;
+import org.gradle.plugin.management.internal.PluginRequests;
 import org.gradle.plugin.repository.PluginRepository;
-import org.gradle.plugin.repository.internal.BackedByArtifactRepository;
+import org.gradle.plugin.repository.internal.BackedByArtifactRepositories;
+import org.gradle.plugin.repository.internal.PluginRepositoryRegistry;
+import org.gradle.plugin.use.PluginId;
 import org.gradle.plugin.use.resolve.internal.NotNonCorePluginOnClasspathCheckPluginResolver;
 import org.gradle.plugin.use.resolve.internal.PluginResolution;
 import org.gradle.plugin.use.resolve.internal.PluginResolutionResult;
 import org.gradle.plugin.use.resolve.internal.PluginResolveContext;
 import org.gradle.plugin.use.resolve.internal.PluginResolver;
 
-import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Formatter;
 import java.util.LinkedList;
@@ -63,13 +64,15 @@ import static org.gradle.util.CollectionUtils.collect;
 public class DefaultPluginRequestApplicator implements PluginRequestApplicator {
     private final PluginRegistry pluginRegistry;
     private final PluginResolverFactory pluginResolverFactory;
-    private PluginRepositoryRegistry pluginRepositoryRegistry;
+    private final PluginRepositoryRegistry pluginRepositoryRegistry;
+    private final PluginResolutionStrategyInternal pluginResolutionStrategy;
     private final CachedClasspathTransformer cachedClasspathTransformer;
 
-    public DefaultPluginRequestApplicator(PluginRegistry pluginRegistry, PluginResolverFactory pluginResolver, PluginRepositoryRegistry pluginRepositoryRegistry, CachedClasspathTransformer cachedClasspathTransformer) {
+    public DefaultPluginRequestApplicator(PluginRegistry pluginRegistry, PluginResolverFactory pluginResolver, PluginRepositoryRegistry pluginRepositoryRegistry, PluginResolutionStrategyInternal pluginResolutionStrategy, CachedClasspathTransformer cachedClasspathTransformer) {
         this.pluginRegistry = pluginRegistry;
         this.pluginResolverFactory = pluginResolver;
         this.pluginRepositoryRegistry = pluginRepositoryRegistry;
+        this.pluginResolutionStrategy = pluginResolutionStrategy;
         this.cachedClasspathTransformer = cachedClasspathTransformer;
     }
 
@@ -85,9 +88,10 @@ public class DefaultPluginRequestApplicator implements PluginRequestApplicator {
 
         final PluginResolver effectivePluginResolver = wrapInNotInClasspathCheck(classLoaderScope);
 
-        List<Result> results = collect(requests, new Transformer<Result, PluginRequest>() {
-            public Result transform(PluginRequest request) {
-                return resolveToFoundResult(effectivePluginResolver, request);
+        List<Result> results = collect(requests, new Transformer<Result, PluginRequestInternal>() {
+            public Result transform(PluginRequestInternal request) {
+                PluginRequestInternal configuredRequest = pluginResolutionStrategy.applyTo(request);
+                return resolveToFoundResult(effectivePluginResolver, configuredRequest);
             }
         });
 
@@ -99,18 +103,12 @@ public class DefaultPluginRequestApplicator implements PluginRequestApplicator {
         if (!results.isEmpty()) {
             final RepositoryHandler repositories = scriptHandler.getRepositories();
 
-            List<ArtifactRepository> pluginArtifactRepositories = new ArrayList<ArtifactRepository>();
-            pluginRepositoryRegistry.lock();
             for (PluginRepository pluginRepository : pluginRepositoryRegistry.getPluginRepositories()) {
-                if (pluginRepository instanceof BackedByArtifactRepository) {
-                    pluginArtifactRepositories.add(((BackedByArtifactRepository) pluginRepository).createArtifactRepository(repositories));
+                if (pluginRepository instanceof BackedByArtifactRepositories) {
+                    ((BackedByArtifactRepositories) pluginRepository).createArtifactRepositories(repositories);
                 }
             }
 
-            // The plugin repositories were appended as they were added, but we want them at the front.
-            repositories.removeAll(pluginArtifactRepositories);
-            repositories.addAll(0, pluginArtifactRepositories);
-
             final List<MavenArtifactRepository> mavenRepos = repositories.withType(MavenArtifactRepository.class);
             final Set<String> repoUrls = Sets.newLinkedHashSet();
 
@@ -166,7 +164,7 @@ public class DefaultPluginRequestApplicator implements PluginRequestApplicator {
         // Because we are only build.gradle files right now, this holds.
         // It won't for arbitrary scripts though.
         for (final Map.Entry<Result, PluginId> entry : legacyActualPluginIds.entrySet()) {
-            final PluginRequest request = entry.getKey().request;
+            final PluginRequestInternal request = entry.getKey().request;
             final PluginId id = entry.getValue();
             applyPlugin(request, id, new Runnable() {
                 public void run() {
@@ -182,7 +180,7 @@ public class DefaultPluginRequestApplicator implements PluginRequestApplicator {
             applyPlugin(result.request, result.found.getPluginId(), new Runnable() {
                 public void run() {
                     if (result.request.isApply()) {
-                      target.apply(entry.getValue());
+                        target.apply(entry.getValue());
                     }
                 }
             });
@@ -206,7 +204,7 @@ public class DefaultPluginRequestApplicator implements PluginRequestApplicator {
         return new NotNonCorePluginOnClasspathCheckPluginResolver(pluginResolverFactory.create(), pluginRegistry, scriptClasspathPluginDescriptorLocator);
     }
 
-    private void applyPlugin(PluginRequest request, PluginId id, Runnable applicator) {
+    private void applyPlugin(PluginRequestInternal request, PluginId id, Runnable applicator) {
         try {
             try {
                 applicator.run();
@@ -228,7 +226,7 @@ public class DefaultPluginRequestApplicator implements PluginRequestApplicator {
         }
     }
 
-    private Result resolveToFoundResult(PluginResolver effectivePluginResolver, PluginRequest request) {
+    private Result resolveToFoundResult(PluginResolver effectivePluginResolver, PluginRequestInternal request) {
         Result result = new Result(request);
         try {
             effectivePluginResolver.resolve(request, result);
@@ -247,7 +245,7 @@ public class DefaultPluginRequestApplicator implements PluginRequestApplicator {
         return result;
     }
 
-    private String buildNotFoundMessage(PluginRequest pluginRequest, Result result) {
+    private String buildNotFoundMessage(PluginRequestInternal pluginRequest, Result result) {
         if (result.notFoundList.isEmpty()) {
             // this shouldn't happen, resolvers should call notFound()
             return String.format("Plugin %s was not found", pluginRequest.getDisplayName());
@@ -278,10 +276,10 @@ public class DefaultPluginRequestApplicator implements PluginRequestApplicator {
 
     private static class Result implements PluginResolutionResult {
         private final List<NotFound> notFoundList = new LinkedList<NotFound>();
-        private final PluginRequest request;
+        private final PluginRequestInternal request;
         private PluginResolution found;
 
-        public Result(PluginRequest request) {
+        public Result(PluginRequestInternal request) {
             this.request = request;
         }
 
