@@ -17,7 +17,7 @@
 package org.gradle.plugin.use.resolve.service.internal;
 
 import org.gradle.api.Action;
-import org.gradle.plugin.use.internal.PluginRequest;
+import org.gradle.plugin.management.internal.PluginRequestInternal;
 import org.gradle.util.DeprecationLogger;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -44,7 +44,7 @@ public class DeprecationListeningPluginResolutionServiceClient implements Plugin
         this.action = action;
     }
 
-    public Response<PluginUseMetaData> queryPluginMetadata(String portalUrl, boolean shouldValidate, PluginRequest pluginRequest) {
+    public Response<PluginUseMetaData> queryPluginMetadata(String portalUrl, boolean shouldValidate, PluginRequestInternal pluginRequest) {
         Response<PluginUseMetaData> response = delegate.queryPluginMetadata(portalUrl, shouldValidate, pluginRequest);
         String statusChecksum = response.getClientStatusChecksum();
         if (statusChecksum != null) {
