@@ -16,7 +16,6 @@
 package org.gradle.plugins.signing;
 
 import groovy.lang.Closure;
-import groovy.lang.GroovyObjectSupport;
 import org.gradle.api.Action;
 import org.gradle.api.InvalidUserDataException;
 import org.gradle.api.Project;
@@ -48,7 +47,7 @@ import static org.gradle.util.GUtil.uncheckedCall;
 /**
  * The global signing configuration for a project.
  */
-public class SigningExtension extends GroovyObjectSupport {
+public class SigningExtension {
 
     /**
      * The name of the configuration that all signature artifacts will be placed into ("signatures")
@@ -375,10 +374,23 @@ public class SigningExtension extends GroovyObjectSupport {
     /**
      * Signs the POM artifact for the given Maven deployment.
      *
-     * <p>You can use this method to sign the generated POM when publishing to a Maven repository with the Maven plugin. </p> <pre autoTested=''> uploadArchives { repositories { mavenDeployer {
-     * beforeDeployment { MavenDeployment deployment -> signing.signPom(deployment) } } } } </pre> <p>You can optionally provide a configuration closure to fine tune the {@link SignOperation sign
+     * <p>You can use this method to sign the generated POM when publishing to a Maven repository with the Maven plugin. </p>
+     * <pre autoTested=''>
+     * uploadArchives {
+     *   repositories {
+     *     mavenDeployer {
+     *       beforeDeployment { MavenDeployment deployment ->
+     *         signing.signPom(deployment)
+     *       }
+     *     }
+     *   }
+     * }
+     * </pre>
+     * <p>You can optionally provide a configuration closure to fine tune the {@link SignOperation sign
      * operation} for the POM.</p> <p> If {@link #isRequired()} is false and the signature cannot be generated (e.g. no configured signatory), this method will silently do nothing. That is, a
      * signature for the POM file will not be uploaded.
+     * <p>
+     * <b>Note:</b> Signing the generated POM file generated by the Maven Publishing plugin is currently not supported. Future versions of Gradle might add this functionality.
      *
      * @param mavenDeployment The deployment to sign the POM of
      * @param closure the configuration of the underlying {@link SignOperation sign operation} for the POM (optional)
@@ -410,10 +422,23 @@ public class SigningExtension extends GroovyObjectSupport {
     /**
      * Signs the POM artifact for the given Maven deployment.
      *
-     * <p>You can use this method to sign the generated POM when publishing to a Maven repository with the Maven plugin. </p> <pre autoTested=''> uploadArchives { repositories { mavenDeployer {
-     * beforeDeployment { MavenDeployment deployment -> signing.signPom(deployment) } } } } </pre> <p>You can optionally provide a configuration closure to fine tune the {@link SignOperation sign
+     * <p>You can use this method to sign the generated POM when publishing to a Maven repository with the Maven plugin. </p>
+     * <pre autoTested=''>
+     * uploadArchives {
+     *   repositories {
+     *     mavenDeployer {
+     *       beforeDeployment { MavenDeployment deployment ->
+     *         signing.signPom(deployment)
+     *       }
+     *     }
+     *   }
+     * }
+     * </pre>
+     * <p>You can optionally provide a configuration closure to fine tune the {@link SignOperation sign
      * operation} for the POM.</p> <p> If {@link #isRequired()} is false and the signature cannot be generated (e.g. no configured signatory), this method will silently do nothing. That is, a
      * signature for the POM file will not be uploaded.
+     * <p>
+     * <b>Note:</b> Signing the generated POM file generated by the Maven Publishing plugin is currently not supported. Future versions of Gradle might add this functionality.
      *
      * @param mavenDeployment The deployment to sign the POM of
      * @return the generated signature artifact
@@ -452,9 +477,4 @@ public class SigningExtension extends GroovyObjectSupport {
             ? uncheckedCall((Callable) maybeCallable)
             : maybeCallable;
     }
-
-    public static String getDEFAULT_CONFIGURATION_NAME() {
-        return DEFAULT_CONFIGURATION_NAME;
-    }
-
 }
