@@ -17,6 +17,8 @@
 package org.gradle.tooling.internal.provider.runner;
 
 import org.gradle.internal.invocation.BuildActionRunner;
+import org.gradle.internal.progress.BuildOperationExecutor;
+import org.gradle.internal.progress.BuildOperationService;
 import org.gradle.internal.service.ServiceRegistration;
 import org.gradle.internal.service.scopes.PluginServiceRegistry;
 import org.gradle.launcher.exec.ChainingBuildActionRunner;
@@ -28,14 +30,17 @@ public class ToolingBuilderServices implements PluginServiceRegistry {
     public void registerGlobalServices(ServiceRegistration registration) {
 
         registration.addProvider(new Object() {
-            BuildActionRunner createBuildActionRunner() {
-                return new SubscribableBuildActionRunner(new ChainingBuildActionRunner(Arrays.asList(
-                                                                new BuildModelActionRunner(),
-                                                                new TestExecutionRequestActionRunner(),
-                                                                new ClientProvidedBuildActionRunner())));
+            BuildActionRunner createBuildActionRunner(BuildOperationExecutor buildOperationExecutor, BuildOperationService buildOperationService) {
+                return new SubscribableBuildActionRunner(
+                    new RunAsBuildOperationBuildActionRunner(
+                        new ChainingBuildActionRunner(
+                            Arrays.asList(
+                                new BuildModelActionRunner(),
+                                new TestExecutionRequestActionRunner(buildOperationService),
+                                new ClientProvidedBuildActionRunner())),
+                        buildOperationExecutor), buildOperationService);
             }
         });
-
     }
 
     public void registerBuildSessionServices(ServiceRegistration registration) {
